function [rho,Fex,msdMean,msdIdeal,lambda,msdIdeal2,lambda2,msdEin] = calcExcessFreeEnergy(dim,numCells,lambdaMin,~,phiString,cellColor,basedir,isPer,varargin)
%This function determines the excess free energy in an einstein crystal
%after the simulations have been done. It takes dimension 'dim', the cell
%characterization 'numCells', the lower limit of lambda integration,
%'lambdaMin', a string containing the packing fraction 'phiString', the
%color used to plot at this system size 'cellColor', the directory where
%the data is located 'basedir', a boolean stating if the periodic crystal
%is used instead of the einstein crystal 'isPer', and 

%Comment next line if you would like both Z and D embedded data in d < 6
%plotted

plotOnlyD = 0;

% Get list of lambdas
dirList = dir([basedir,filesep,num2str(dim),filesep,num2str(numCells),filesep,phiString,filesep,'*e*']);
lambda = zeros(1,numel(dirList));
lambdaList = cell(1,numel(dirList));
for i = 1:numel(lambda)
    lambda(i) = str2double(dirList(i).name);
    lambdaList{i} = dirList(i).name;
end

symbol = '.';
symSize = 15;
[lambda,order] = sort(lambda);
lambdaList = lambdaList(order);

%Conver numCells to number of particles
if dim >= 6
    nParticles = numCells;
else
    if numCells < 100
        nParticles = (numCells^dim)/2;
    else
        nParticles = (numCells - 99)^dim;
    end
end

%When nParticles = 1/2 from formula above, set it to 1
nParticles(nParticles==1/2) = 1;
 
msdMean = NaN(numel(lambda),1);

%Radii are the same for all einstein crystal calculations. Choose one for
%which the radii file exists. (Here, 1e2)
radii = load([basedir,filesep,num2str(dim),filesep,num2str(numCells),filesep,phiString,filesep,'1e2',filesep,'radii.dat']);

for ii = 1:numel(lambda)
    try
        msdMean(ii,1) = load([basedir,filesep,num2str(dim),filesep,num2str(numCells),filesep,phiString,filesep,lambdaList{ii},filesep,'msdPlateau.dat']);
    catch
    end
end

lambda = lambda(~isnan(msdMean));
msdMean = msdMean(~isnan(msdMean));

%boxVolume is 2 if we use D boundary conditions and 1 if we use Z or E
if ((numCells>100 && dim < 6) || dim == 9 || (dim == 8 && nParticles > 10000 && nParticles < 20000) || (dim == 10 && nParticles > 50000))
    boxVolume = 2;
else
    boxVolume = 1;
end

%Calculate density directly by taking N/V
rho = nParticles * (2*radii)^dim/boxVolume;

[lambdaMax,msdIdeal,lambda,msdIdeal2,lambda2,msdEin] = findLambdaMax(msdMean,lambda',nParticles,dim,rho,isPer);


% Set lambdaMax manually for lambda>=8
if (dim == 3 && isPer)
    lambdaMax = 1e6;
end
if (dim == 8)
    lambdaMax = 3000;
elseif (dim == 9)
    lambdaMax = 1e6;
elseif (dim == 10)
    lambdaMax = 1e8;
end

disp(['lambdaMax is ',num2str(lambdaMax)])
Fex = [];
if sum(~isnan(msdMean)) > 4
    if ((dim >= 6 || (plotOnlyD && numCells > 100 && dim < 6) || (~plotOnlyD)))
        plot(lambda',lambda'.*msdMean * 2/dim,[symbol,'-'],'MarkerSize',symSize,'Color',cellColor,'MarkerFaceColor',cellColor)
    end
    range = ~isnan(msdMean);
    if numel(range) > 2
        f = fit(log(lambda(range)'),msdMean(range).*lambda(range)','cubicinterp');
        [gaussianPoints,gaussianWeights] = lgwt(130,log(lambdaMin),log(lambdaMax));
        FInt = sum(feval(f,gaussianPoints).*gaussianWeights);
        disp(['dim = ',num2str(dim),', nParticles = ',num2str(nParticles),' :'])
 
        if isPer == 1
           Fex = -dim/2*log(pi/lambdaMax) - FInt - dim/(2*nParticles)*log(lambdaMax/pi) - log(rho) + 1 - log(2*pi)/(2*nParticles) ;
        else
           Fex = -dim/2*log(pi/lambdaMax) - dim/(2*nParticles)*log(lambdaMax/pi) - FInt + log(rho)/nParticles - (dim+1)/2 * log(nParticles)/nParticles - log(rho) + 1 - log(2*pi)/(2*nParticles);
        end
        
        disp(['FInt is ', num2str(FInt)])
        disp(['FEx is ', num2str(Fex)]) 
    end
end

end