function [Finf,Ferr,a,aErr] = einsteinCrystalHX()

% This function plots Fig. 9

% This function performs the einstein integration by pulling the einstein
% crystal simulations, calling an integration function, and plotting Free 
% energy as a function of system size.

% Set to 0 if you would like to have both Z and D embedded data in d < 6
% plotted

plotOnlyD = 1;

% Set relative base directory
currentFile = mfilename( 'fullpath' );
[pathstr,~,~] = fileparts( currentFile );
base = [pathstr,filesep,'..',filesep,'data',filesep,'figure3',filesep];

lambdaMin = 1e-6;
isPer = 0;
lambdaCutoff = 0.99;
above = 1;
fitStart = 1;

Finf = NaN(10,1);
Ferr = NaN(10,1);
a = NaN(10,1);
aErr = NaN(10,1);

% Manually correct the offset
FInfAdj = [0; 0; 14.140461943636394; 19.981257160677430; 26.041408973337994; 32.368; 0; 45.5639; 0; 0];

% 45.5623 when 4

%linewidth
lw = 1.5;
%marker size dot
ms = 20;
%marker size large
msl = 10;

rho0 = NaN(1,10);

figure(103); clf; hold all

numCellSet = {1,4:2:18,4:2:10,4:2:8,[1536, 2048, 2304, 3456, 5184 23328],[17496],[65536],[39366],[]};
numCellSetD = {1,102:122,102:110,102:106,[],[2048],13122,39366,81920};
numCellSetE = {[],[],[],[],[],[],[6561],[],[]};
phiStringList = {'','0.7256708798991999','0.6045132695667231','0.45595246104307346','0.3654885946704236','0.28939191568279826','0.248596117743027','0.14285937829200768','0.09762346715155462'};

dimColor = [NaN NaN NaN; NaN NaN NaN; 1 0 0; 0.5 0 0.5; 0 0 1; 0 0 0; 13/127 55/127 13/127; 0.5 0.5 0.5; 0.3 0.3 0.3];

dimList = 3:10;
for dim = dimList
    figure(10*dim)
    clf; hold all
    figure(100*dim)
    clf; hold all
    NAll = [];
    FexListAll = [];
    NFit = [];
    FexListFit = [];
    % Here dLatticeGroup gives the set of boundary conditions considered.
    % E=2, D=1, Z=0
    if dim <= 7
        dLatticeGroup = 0;
    elseif dim == 8
        dLatticeGroup = 2:-1:0;
    else
        dLatticeGroup = 1;
    end
    for dLattice = dLatticeGroup
        if dLattice==1
            basedir = [base,filesep,'DBoundaries'];
            if dim == 9
                isPer = 1;
            end
            numCellsList = numCellSetD{dim-1};
        elseif dLattice == 0
            basedir = [base,filesep,'ZBoundaries'];
            numCellsList = numCellSet{dim-1};
        else
            basedir = [base,filesep,'EBoundaries'];
            numCellsList = numCellSetE{dim-1};
        end
        if dim <= 8
            basedir = '/Users/pkmorse/Documents/Data/Duke/einsteinCrystalHighX';
        else
            basedir = '/Users/pkmorse/Documents/Data/Duke/einsteinCrystalHigh';
        end
        phiString = phiStringList{dim-1};
        cellColor = zeros(numel(numCellsList),3);
        cellColor(:,1) = linspace(0,1,numel(numCellsList));
        cellColor(:,3) = linspace(1,0,numel(numCellsList));
        if dim==8
            if dLattice==1
                cellColor = [1 0 0];
            else
                cellColor = [0 0 1];
            end
        end
        cellLength = numel(numCellsList);
        FexList = NaN(cellLength,1);
        for j = 1:cellLength
            try
                figure(dim*10)
                [rho, FexList(j), msdMean,msdIdeal,lambda,msdIdeal2,lambda2,msdEin] = calcExcessFreeEnergy(dim,numCellsList(j),lambdaMin,1000000,phiString,cellColor(j,:),basedir,isPer,above,lambdaCutoff);
                figure(dim*100)
                FexList(j) = FexList(j);
                if ((dim >= 6 || (plotOnlyD && dLattice==1 && dim < 6) || (~plotOnlyD)))
                    plot(lambda',msdMean,'.','Linewidth',lw,'MarkerSize',2*ms,'Color',cellColor(j,:))
                    plot(lambda2',msdIdeal2,'-','Linewidth',2*lw,'MarkerSize',ms,'Color',cellColor(j,:))
                end
            catch
            end
            disp(FexList(j))
        end
        disp(rho)
        disp(FexList)
        rho0(dim) = rho;
        
        % Convert numCells to N
        if dim <= 5
            if numCellsList(1) > 100
                N = (numCellsList - 99).^dim;
            else
                N = (numCellsList.^dim)/2;
            end
        else
            N = numCellsList;
        end
        N(N==1/2)=1;
        flip = numel(NAll);
        N = N(~isnan(FexList));
        FexList = FexList(~isnan(FexList));
        NAll = [NAll N];
        if numel(N)==1
            NFit = [NFit N];
        else
            NFit = [NFit N(fitStart:end)];
        end
        if dim==6
            flip = numel(NAll);
        end
        figure(dim*10)
        set(gca,'xscale','log');
        xlabel('$\lambda$','interpreter','latex')
        ylabel('$\frac{2}{d}\lambda \langle U\rangle_\lambda$','interpreter','latex','fontname','helvetica')
        plotSpecs()
        set(gca,'Linewidth',1.5)
        set(gca,'FontSize',24)
        set(gca,'xminortick','off')
        xticks([1e-4 1e-2 1e0 1e2 1e4 1e6 1e8 1e10])
        
        xlim([1e-4 1e6])
        ylim([0 1])
        addTextToFigure(dim*10,0.85,0.1,['d=',num2str(dim)],'fontsize',28,'fontname','helvetica')
        
        figure(103)
        x = logspace(log10(min(1./NAll)),log10(max(1./NAll)),100);
        
        FexList = FexList + (dim-1)/2 * log(N')./N';
        FexListAll = [FexListAll; FexList];
        if numel(N) == 1
            FexListFit = [FexListFit; FexList];
        else
            FexListFit = [FexListFit; FexList(fitStart:end)];
        end
        FexList = FexList(~isnan(FexList));
        if dLattice==dLatticeGroup(end) && numel(NAll) >= 2
            [res,err] = fit(1./NFit',FexListFit,'poly1');
%             [res,err] = fit(1./NAll',FexListAll,'poly1');

            if numel(NFit)>2
                con = confint(res);
                aF(dim) = res.p1;
                aFErr(dim) = con(2,1) - res.p1;             
            end
            
            [res2,err] = fit(1./NFit',FInfAdj(dim)-FexListFit,'poly1');
            
            if dim == 8
                h = plot(x,2^(dim-3)*(-res.p1*x),'-','Linewidth',lw,'Color',dimColor(dim,:));
                h1 = plot(1./NAll(2),2^(dim-3)*(FInfAdj(dim) - FexListAll(2)),'d','MarkerSize',msl,'Color',dimColor(dim,:),'MarkerFaceColor',dimColor(dim,:));
                h2 = plot(1./NAll(1),2^(dim-3)*(FInfAdj(dim) - FexListAll(1)),'o','MarkerSize',1.25*msl,'Color',dimColor(dim,:),'MarkerFaceColor','w','Linewidth',2.5);
                h3 = plot(1./NAll(3),2^(dim-3)*(FInfAdj(dim) - FexListAll(3)),'s','MarkerSize',1.25*msl,'Color',dimColor(dim,:),'MarkerFaceColor','w','Linewidth',2.5);
                set(get(get(h2,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
            else
                h = plot(x,2^(dim-3)*(-res.p1*x),'-','Linewidth',lw,'Color',dimColor(dim,:));
                h1 = plot(1./NAll((flip+1):end),2^(dim-3)*(FInfAdj(dim) - FexListAll((flip+1):end)),'d','MarkerSize',msl,'Color',dimColor(dim,:),'MarkerFaceColor',dimColor(dim,:));
                h2 = plot(1./NAll(1:flip),2^(dim-3)*(FInfAdj(dim) - FexListAll(1:flip)),'s','MarkerSize',1.25*msl,'Color',dimColor(dim,:),'MarkerFaceColor','w','Linewidth',2.5);
            end
            set(get(get(h1,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
            set(get(get(h2,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
            Finf(dim) = res.p2;
            
            if numel(NFit) > 2
                con = confint(res);
                Ferr(dim) = con(2,2) - res.p2;
            end
        else
        end
        
        xlabel('$1/N$','interpreter','latex')
        ylabel('$2^{d-3}(\beta f_\mathrm{ex} - \beta F_\mathrm{ex}/N)$','interpreter','latex')
        set(gca,'Linewidth',1.5)
        set(gca,'FontSize',20)
        set(gca,'xminortick','off')
        
        figure(dim*10)
        legendEntry = {};
        for i = 1:numel(N)
            legendEntry{i} = ['N=',num2str(N(i))];
        end
        legend(legendEntry{:},'numcolumns',2,'fontsize',16,'location','northwest')
        if dim == 8
            legend('N=6561','N=13122','N=65536','numcolumns',2,'fontsize',16,'location','northwest')
        end
        figure(dim*100)
        plot(lambda2',msdEin,'--k','Linewidth',2*lw,'MarkerSize',ms)
        set(gca,'xscale','log');
        axis([100 2000 0 0.006])
        xlabel('\lambda')
        ylabel('$\langle r^2\rangle_\lambda$','interpreter','latex','fontname','helvetica')
        plotSpecs()
        set(gca,'Linewidth',3)
        set(gca,'FontSize',36) 
    end
    if (dim == 8)
        [res,err] = fit(log10(1./NAll'),log10(FInfAdj(dim)-FexListAll),'x + a','startpoint',1);
        con = confint(res);
        aF(dim) = -10^res.a;
        aFErr(dim) = -aF(dim) - 10^con(1);
    end
end

figure(103)
xlabel('$1/N$','interpreter','latex')
ylabel('$2^{d-3}(\beta f_\mathrm{ex} - \beta F_\mathrm{ex}/N)$','interpreter','latex')
plotSpecs()
set(gca,'Linewidth',1.5)
set(gca,'FontSize',20)
set(gca,'xscale','log')
set(gca,'yscale','log')
set(gca,'xminortick','off')
set(gca,'yminortick','off')
xlim([1e-5 1e-1])
ylim([1e-3 1e0])

legend('$d=3$','$d=4$','$d=5$','$d=6$','$d=8$','numcolumns',3,'interpreter','latex','location','northwest','fontsize',14)
xticks([1e-5 1e-4 1e-3 1e-2 1e-1])
yticks([1e-4 1e-3 1e-2 1e-1 1e0])

%Plot finite size coefficient
figure(1)
clf; hold all
errorbar([3:6 8]',-aF([3:6 8]),aFErr([3:6 8]),'.k','markersize',15,'linewidth',2)
[res,err] = fit([3:6 8]',-aF([3:6 8])','a + b*x^2','weights',1./sqrt(aFErr([3:6 8])'),'startPoint',[1 1]);
aPred = feval(res,[7 9 10]);
conPredSet = predint(res,[7 9 10]);
conPred = conPredSet(:,2) - aPred;
errorbar([7 9 10]',aPred,conPred,'.b','markersize',15,'linewidth',2)
x = linspace(3,10,100);
plot(x,feval(res,x),'--r','linewidth',2)

xlim([2 11])
ylim([0 100])
plotSpecs()
xlabel('$d$','interpreter','latex')
ylabel('$\Xi$','interpreter','latex')

disp(res)

end
