function [] = parallelTangentIllustration()

% Plots Fig. 1. Values chosen here are arbitrary and used
% solely for illustration.

x = linspace(0,8,10000);

xoff1 = 0.5;
xoff2 = 1.5;

yoff1 = 0.5;
yoff2 = 0.62;

xstretch1 = 0.5;
xstretch2 = 0.7;

ystretch1 = 0.7;
ystretch2 = 1.2;

xklo = 2;
xkhi = 2.8;

xs = 3.9;

figure(3)
clf; hold all
plot(x(x>xoff1&x<xs),ystretch1*log((x(x>xoff1&x<xs)-xoff1)/xstretch1)+yoff1,'-b','linewidth',2)
plot(x(x>xoff2&x<xkhi),ystretch2*log((x(x>xoff2&x<xkhi)-xoff2)/xstretch2)+yoff2,'-','Color',[0.5 0 0.5],'linewidth',2)
plot(x(x>xoff2&x>xkhi),ystretch2*log((x(x>xoff2&x>xkhi)-xoff2)/xstretch2)+yoff2,'-r','linewidth',2)

plot([xoff1 xoff1],[-10 10],'--k','linewidth',2)
plot([xoff2 xoff2],[-10 10],'--k','linewidth',2)

xlabel('$\overline{v}$','interpreter','latex')
ylabel('$s$','interpreter','latex')
axis([min(x) 5 0 3])
plotSpecs()
set(gca,'Fontsize',24)


fun = @parallelTangentIllustrationFunction;
x0 = [2 2 5 2.5];
options = optimoptions('fsolve','Display','iter');
x = fsolve(fun,x0,options);
plot([x(1) x(3)],[x(2) x(4)],':k','linewidth',2)
disp(x)

xticks([xoff1 xoff2])
xticklabels({'$\overline{v}_{cp}$','$\overline{v}_{gcp}$'})
set(gca,'TickLabelInterpreter','latex');
yticks([])
addTextToFigure(3,0.55,0.39,'$\overline{v}_k$','fontSize',24,'interpreter','latex')
addTextToFigure(3,0.82,0.82,'$\overline{v}_f$','fontSize',24,'interpreter','latex')
addTextToFigure(3,0.4,0.51,'$\overline{v}_m$','fontSize',24,'interpreter','latex')
addTextToFigure(3,0.78,0.57,'$\overline{v}_s$','fontSize',24,'interpreter','latex')
end

function F = parallelTangentIllustrationFunction(x)

xoff1 = 0.5;
xoff2 = 1.5;

yoff1 = 0.5;
yoff2 = 0.62;

xstretch1 = 0.5;
xstretch2 = 0.7;

ystretch1 = 0.7;
ystretch2 = 1.2;

coeff = [ystretch1 xoff1 xstretch1 yoff1 ystretch2 xoff2 xstretch2 yoff2];

F(1) = x(2) - coeff(1)*log((x(1)-coeff(2))/coeff(3)) - coeff(4);
F(2) = x(4) - coeff(5)*log((x(3)-coeff(6))/coeff(7)) - coeff(8);
F(3) = x(1) - coeff(2) - coeff(1)/coeff(5) * (x(3)-coeff(6));
F(4) = coeff(5)/(x(3)-coeff(6)) - (x(4)-x(2))/(x(3)-x(1));

end