function [pressure,pressureError,kTI,kTIErr] = pressureHighDensity()

% This function is used to plot Fig. 5a

% This function plots the excess pressure (difference from the free
% volume equation of state) and gives the Taylor expansion 
% parameters.

basedir = [pathstr,filesep,'..',filesep,'data',filesep,'pressureData',filesep];

figure(97); clf; hold all;
dimColor = [NaN NaN NaN; NaN NaN NaN; 1 0 0; 6/7 0 1/7; 5/7 0 2/7; 4/7 0 3/7; 3/7 0 4/7; 2/7 0 5/7; 1/7 0 6/7; 0 0 1];
nParticlesList = [2916,2048,3888,23328,17496,6561,39366,81920];
fitRange = (2:35)';

kTI = NaN(1,10);
kTIErr = NaN(1,10);
slopTI = NaN(1,10);
slopTIErr = NaN(1,10);
curv = NaN(1,10);
curvErr = NaN(1,10);
curv1 = NaN(1,10);
curv1Err = NaN(1,10);
curv2 = NaN(1,10);
curv2Err = NaN(1,10);

% Taken from 

for dim = [3:8 9 10]
    rhoList = 0.9:0.005:0.995;
    rhoList = rhoList(rhoList<=0.99);
    if dim == 3
        rhoList = rhoList(1-rhoList<0.13);
    elseif dim == 6
        fitRange = (2:35);
        rhoList = rhoList(1-rhoList>=0.0);
    elseif dim == 7
        fitRange = (2:35);
        rhoList = rhoList((1-rhoList>=0.0)&(1-rhoList<=0.16));
    elseif dim == 8
        fitRange = (2:18);
        rhoList = rhoList(1-(rhoList)>0.02);
    elseif dim == 9
        fitRange = (2:35);
        rhoList = rhoList(1-(rhoList)>0.025);
    elseif dim == 10
        fitRange = (2:35);
        rhoList = rhoList(1-(rhoList)>0.02);
    end
    nParticles = nParticlesList(dim-2);
    pressure = NaN(1,numel(rhoList));
    pressureDiff = NaN(1,numel(rhoList));
    pressureError = NaN(2,numel(rhoList));
    rad = NaN(1,numel(rhoList));
    if dim ==6
        phic = pi^3/(48 * sqrt(3));
    elseif dim == 7
        phic = hyperBallVolume(dim)/16;
    elseif dim == 8
        phic = pi^4/(2^4 * 4*3*2);
    elseif dim == 9
        phic = hyperBallVolume(dim) * 2^(-dim/2);
    elseif dim == 10
        phic = 5/128 * pi^5 / 120;
    else
        phic = hyperBallVolume(dim) * 2^(-dim/2 - 1);
    end

    for i=1:numel(rhoList)
        try
            redRho = rhoList(i);
            phi = phic * redRho^dim;
            rad(i) = (phi/(nParticles * hyperBallVolume(dim)))^(1/dim);
            saveFile = [basedir,filesep,num2str(dim),filesep,num2str(redRho),filesep,num2str(nParticles),filesep,'savedGr.dat'];
            gofr = load(saveFile);
            r = gofr(fitRange,1);
            g = gofr(fitRange,2);
            gstd = gofr(fitRange,3);
            [res,~] = fit(r,g,'poly1','weights',1./sqrt(gstd));
            value = res.p2;
            coplace = 2;
            coeffError = confint(res);
            pressureError(1,i) = 2^(dim-1) * phi * (value - coeffError(1,coplace));
            pressureError(2,i) = 2^(dim-1) * phi * (coeffError(2,coplace) - value);
            pressure(i) = 1 + 2^(dim-1) * phi * value;
            thisx = 1-redRho;
            pressureDiff(i) = pressure(i) - (1/thisx);
        catch
        end
    end
    x = linspace(0.86,0.999,100);
    r = rhoList'.^dim;
    p = pressure';
    r = r(~isnan(p));
    pe = pressureError(1,~isnan(p));
    p = p(~isnan(p));
    x1 = (1-r.^(1/dim));
    y1 = p - 1./(1-r.^(1/dim));
    if dim == 3
        [res,~] = fit(x1(x1<=0.09),y1(x1<=0.09),'poly1','weights',1./sqrt(pe(x1<=0.09)));
    elseif dim >= 9
        [res,~] = fit(x1(x1<=0.1 & x1>=0.04),y1(x1<=0.1 & x1>=0.04),'poly1','weights',1./sqrt(pe(x1<=0.1 & x1>=0.04)));
    else
        [res,~] = fit(x1(x1<=0.1),y1(x1<=0.1),'poly1','weights',1./sqrt(pe(x1<=0.1)));
    end
    con = confint(res);
    kTIErr(dim) = con(2,2) - res.p2;
    kTI(dim) = res.p2;
    slopTI(dim) = res.p1;
    slopTIErr(dim) = con(2,1) - res.p1;
    if dim == 3
        [res2,~] = fit(x1(x1<=0.09),y1(x1<=0.09),'poly2','weights',1./sqrt(pe(x1<=0.09)));
    elseif dim >= 9
        [res2,~] = fit(x1(x1<=0.1 & x1>=0.01),y1(x1<=0.1 & x1>=0.01),'poly2','weights',1./sqrt(pe(x1<=0.1 & x1>=0.01)));
    else
        [res2,~] = fit(x1(x1<=0.1),y1(x1<=0.1),'poly2','weights',1./sqrt(pe(x1<=0.1)));
    end
    con2 = confint(res2);
    curv(dim) = res2.p1;
    curvErr(dim) = con2(2,1) - res2.p1;
    curv1(dim) = res2.p2;
    curv1Err(dim) = con2(2,2) - res2.p2;
    curv2(dim) = res2.p3;
    curv2Err(dim) = con2(2,3) - res2.p3;
    errorbar((1-r.^(1/dim)),p - 1./(1-r.^(1/dim))+dim*0.2,pe','.','Color',dimColor(dim,:),'markerSize',15,'Linewidth',1.3)
    x = linspace(min((1-r.^(1/dim))),max((1-r.^(1/dim))),100);
    h = plot(x,feval(res,x) + dim*0.2,'-','Color',dimColor(dim,:),'linewidth',1.5);
    set(get(get(h,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
end

legend('d=3','d=4','d=5','d=6','d=7','d=8','d=9','d=10','numcolumns',4,'location','southeast','fontsize',14)
axis([0 0.11 0 4])
plotSpecs()
set(gca,'FontSize',16)
xlabel('$1 - x^{1/d}$','interpreter','latex','FontSize',20)
ylabel('$p - 1/(1-x^{1/d}) + d/5$','interpreter','latex','FontSize',20)

end