import os
import numpy as np
from .. import enums
# from .. import fileIO
import time
import shutil
from operator import xor

class Mixin():
	def thermalizePacking(self, basedir=None, numTau = 10, mcBetweenSaves = 10000, overRideNumSteps = -1, overRideOverlap = 0, verbose = False, numTrials = 100,numCosts=50,neighborCutHigh=3,maxMCDisp=None,neighborCutDistance=None,firstPass=True,calcEnergy=True,calcMSD=False):
		msd = None
		expCutoff = 0.367879441171442
		firstPassCutoff = 0.9
		minRadius = np.min(self.getRadii())
		maxRadius = np.max(self.getRadii())
		if basedir is not None and os.path.isfile(basedir + '/tauAlpha.dat'):
			with open(basedir + '/tauAlpha.dat', 'r') as file:
				tauAlpha = file.read()
			with open(basedir + '/numSteps.dat', 'r') as file:
				numSteps = file.read()
			with open(basedir + '/neighborCutDistance.dat', 'r') as file:
				neighborCutDistance = file.read()
			with open(basedir + '/maxMCDisp.dat', 'r') as file:
				maxMCDisp = file.read()
			self.load(basedir + '/initial/')
			self.load(basedir)
			self.calcNeighborsCut(1.0)
			if (self.getMonteCarloType() != enums.monteCarloEnum.ecmc):
				if maxMCDisp is None:
					maxMCDisp = self.findMCMaxDisplacement(verbose = verbose,numTrials = numTrials)
				self.setMCMaxDisplacement(maxMCDisp)
			else:
				if maxMCDisp is not None:
					self.setECMCLength(maxMCDisp)
			initialPositions = self.getUnwrappedPositions()
		else:
			if basedir is not None:
				os.makedirs(basedir,exist_ok=True)
			tauAlpha = -1
			numSteps = 0
			self.calcNeighborsCut(1.0)
			if (self.getMonteCarloType() != enums.monteCarloEnum.ecmc):
				if maxMCDisp is None:
					maxMCDisp = self.findMCMaxDisplacement(verbose = verbose,numTrials = numTrials)
				self.setMCMaxDisplacement(maxMCDisp)
				# neighborCutDistance = self.findNeighborCutDistance(low = np.sqrt(self.getNDim())*maxMCDisp - 1, high = 10, numCosts = numCosts, verbose = verbose, numTrials = numTrials)
				# neighborCutDistance = self.findNeighborCutDistance(low = np.sqrt(self.getNDim())*maxMCDisp/minRadius - 1, high = 10, numCosts = numCosts, verbose = verbose, numTrials = numTrials)
				if neighborCutDistance is None:
					if firstPass:
						neighborCutDistance = np.max([3.0,2.0*np.sqrt(self.getNDim())*maxMCDisp])
					else:
						neighborCutDistance = self.findNeighborCutDistance(low = 2*np.sqrt(self.getNDim())*maxMCDisp, high = neighborCutHigh, numCosts = numCosts, verbose = verbose, numTrials = numTrials)
				self.calcNeighborsCut(neighborCutDistance)
			else:
				neighborCutDistance = 10.0
				if maxMCDisp is None:
					maxMCDisp = self.getECMCLength()
				else:
					self.setECMCLength(maxMCDisp)
			initialPositions = self.getUnwrappedPositions()
			# self.calcEnergy()
			# print("energy is " + str(self.getEnergy()))
		overlap = np.array([],dtype=np.float64)
		if overRideNumSteps == -1 or firstPass == True:
			initialPositions = self.getUnwrappedPositions()
			if calcMSD:
				self.resetCenterOfMass()
				msd = np.array([],dtype=np.float64)
			while not xor(tauAlpha < 0,numTau*tauAlpha < numSteps):				
				if (self.getMonteCarloType() != enums.monteCarloEnum.ecmc):	
					if (self.getSumOfMaxDisplacements() >= 2*neighborCutDistance*minRadius):
						self.calcNeighborsCut(neighborCutDistance)
						self.resetDisplacement()
						if calcEnergy:
							self.calcEnergy()
				self.MCStep()
				if calcMSD:
					delta = self.getUnwrappedPositions() - initialPositions - self.getCenterOfMassShift()
					msd = np.append(msd,np.mean(np.sum(delta**2,1))/(minRadius**2))
				thisOverlap = self.calcMCOverlap(initialPositions)
				overlap = np.append(overlap,thisOverlap)
				if verbose:
					print(thisOverlap)
				if thisOverlap < overRideOverlap:
					return numSteps,-1,neighborCutDistance,maxMCDisp,msd,overlap
				# if (self.getSumOfMaxDisplacements()/2*minRadius >= neighborCutDistance):
				if (firstPass and thisOverlap < firstPassCutoff):
					numSteps,tauAlpha,neighborCutDistance,maxMCDisp,msd,overlap = self.thermalizePacking(basedir=basedir, numTau = numTau, mcBetweenSaves = mcBetweenSaves, overRideNumSteps = overRideNumSteps, overRideOverlap = overRideOverlap, verbose = verbose, numTrials = numTrials,numCosts=numCosts,neighborCutHigh=neighborCutHigh,firstPass = False)
					return numSteps,tauAlpha,neighborCutDistance,maxMCDisp,msd,overlap
				if thisOverlap < expCutoff and tauAlpha == -1 and numSteps > 0:
					tauAlpha = numSteps
					print("tauAlpha is " + str(tauAlpha))
				numSteps += 1
				if basedir is not None:
					if numSteps%mcBetweenSaves == 0:
						self.save(basedir)
						with open(basedir + '/tauAlpha.dat', 'w') as file:
							file.write(str(tauAlpha) + '\n')
						with open(basedir + '/numSteps.dat', 'w') as file:
							file.write(str(numSteps) + '\n')
						with open(basedir + '/neighborCutDistance.dat', 'w') as file:
							file.write(str(neighborCutDistance) + '\n')
			self.setTauAlpha(tauAlpha)
			return numSteps, tauAlpha, neighborCutDistance, maxMCDisp, msd, overlap
		else:
			initialPositions = self.getUnwrappedPositions()
			if calcMSD:
				self.resetCenterOfMass()
				msd = np.zeros(overRideNumSteps)
			while numSteps < overRideNumSteps:
				self.MCStep()
				if (self.getMonteCarloType() != enums.monteCarloEnum.ecmc):
					if (self.getSumOfMaxDisplacements() >= 2*neighborCutDistance*minRadius):
						self.calcNeighborsCut(neighborCutDistance)
						self.resetDisplacement()
				overlap = np.array(overlap,self.calcMCOverlap(initialPositions))
				if calcMSD:
					delta = self.getUnwrappedPositions() - initialPositions - self.getCenterOfMassShift()
					msd[numSteps] = np.mean(np.sum(delta**2,1))/(minRadius**2)
				numSteps += 1
			return numSteps, tauAlpha, neighborCutDistance, maxMCDisp, msd, overlap

	def findMCMaxDisplacement(self, loProp = 0.4, hiProp = 0.5, loDist = 0.0, hiDist = 0.5, distTol = 1e-3, numTrials = 100, neighborCut = 3,verbose = False,neighborRecalc=False):
		# Finds the monte carlo displacement that is appropriate for a given packing
		# Here, we want the proportion of accepted moves to be between loProp and hiProp
		# and we start with a guess that this displacement is between loDist and hiDist
		gold = 0.61803398875
		initialPos = self.getUnwrappedPositions()
		acceptanceRatio = 0
		minRadius = np.min(self.getRadii())
		trial = (hiDist - loDist)*gold + loDist
		self.setPositions(initialPos)
		self.calcNeighborsCut(neighborCut)
		while (acceptanceRatio < loProp or acceptanceRatio > hiProp) and ((hiDist - loDist)/hiDist >= distTol):
			trial = (hiDist - loDist)*gold + loDist
			self.setPositions(initialPos)
			if neighborRecalc:
				self.calcNeighborsCut(neighborCut)
			self.calcEnergy()
			self.resetCenterOfMass()
			acceptanceRatioVector = np.zeros(numTrials)
			self.setMCMaxDisplacement(trial)
			self.resetDisplacement()
			for i in range(numTrials):
				if (neighborRecalc and self.getSumOfMaxDisplacements() >= 2*neighborCut*minRadius):
					self.calcNeighborsCut(neighborCut)
					self.resetDisplacement()
					self.calcEnergy()
				self.MCStep()
				acceptanceRatioVector[i] = self.getAcceptanceRatio()
			acceptanceRatio = np.mean(acceptanceRatioVector)
			if verbose:
				print("loDist = " + str(loDist) + " and hiDist = " + str(hiDist))
				print("maxDisp = " + str(trial) + " and acceptanceRatio = " + str(acceptanceRatio))
			if acceptanceRatio > hiProp:
				loDist = trial
			if acceptanceRatio < loProp:
				hiDist = trial
		self.setPositions(initialPos)
		self.resetCenterOfMass()
		self.resetDisplacement()
		if verbose:
			print("Using a max displacement of " + str(trial))
		return trial

	def findMCMaxDisplacementCM(self, loProp = 0.4, hiProp = 0.5, loDist = 0.0, hiDist = 0.5, distTol = 1e-3, numTrials = 1000, verbose = False):
		# Finds the monte carlo center of mass displacement appropriate for a given packing
		# Here, we want the proportion of accepted moves to be between loProp and hiProp
		# and we start with a guess that this displacement is between loDist and hiDist
		# Note that because there is no internal motion, neighbors don't need to be recalculated
		gold = 0.61803398875
		initialPos = self.getUnwrappedPositions()
		acceptanceRatio = 0
		minRadius = np.min(self.getRadii())
		trial = (hiDist - loDist)*gold + loDist
		self.setPositions(initialPos)
		while (acceptanceRatio < loProp or acceptanceRatio > hiProp) and ((hiDist - loDist)/hiDist >= distTol):
			trial = (hiDist - loDist)*gold + loDist
			self.setPositions(initialPos)
			self.calcEnergy()
			self.resetCenterOfMass()
			acceptanceVector = np.zeros(numTrials)
			self.setMCMaxDisplacementCM(trial)
			self.resetDisplacement()
			for i in range(numTrials):
				acceptanceVector[i] = self.MCStepCM()
			acceptanceRatio = np.mean(acceptanceVector)
			if verbose:
				print("loDist = " + str(loDist) + " and hiDist = " + str(hiDist))
				print("maxDisp = " + str(trial) + " and acceptanceRatio = " + str(acceptanceRatio))
			if acceptanceRatio > hiProp:
				loDist = trial
			if acceptanceRatio < loProp:
				hiDist = trial
		self.setPositions(initialPos)
		self.resetCenterOfMass()
		self.resetDisplacement()
		if verbose:
			print("Using a max center of mass displacement of " + str(trial))
		return trial

	def findNeighborCutDistance(self, low = 0, high = 10, numCosts = 30, numTrials = 100,verbose=False):
		initialPos = self.getUnwrappedPositions()
		cost = np.zeros(numCosts)
		minCost = 1000000
		minCostCut = 0
		minRadius = np.min(self.getRadii())
		for i in range(numCosts):
			self.setPositions(initialPos)
			cutDistance = low + (high - low)*i/numCosts
			self.calcNeighborsCut(cutDistance)
			self.calcEnergy()
			if verbose:
				print("cutDistance is " + str(cutDistance))
			start = time.time()
			for j in range(numTrials):
				if (self.getSumOfMaxDisplacements() >= 2*cutDistance*minRadius):
					self.calcNeighborsCut(cutDistance)
					self.resetDisplacement()
					self.calcEnergy()
				self.MCStep()
			cost[i] = time.time() - start
			self.calcNeighborsCut(high)
			self.calcEnergy()
			if verbose:
				print("energy is " + str(self.getEnergy()))
				print("cost is " + str(cost[i]))
			if cost[i] < minCost and self.getEnergy() == 0.0:
				minCost = cost[i]
				minCostCut = cutDistance
			self.resetDisplacement()
			self.resetCenterOfMass()
		self.setPositions(initialPos)
		self.calcNeighborsCut(10.0)
		return minCostCut

	def findMaxVolumeStep(self, loDist=0.0, hiDist=1, loProp=0.4, hiProp=0.5, distTol=1e-2, numTrials=100, stretchDirection=0, neighborCut=1, verbose=False):
		gold = 2.0/(1.0 + np.sqrt(5.0))
		acceptanceRatio = 0
		initialPos = self.getUnwrappedPositions()
		initialBoxSize = self.getBoxSize()
		numTrue = np.zeros(numTrials)
		minRadius = np.min(self.getRadii())
		neighborRecalc = ~np.all(self.getMCMCBlockedDirection())
		while (acceptanceRatio < loProp or acceptanceRatio > hiProp) and ((hiDist - loDist)/hiDist >= distTol):
			self.setBoxSize(initialBoxSize)
			self.setPositions(initialPos)
			self.resetDisplacement()
			self.calcNeighborsCut(neighborCut)
			self.calcEnergy()
			trial = (hiDist - loDist)*gold + loDist
			if verbose:
				print(str(loDist) + " " + str(trial) + " " + str(hiDist))
			self.setMaxVolumeShift(trial)
			for i in range(numTrials):
				if (neighborRecalc and self.getSumOfMaxDisplacements() >= 2*neighborCut*minRadius):
					self.calcNeighborsCut(neighborCut)
					self.resetDisplacement()
					self.calcEnergy()
				self.MCStep()
				if (neighborRecalc and self.getSumOfMaxDisplacements() >= 2*neighborCut*minRadius):
					self.calcNeighborsCut(neighborCut)
					self.resetDisplacement()
					self.calcEnergy()
				numTrue[i] = self.MCStepAspectRatio(stretchDirection)
			acceptanceRatio = np.sum(numTrue)/numTrials
			if acceptanceRatio > hiProp:
				loDist = trial
			if acceptanceRatio < loProp:
				hiDist = trial
			if verbose:
				print(acceptanceRatio)
			self.setBoxSize(initialBoxSize)
			self.setPositions(initialPos)
			self.resetDisplacement()
			self.calcNeighborsCut(neighborCut)
			self.calcEnergy()
		if verbose:
			print("maxVolumeShift is " + str(trial))
		return trial

	def MKThermalization(self, resetLambdas = False, neighborCut = 0.01,lambdaMax=1):
		if resetLambdas:
			self.setRandomMKLambdas()
		self.calcNeighborsCut(neighborCut)
		lam = self.getMKLambdas()
		contacts = self.getContacts().tocoo()
		while contacts.size > 0:
			row = contacts.row
			col = contacts.col
			randVector = self.getRandomVector(row[row>col].size,self.getNDim(),maxElement=lambdaMax)
			lam[row[row>col],col[row>col],:] = randVector
			lam[col[row>col],row[row>col],:] = -randVector
			self.setMKLambdas(lam)
			self.calcNeighborsCut(neighborCut)
			contacts = self.getContacts().tocoo()



