#ifndef MCPACKING_H
#define MCPACKING_H

#include "spimpl.h"
#include "enumTypes.h"
#include <algorithm>
#include <fstream>
#include <functional>
#include <iomanip>
#include <iostream>
#include <math.h>
#include <random>
#include <stdlib.h>
#include <string>
#include <sstream>
#include <vector>



using namespace std;

class MCPacking {
public:
	MCPacking(int dim_i, long nParticles_i, double phi_i);
	~MCPacking();

	bool anyOverlap();
	void calcCenterOfMass();
	void calcContacts(double gap);
	void calcDeltaNormSq(vector<double>::iterator x, vector<double>::iterator y, vector<double>::iterator delta, double& normSq);
	void calcEnergy();
	void calcForceEnergy();
	double calcGap(long pID, long nID);
	vector<double> calcGofR(const vector<double> binEdges, const long numVolumePoints);
	double calcGradMultiple(const double norm, const double overlap, const double radSum);
	vector<double> calcGRBinVolume(vector<double> binEdges,long numVolumePoints);
	double calcMCOverlap(vector<double>& initialPos);
	void calcNeighborsCut(const double cutDistance);
	// double calcOverlap(long pID, long nID);
	double calcOverlap(vector<double>::iterator pos1, vector<double>::iterator pos2, long pID, long nID);
	vector<double> createD9Plus(vector<double> crystalPos);
	void dnLatticePoint(vector<double>::iterator p, vector<int32_t> &mirror);
	int e8LatticePoint(vector<double>::iterator p, vector<int32_t> &mirror);
	double fastRound(double numToRound);
	// float fastRound(float fastRound);
	double getAcceptanceRatio();
	vector<double> getBoxSize();
	vector<double> getCenterOfMassShift();
	vector<double> getConstraint();
	double getConstraintMagnitude();
	simControlStruct::constraintEnum getConstraintType();
	tuple< vector<long>, vector<long> > getContacts(double gap);
	tuple< vector<long>, vector<long>, vector<double> > getContactVectors(double gap);
	vector<double> getCrystalPositions();
	simControlStruct::distanceEnum getDistanceType();
	vector<double> getD9Shift();
	double getECMCLength();
	double getECMCPressure();
	double getEinsteinEnergy();
	double getEinsteinLambda();
	simControlStruct::einsteinEnum getEinsteinType();
	double getEinsteinWeightedConstant();
	double getEnergy();
	vector<double> getEnergyVector();
	simControlStruct::ensembleEnum getEnsembleType();
	vector<double> getExternalForce();
	double getFIREDamping();
	long getFIREIteration();
	double getFIRETimeStep();
	simControlStruct::geometryEnum getGeometryType();
	tuple<vector<long>, vector<long>, vector<double> > getHessian();
	int getInversePower();
	double getInversePowerCutoff();
	double getLatticeSpacing();
	double getLEshift();
	double getLogZero();
	double getMaxDisplacement();
	double getMaxUnbalancedForce();
	double getMaxVolumeShift();
	double getMCMaxDisplacement();
	double getMCMaxDisplacementCM();
	double getMCMaxDisplacementD9Relative();
	vector<bool> getMCMCBlockedDirection();
	double getMCOverlap();
	double getMeanRadius();
	double getMinGap(double maxSearchGap);
	vector<double> getMKLambdas();
	simControlStruct::monteCarloEnum getMonteCarloType();
	int getNDim();
	tuple<vector<long>, vector<long> > getNeighbors();
	simControlStruct::neighborEnum getNeighborType();
	vector<double> getNetForces();
	double getNonAdditivity();
	long getNumParticles();
	int getNumRows();
	simControlStruct::particleEnum getParticleType();
	vector<double> getPerParticleStressTensor();
	double getPhi();
	vector<double> getPositions();
	double getPotentialPower();
	simControlStruct::potentialEnum getPotentialType();
	vector<double> getRadii();
	double getRadSum(double rad1, double rad2);
	vector<double> getRandomGaussianVector(long numEntries, double mean, double std);
	vector<double> getRandomVector(long numEntries, double maxElement);
	tuple< vector<long>, vector<long>, vector<double> > getSmallGaps(double maxGap);
	tuple< vector<long>, vector<long>, vector<double> > getSmallOverlaps(double maxOverlap);
	double getSumOfMaxDisplacements();
	double getTargetPressure();
	long getTauAlpha();
	double getTemperature();
	vector<double> getTotalDisplacement();
	vector<double> getUnwrappedPositions();
	double getVolume();
	double hyperBallCoefficient(int dim);
	void initFIREMinimizer(vector<double> FIREParams_, double maxForceRescale_, long N_min_, long lastRestart_);
	void initForceEnergy();
	bool isContact(double dist, double radSum, double gapSize);
	bool isNeighbor(double dist, double radSum, double cutDistance);
	void load(const string basedir);
	void MCStep();
	bool MCStepAspectRatio(int direction);
	bool MCStepCM();
	bool MCStepD9Relative(int dim, double translation);
	bool MCStepVolume();
	bool MCStepVolumeStretch(int dir);
	void minimizerLoopFIRE();
	vector<double> pbcDisplacement(vector<double>::iterator pos1, vector<double>::iterator pos2, long pID, long nID);
	vector<double> pbcDisplacementOutside(vector<double> pos1, vector<double> pos2);
	double pbcDistanceOutside(vector<double> pos1, vector<double> pos2);
	double pbcDistance(vector<double>::iterator pos1, vector<double>::iterator pos2, long pID, long nID);
	double pbcDistance(const double x, const double y, const double size);
	double pbcDistance(const double x, const double y);
	double pbcDistanceLE(const double x1, const double x2, const double y1, const double y2);
	double pbcDistanceSq(vector<double>::iterator pos1, vector<double>::iterator pos2, long pID, long nID);
	void removeParticles(vector<long> pIDList);
	void rescaleForces(double maxForceRescale);
	void resetCenterOfMass();
	void resetDisplacement();
	void resetFIREMaxTimeStep(double dt_max_);
	void save(const string basedir);
	void setConstraint(vector<double> constraint);
	void setConstraintType( simControlStruct::constraintEnum constraintType_ );
	void setCrystalPositions(vector<double> crystalPos_i);
	void setDistanceType(simControlStruct::distanceEnum distanceType_);
	void setD9Shift(vector<double> d9Shift_i);
	void setECMCLength(double ecmcLength_i);
	void setEinsteinLambda(double einsteinLambda);
	void setEinsteinType(simControlStruct::einsteinEnum einsteinType_);
	void setEnsembleType(simControlStruct::ensembleEnum ensembleType_);
	void setExternalForce(vector<double> externalForceArray);
	void setFIREDamping(double fireDamping_);
	void setFIREIteration(long lastRestart_);
	void setFIRETimeStep(double fireTimeStep_);
	void setGeometryType( simControlStruct::geometryEnum geometryType_ );
	void setInitialPositions();
	void setInversePower(int inversePower_i);
	void setInversePowerConstants(double inversePowerCutoff_i, double inversePowerC0_i, double inversePowerC1_i, double inversePowerC2_i);
	void setLatticeSpacing(double latticeSpacing_);
	void setLEshift(double LEData);
	void setLogZero(const double logZero_i);
	void setMaxForceRescale(double maxForceScale);
	void setMaxVolumeShift(double maxVolumeShift_i);
	void setMCMaxDisplacement(double maxDisplacement_i);
	void setMCMaxDisplacementCM(double maxDisplacementCM_i);
	void setMCMaxDisplacementD9Relative(double maxDisplacementD9Relative_i);
	void setMCMCBlockedDirection(vector<bool> mcmcBlockedDirection_i);
	void setMCOverlap(double overlapCutoff_i);
	void setMKLambdas(vector<double> MKLambdas_i);
	void setMonteCarloType(simControlStruct::monteCarloEnum monteCarloType_);
	void setNDim(int dim_i);
	void setNeighborType( simControlStruct::neighborEnum neighborType_ );
	void setNonAdditivity(double nonAdditivity);
	void setNumParticles(long nParticles_i);
	void setNumRows(int numRows);
	void setParticleType(simControlStruct::particleEnum particleType_);
	void setPhi(double phi_i);
	void setPositions(vector<double> newPos);
	void setPotentialPower(double potentialPower_i);
	void setPotentialType( simControlStruct::potentialEnum potentialType_ );
	void setRadii(vector<double> newRadii);
	void setRandomMKLambdas();
	void setRandomPositions();
	void setRandomSeed(long seedValue);
	void setBoxSize(vector<double> newBoxSize);
	void setTargetPressure(double targetPressure_i);
	void setTauAlpha(long tauAlpha_i);
	void setTemperature(double temperature_i);
	void translate(const double stepSize, vector<double> translationVector);
	vector<bool> uniqueParticles(double cutoff);
	void zeroVelocities();

	class PackingImplementation;
	spimpl::impl_ptr<PackingImplementation> pi_;

	class Neighbors;
	spimpl::impl_ptr<Neighbors> neigh_;

	class MK;
	spimpl::impl_ptr<MK> mk_;

	class FIRE;
	spimpl::impl_ptr<FIRE> fire_;

	class CG;
	spimpl::impl_ptr<CG> cg_;

	class SD;
	spimpl::impl_ptr<SD> sd_;

private:
	mt19937 mt;
	double EPSILON;
	double acceptanceRatio;
	double einsteinLambda;
	uniform_real_distribution<double> uniformDist = uniform_real_distribution<double>(0.0,1.0);
	const double pi = 3.14159265358979323846;
	vector<double> initialPositions;
	double ecmcLength;
	double ecmcPressure;
	double energy;
	vector<double> energyVector;
	vector< vector<long> > contactList;
	vector<long> numContacts;
	vector< vector<long> > neighborList;
	double maxDisplacement;
	double maxDisplacementCM;
	double maxDisplacementD9Relative;
	double maxVolumeShift;
	vector<bool> mcmcBlockedDirection;
	double logZero;
	vector<double> HCMaxDispSet{0,0,0.110,0.060,0.040,0.033,0.027,0.021};
	long tauAlpha = -1;
	double temperature;
	vector<double> d9Shift;
	double calcEnergyPerInteraction(const vector<double>::iterator pos1, const vector<double>::iterator pos2, const long pID, const long nID);
	double calcEnergyPerParticle(const long pID);
	double calcForcePerInteraction(const vector<double>::iterator pos1, const vector<double>::iterator pos2, const long pID, const long nID);
	double calcForceEnergyPerParticle(const long pID);
	vector<double> calcNVector(const long pID, const long nID);
	bool centerOfMassMoveMC(const vector<double>& translationVector);
	double calcPeriodicCosList(const long pID, const double ksp, const vector<double> translationVector);
	double getLamNorm();
	double getPeriodicRefNormalization();
	vector<long> getRandomSequence();
	double halfPowerPow(const double base, const double powerNum);
	void MCStepMCMC();
	void MCStepECMC(int dir);
	void saveScalars(const string basedir);
	void shiftCenterOfMass(const vector<double>& translationVector, long weight);
	bool singleMCEinstein(const long pID, const vector<double>& translationVector);
	bool singleMCHardSphere(const long pID, const vector<double>& translationVector);
	bool singleMCThermal(const long pID, const vector<double>& translationVector);
};

#endif