#ifndef PIMPLCLASSES_H_
#define PIMPLCLASSES_H_

#include "MCPacking.h"
#include "enumTypes.h"

class MCPacking::PackingImplementation {
public:

	PackingImplementation(MCPacking* ptr);
	~PackingImplementation() {
	};

	MCPacking* packing_; //Pointer to the enclosing class

	double volume;
	double potentialPower;
	int nDim;
	long numParticles;
	double acceptanceRatio;
	double einsteinLambda;
	double latticeSpacing;
	double periodicSign;
	double lastNeighborCut;
	vector<double> pos;
	vector<double> lastPos;
	vector<double> radii;
	vector<double> initialPositions;
	vector<double> totalDisplacement;
	vector<double> distTravelledSq;
	vector<double> crystalPos;
	vector<double> centerOfMassShift;
	vector<double> centerOfMass;
	vector<double> unwrappedPos;
	vector<double> boxSize;
	int numRows;
	double phi;
	double energy;
	vector<double> mass;
	vector<double> energyVector;
	vector<double> netForce;
	// vector< vector<long> > contactList;
	double maxDisplacement;
	double logZero;
	double temperature;
	double maxForceRescale;
	vector<double> MKLambdas;
	vector<double> constraint;
	vector<double> externalForce;
	double forceDotConstraint;
	double LEshift;
	double inversePowerCutoff;
	int inversePower;
	double inversePowerC0;
	double inversePowerC1;
	double inversePowerC2;
	double nonAdditivity;
	double targetPressure;
	double overlapParameter;
	
	simControlStruct simControl;

	double calcVolumePrefactor(int dim);
	void rescaleForces(double maxForceRescale_);
	void calcCurrentPhi();
	void translateParticle(const long particleID, const vector<double> translation);
	void doubleVectorToCSR(vector< vector<long> >& doubleVector, vector<long> &indices, vector<long> &indptr);
	void doubleVectorToCOO(vector< vector<long> >& doubleVectorIndices, vector< vector<double> >& doubleVectorData, vector<long> &indices1, vector<long> &indices2, vector<double> &data, long blockSize1, long blockSize2);
	void paddedToCSR(vector<long> &paddedIndices, vector<long> &rowSizes, vector<long> &indices, vector<long> &indptr);
	void paddedToCOO(vector<long> &paddedIndices, vector<long> &rowSizes, vector<double> &paddedData, vector<long> &indices1, vector<long> &indices2, vector<double> &data, long blockSize1, long blockSize2);
	void calcSmallGaps(vector< vector<double> >& gapList, vector< vector<long> >& gapIndices, double maxGap);
	void calcSmallOverlaps(vector< vector<double> >& overlapList, vector< vector<long> >& overlapIndices, double minOverlap);

	//Project one vector (constraint) out of another (vectorIn)
	double projectVector(vector<double>& vectorIn);
};

class MCPacking::FIRE {
public:
	FIRE(MCPacking* ptr);
	~FIRE() {
	};
	MCPacking* packing_;  //Pointer to the enclosing class

	vector<double> mass;
	vector<double> velocity;
	vector<double> FIREParams;
	//Global FIRE params
	double a_start;
	double fire_alpha;
	double f_dec;
	double f_inc;
	double f_a;

	double dt_max;
	double dt;

	long N_min;
	long lastRestart;

	//Set the mass for each degree of freedom
	void setMass(const double massFactor);

	//Update position and velocity in response to an applied force
	void updatePositionAndVelocityFIRE();

	//Update velocity in response to an applied force and return the maximum displacement in the previous step
	void updateVelocityFIRE();

	//Bend the velocity towards the force
	void bendVelocityTowardsForceFIRE();

	//Functions related to the minimizeFIRE
	void initMinimizer(double a_start_, double f_dec_, double f_inc_, double f_a_, double dt_, double dt_max_, double maxForceRescale_, long N_min_, double fire_alpha_, long lastRestart_);

	void minimizerLoop();

};

#endif /* PIMPLCLASSES_H_ */