(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 12.3' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    265412,       5828]
NotebookOptionsPosition[    258598,       5733]
NotebookOutlinePosition[    258990,       5749]
CellTagsIndexPosition[    258947,       5746]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[TextData[{
 "To begin, find and replace the term \[OpenCurlyDoubleQuote]basedir\
\[CloseCurlyDoubleQuote] with a directory on your computer for these files to \
save to. File separators will need to be switched from \
\[OpenCurlyDoubleQuote]/\[CloseCurlyDoubleQuote] to \[OpenCurlyDoubleQuote]\\\
\[CloseCurlyDoubleQuote] on Windows machines, and the subfolders which sort \
the data will have to be created (Namely, \
latteIntegrals/MathematicaFormat/E1/, latteIntegrals/MathematicaFormat/E2/, \
and latteIntegrals/MathematicaFormat/D2/). This program will create a set of \
H-Representation files and their associated polynomials, which can then be \
run with the LattE program.\n\nOnly d=3 is provided, but this can be quickly \
modified to do calculations in d>3 by simply modifying w1 (the set of nearest \
neighbor vectors).\n\nHere, we walk through the calculation of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "1"], TraditionalForm]],ExpressionUUID->
  "935bb9e4-f11e-4b28-b285-4f7c02ebe341"],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "1"], TraditionalForm]],ExpressionUUID->
  "deef6f47-67d7-4e9b-b6f9-c9c7bce3e138"],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "2"], TraditionalForm]],ExpressionUUID->
  "5267326d-bf06-4eb2-b1ec-5bf3af262a67"],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "2"], TraditionalForm]],ExpressionUUID->
  "acb3711e-8477-4e28-a865-18e84e2a03cd"],
 " in d=3. To begin, we introduce the neighbor vectors w at first order, \
where all particles are pinned except for one, whose coordinates are z = [x1, \
x2,x3] where  x1, x2, and x3 are real numbers. A convenient orientation \
aligns them as such:"
}], "Text",
 CellChangeTimes->{{3.836378350924337*^9, 3.8363787815311317`*^9}, {
   3.8363839265416937`*^9, 3.836383936458385*^9}, 3.836385611499221*^9, {
   3.847983613881708*^9, 3.847983763144606*^9}, {3.8479838348697453`*^9, 
   3.847983851248775*^9}, {3.8479838842788258`*^9, 3.847983935959984*^9}, {
   3.847984096977997*^9, 
   3.847984168642479*^9}},ExpressionUUID->"98bf8393-6c63-45f0-93be-\
f29ffd94c896"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"w1", " ", "=", 
   RowBox[{"-", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", 
        RowBox[{"-", "1"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        FractionBox["1", "2"], ",", 
        FractionBox["1", 
         SqrtBox["2"]], ",", 
        RowBox[{"-", 
         FractionBox["1", "2"]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        FractionBox["1", "2"], ",", 
        FractionBox["1", 
         SqrtBox["2"]], ",", 
        FractionBox["1", "2"]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        FractionBox["1", "2"], ",", 
        RowBox[{"-", 
         FractionBox["1", 
          SqrtBox["2"]]}], ",", 
        FractionBox["1", "2"]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        FractionBox["1", "2"], ",", 
        RowBox[{"-", 
         FractionBox["1", 
          SqrtBox["2"]]}], ",", 
        RowBox[{"-", 
         FractionBox["1", "2"]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], ",", 
        FractionBox["1", 
         SqrtBox["2"]], ",", 
        RowBox[{"-", 
         FractionBox["1", "2"]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], ",", 
        FractionBox["1", 
         SqrtBox["2"]], ",", 
        FractionBox["1", "2"]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], ",", 
        RowBox[{"-", 
         FractionBox["1", 
          SqrtBox["2"]]}], ",", 
        RowBox[{"-", 
         FractionBox["1", "2"]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], ",", 
        RowBox[{"-", 
         FractionBox["1", 
          SqrtBox["2"]]}], ",", 
        FractionBox["1", "2"]}], "}"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"d", " ", "=", " ", 
   RowBox[{
    RowBox[{"Dimensions", "[", "w1", "]"}], "[", 
    RowBox[{"[", "2", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"z", " ", "=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Subscript", "[", 
      RowBox[{"x", ",", "i"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "d"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"$Assumptions", " ", "=", " ", 
   RowBox[{
    RowBox[{"Evaluate", "[", "z", "]"}], " ", "\[Element]", " ", "Reals"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sites", " ", "=", " ", 
   RowBox[{"Length", "[", "w1", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"d", " ", "=", " ", 
   RowBox[{"Length", "[", "z", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"getLinearForm", "[", 
   RowBox[{
    RowBox[{"ineqs", ":", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"_LessEqual", "|", "_GreaterEqual"}], ")"}], ".."}], "}"}]}], 
    ",", "vars_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"mults", ",", "lpolys", ",", "rhs", ",", "lhs"}], "}"}], ",", 
    RowBox[{
     RowBox[{"mults", "=", 
      RowBox[{
       RowBox[{"Map", "[", 
        RowBox[{"Head", ",", "ineqs"}], "]"}], "/.", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"LessEqual", "->", "1"}], ",", 
         RowBox[{"GreaterEqual", "->", 
          RowBox[{"-", "1"}]}]}], "}"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"lpolys", "=", 
      RowBox[{"mults", "*", 
       RowBox[{"Apply", "[", 
        RowBox[{"Subtract", ",", "ineqs", ",", 
         RowBox[{"{", "1", "}"}]}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"rhs", ",", "lhs"}], "}"}], "=", 
      RowBox[{"Normal", "[", 
       RowBox[{"CoefficientArrays", "[", 
        RowBox[{"lpolys", ",", "vars"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"lhs", ",", 
       RowBox[{"-", "rhs"}]}], "}"}]}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.8354530449808693`*^9, 3.83545318172403*^9}, {
   3.835466684290997*^9, 3.8354669789223957`*^9}, 3.8354670192738953`*^9, {
   3.835467141655026*^9, 3.8354672329366827`*^9}, {3.835467266130377*^9, 
   3.835467490623328*^9}, {3.835467568513277*^9, 3.835467609264051*^9}, {
   3.835467845884493*^9, 3.835467859621139*^9}, 3.8354678931728287`*^9, 
   3.835467964683958*^9, {3.835468128981934*^9, 3.835468149404161*^9}, {
   3.835468346394278*^9, 3.835468364263399*^9}, {3.835468432382189*^9, 
   3.835468438329624*^9}, {3.8354688906661577`*^9, 3.8354688940224953`*^9}, {
   3.835468953723577*^9, 3.83546895416929*^9}, {3.835469010652217*^9, 
   3.8354690113065767`*^9}, {3.835471864869014*^9, 3.835471876518332*^9}, {
   3.835471944040312*^9, 3.835471944667301*^9}, {3.835472133568591*^9, 
   3.835472146458373*^9}, {3.8354722398527403`*^9, 3.8354722684825993`*^9}, {
   3.835472327189047*^9, 3.835472441444352*^9}, {3.835472477954636*^9, 
   3.8354725307475023`*^9}, {3.8354725873848658`*^9, 3.835472589858736*^9}, {
   3.835472677067501*^9, 3.835472677382126*^9}, {3.835472727006681*^9, 
   3.835472739451055*^9}, {3.835472788537559*^9, 3.835472803024736*^9}, {
   3.83547286249133*^9, 3.835472862651248*^9}, {3.8354729843857927`*^9, 
   3.835473003457418*^9}, {3.835473033608646*^9, 3.835473068149043*^9}, {
   3.83547330823114*^9, 3.835473311573303*^9}, {3.835473451426152*^9, 
   3.835473453654476*^9}, {3.835473491704762*^9, 3.8354734938963327`*^9}, {
   3.835473553080357*^9, 3.835473560735557*^9}, {3.835479086588212*^9, 
   3.835479121584784*^9}, {3.835479170024287*^9, 3.8354791786793613`*^9}, {
   3.835479451447424*^9, 3.83547945959903*^9}, {3.8354799750360327`*^9, 
   3.835480115178803*^9}, {3.835480174479991*^9, 3.835480286107811*^9}, {
   3.835480349910143*^9, 3.8354806066934767`*^9}, {3.8354807107012663`*^9, 
   3.835480803537777*^9}, {3.8354812902265244`*^9, 3.835481336218375*^9}, {
   3.8356939541238613`*^9, 3.835693960941977*^9}, {3.835694086037374*^9, 
   3.835694134340847*^9}, {3.8356941698249493`*^9, 3.835694231661469*^9}, {
   3.835694842649637*^9, 3.835694890402657*^9}, {3.835694984003853*^9, 
   3.835694997662994*^9}, {3.835695047950551*^9, 3.835695051447279*^9}, {
   3.835695095382547*^9, 3.835695116692688*^9}, {3.8356952452073927`*^9, 
   3.835695276050478*^9}, {3.835695315652401*^9, 3.835695331464056*^9}, {
   3.8356965536750593`*^9, 3.835696604284286*^9}, {3.8356966589138603`*^9, 
   3.835696661218536*^9}, {3.8356967113013268`*^9, 3.835696732079521*^9}, {
   3.835696783326808*^9, 3.8356967933364267`*^9}, {3.835696898555019*^9, 
   3.835696973046053*^9}, {3.8356972267731543`*^9, 3.835697312232521*^9}, 
   3.835697591279818*^9, {3.835698562349609*^9, 3.835698616797779*^9}, {
   3.835699142240032*^9, 3.835699216053561*^9}, 3.835699266938794*^9, 
   3.83570007047959*^9, {3.836378161311626*^9, 3.836378239342669*^9}, {
   3.836378333129324*^9, 3.836378335452346*^9}, 3.836378786123816*^9, 
   3.836378823281918*^9, 3.836379589848604*^9, {3.8363809647539186`*^9, 
   3.836380966893023*^9}, {3.8363837181192636`*^9, 3.836383756283239*^9}, {
   3.836383824260098*^9, 3.836383826102229*^9}, {3.836545165981612*^9, 
   3.836545349837686*^9}, 3.836545385828389*^9, {3.836545428793592*^9, 
   3.836545441614017*^9}, {3.836545475655363*^9, 3.8365455598909283`*^9}, 
   3.836555820911029*^9, {3.837000856554017*^9, 3.837000857972892*^9}, {
   3.83734822459297*^9, 3.837348261792622*^9}, 3.837768452932541*^9, 
   3.8377685152930117`*^9, {3.838105439826264*^9, 3.838105440780579*^9}, {
   3.839572245323113*^9, 3.839572289006974*^9}, {3.847983565354785*^9, 
   3.847983572048883*^9}, {3.847983607508451*^9, 
   3.847983611518873*^9}},ExpressionUUID->"ed4d4abb-87c1-4f3f-9d2d-\
574b4feef35f"],

Cell["\<\
From here, we introduce the boundary region, reg. This is the region of half \
spaces whose intersection define the Wigner-Seitz cell of the individual \
particle.\
\>", "Text",
 CellChangeTimes->{{3.8363788510777073`*^9, 3.836379016257597*^9}, {
  3.836379566083387*^9, 
  3.836379581775033*^9}},ExpressionUUID->"fd2a4a22-5b5d-4914-b9c0-\
d983c0f041ef"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"InequalityList", " ", "=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"1", "+", 
       RowBox[{"z", ".", 
        RowBox[{"w1", "[", 
         RowBox[{"[", "j", "]"}], "]"}]}]}], ">=", "0"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"EqualityList", " ", "=", "  ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"1", "+", 
       RowBox[{"z", ".", 
        RowBox[{"w1", "[", 
         RowBox[{"[", "j", "]"}], "]"}]}]}], "==", "0"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"reg", "=", " ", 
   RowBox[{"RegionIntersection", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"ImplicitRegion", "[", 
       RowBox[{
        RowBox[{"And", "@@", "InequalityList"}], ",", 
        RowBox[{"Evaluate", "[", "z", "]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"deltaDir1", "[", "j_", "]"}], ":=", 
  RowBox[{
   RowBox[{"FirstPosition", "[", 
    RowBox[{
     RowBox[{"w1", "[", 
      RowBox[{"[", "j", "]"}], "]"}], ",", 
     RowBox[{"_", "?", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"#", " ", "!=", " ", "0"}], " ", "&"}], ")"}]}]}], "]"}], "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.8354530449808693`*^9, 3.83545318172403*^9}, {
   3.835466684290997*^9, 3.8354669789223957`*^9}, 3.8354670192738953`*^9, {
   3.835467141655026*^9, 3.8354672329366827`*^9}, {3.835467266130377*^9, 
   3.835467490623328*^9}, {3.835467568513277*^9, 3.835467609264051*^9}, {
   3.835467845884493*^9, 3.835467859621139*^9}, 3.8354678931728287`*^9, 
   3.835467964683958*^9, {3.835468128981934*^9, 3.835468149404161*^9}, {
   3.835468346394278*^9, 3.835468364263399*^9}, {3.835468432382189*^9, 
   3.835468438329624*^9}, {3.8354688906661577`*^9, 3.8354688940224953`*^9}, {
   3.835468953723577*^9, 3.83546895416929*^9}, {3.835469010652217*^9, 
   3.8354690113065767`*^9}, {3.835471864869014*^9, 3.835471876518332*^9}, {
   3.835471944040312*^9, 3.835471944667301*^9}, {3.835472133568591*^9, 
   3.835472146458373*^9}, {3.8354722398527403`*^9, 3.8354722684825993`*^9}, {
   3.835472327189047*^9, 3.835472441444352*^9}, {3.835472477954636*^9, 
   3.8354725307475023`*^9}, {3.8354725873848658`*^9, 3.835472589858736*^9}, {
   3.835472677067501*^9, 3.835472677382126*^9}, {3.835472727006681*^9, 
   3.835472739451055*^9}, {3.835472788537559*^9, 3.835472803024736*^9}, {
   3.83547286249133*^9, 3.835472862651248*^9}, {3.8354729843857927`*^9, 
   3.835473003457418*^9}, {3.835473033608646*^9, 3.835473068149043*^9}, {
   3.83547330823114*^9, 3.835473311573303*^9}, {3.835473451426152*^9, 
   3.835473453654476*^9}, {3.835473491704762*^9, 3.8354734938963327`*^9}, {
   3.835473553080357*^9, 3.835473560735557*^9}, {3.835479086588212*^9, 
   3.835479121584784*^9}, {3.835479170024287*^9, 3.8354791786793613`*^9}, {
   3.835479451447424*^9, 3.83547945959903*^9}, {3.8354799750360327`*^9, 
   3.835480115178803*^9}, {3.835480174479991*^9, 3.835480286107811*^9}, {
   3.835480349910143*^9, 3.8354806066934767`*^9}, {3.8354807107012663`*^9, 
   3.835480803537777*^9}, {3.8354812902265244`*^9, 3.835481336218375*^9}, {
   3.8356939541238613`*^9, 3.835693960941977*^9}, {3.835694086037374*^9, 
   3.835694134340847*^9}, {3.8356941698249493`*^9, 3.835694231661469*^9}, {
   3.835694842649637*^9, 3.835694890402657*^9}, {3.835694984003853*^9, 
   3.835694997662994*^9}, {3.835695047950551*^9, 3.835695051447279*^9}, {
   3.835695095382547*^9, 3.835695116692688*^9}, {3.8356952452073927`*^9, 
   3.835695276050478*^9}, {3.835695315652401*^9, 3.835695331464056*^9}, {
   3.8356965536750593`*^9, 3.835696604284286*^9}, {3.8356966589138603`*^9, 
   3.835696661218536*^9}, {3.8356967113013268`*^9, 3.835696732079521*^9}, {
   3.835696783326808*^9, 3.8356967933364267`*^9}, {3.835696898555019*^9, 
   3.835696973046053*^9}, {3.8356972267731543`*^9, 3.835697312232521*^9}, 
   3.835697591279818*^9, {3.835698562349609*^9, 3.835698616797779*^9}, {
   3.835699142240032*^9, 3.835699216053561*^9}, 3.835699266938794*^9, 
   3.83570007047959*^9, {3.836378161311626*^9, 3.836378239342669*^9}, {
   3.836378333129324*^9, 3.836378335452346*^9}, 3.836378786123816*^9, 
   3.836378823281918*^9, {3.8363790362626963`*^9, 3.8363790530133457`*^9}, {
   3.83638373795601*^9, 3.836383741885558*^9}, {3.837000885971054*^9, 
   3.837000901557473*^9}, {3.83734827126632*^9, 3.837348281415062*^9}, {
   3.837758923307762*^9, 3.837758923694744*^9}, {3.837767205649397*^9, 
   3.8377672063946857`*^9}, 3.838801463415812*^9},
 CellLabel->
  "In[429]:=",ExpressionUUID->"afcdac5a-7e95-421c-9071-880540ea7e2e"],

Cell[TextData[{
 "From the manuscript, ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["I", "1", 
    RowBox[{"(", "0", ")"}]], TraditionalForm]],
  FormatType->TraditionalForm,ExpressionUUID->
  "1491c589-4c18-4d68-abc9-6b5abfb4d7f2"],
 ", for which we will use simplified notation Int10, is simply the volume of \
this region."
}], "Text",
 CellChangeTimes->{{3.8354530449808693`*^9, 3.83545318172403*^9}, {
   3.835466684290997*^9, 3.8354669789223957`*^9}, 3.8354670192738953`*^9, {
   3.835467141655026*^9, 3.8354672329366827`*^9}, {3.835467266130377*^9, 
   3.835467490623328*^9}, {3.835467568513277*^9, 3.835467609264051*^9}, {
   3.835467845884493*^9, 3.835467859621139*^9}, 3.8354678931728287`*^9, 
   3.835467964683958*^9, {3.835468128981934*^9, 3.835468149404161*^9}, {
   3.835468346394278*^9, 3.835468364263399*^9}, {3.835468432382189*^9, 
   3.835468438329624*^9}, {3.8354688906661577`*^9, 3.8354688940224953`*^9}, {
   3.835468953723577*^9, 3.83546895416929*^9}, {3.835469010652217*^9, 
   3.8354690113065767`*^9}, {3.835471864869014*^9, 3.835471876518332*^9}, {
   3.835471944040312*^9, 3.835471944667301*^9}, {3.835472133568591*^9, 
   3.835472146458373*^9}, {3.8354722398527403`*^9, 3.8354722684825993`*^9}, {
   3.835472327189047*^9, 3.835472441444352*^9}, {3.835472477954636*^9, 
   3.8354725307475023`*^9}, {3.8354725873848658`*^9, 3.835472589858736*^9}, {
   3.835472677067501*^9, 3.835472677382126*^9}, {3.835472727006681*^9, 
   3.835472739451055*^9}, {3.835472788537559*^9, 3.835472803024736*^9}, {
   3.83547286249133*^9, 3.835472862651248*^9}, {3.8354729843857927`*^9, 
   3.835473003457418*^9}, {3.835473033608646*^9, 3.835473068149043*^9}, {
   3.83547330823114*^9, 3.835473311573303*^9}, {3.835473451426152*^9, 
   3.835473453654476*^9}, {3.835473491704762*^9, 3.8354734938963327`*^9}, {
   3.835473553080357*^9, 3.835473560735557*^9}, {3.835479086588212*^9, 
   3.835479121584784*^9}, {3.835479170024287*^9, 3.8354791786793613`*^9}, {
   3.835479451447424*^9, 3.83547945959903*^9}, {3.8354799750360327`*^9, 
   3.835480115178803*^9}, {3.835480174479991*^9, 3.835480286107811*^9}, {
   3.835480349910143*^9, 3.8354806066934767`*^9}, {3.8354807107012663`*^9, 
   3.835480803537777*^9}, {3.8354812902265244`*^9, 3.835481336218375*^9}, {
   3.8356939541238613`*^9, 3.835693960941977*^9}, {3.835694086037374*^9, 
   3.835694134340847*^9}, {3.8356941698249493`*^9, 3.835694231661469*^9}, {
   3.835694842649637*^9, 3.835694890402657*^9}, {3.835694984003853*^9, 
   3.835694997662994*^9}, {3.835695047950551*^9, 3.835695051447279*^9}, {
   3.835695095382547*^9, 3.835695116692688*^9}, {3.8356952452073927`*^9, 
   3.835695276050478*^9}, {3.835695315652401*^9, 3.835695331464056*^9}, {
   3.8356965536750593`*^9, 3.835696604284286*^9}, {3.8356966589138603`*^9, 
   3.835696661218536*^9}, {3.8356967113013268`*^9, 3.835696732079521*^9}, {
   3.835696783326808*^9, 3.8356967933364267`*^9}, {3.835696898555019*^9, 
   3.835696973046053*^9}, {3.8356972267731543`*^9, 3.835697312232521*^9}, 
   3.835697591279818*^9, {3.835698562349609*^9, 3.835698616797779*^9}, {
   3.835699142240032*^9, 3.835699216053561*^9}, 3.835699266938794*^9, 
   3.83570007047959*^9, {3.836378161311626*^9, 3.836378239342669*^9}, {
   3.836378333129324*^9, 3.836378335452346*^9}, 3.836378786123816*^9, 
   3.836378823281918*^9, {3.8363790362626963`*^9, 3.836379072623633*^9}, {
   3.836379134480321*^9, 
   3.8363792486613894`*^9}},ExpressionUUID->"2f59cdff-a3bc-40ac-a5d6-\
992e9ad57b9f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Int10", " ", "=", " ", 
  RowBox[{"Integrate", "[", 
   RowBox[{"1", ",", 
    RowBox[{
     RowBox[{"Evaluate", "[", "z", "]"}], "\[Element]", "reg"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.8354530449808693`*^9, 3.83545318172403*^9}, {
   3.835466684290997*^9, 3.8354669789223957`*^9}, 3.8354670192738953`*^9, {
   3.835467141655026*^9, 3.8354672329366827`*^9}, {3.835467266130377*^9, 
   3.835467490623328*^9}, {3.835467568513277*^9, 3.835467609264051*^9}, {
   3.835467845884493*^9, 3.835467859621139*^9}, 3.8354678931728287`*^9, 
   3.835467964683958*^9, {3.835468128981934*^9, 3.835468149404161*^9}, {
   3.835468346394278*^9, 3.835468364263399*^9}, {3.835468432382189*^9, 
   3.835468438329624*^9}, {3.8354688906661577`*^9, 3.8354688940224953`*^9}, {
   3.835468953723577*^9, 3.83546895416929*^9}, {3.835469010652217*^9, 
   3.8354690113065767`*^9}, {3.835471864869014*^9, 3.835471876518332*^9}, {
   3.835471944040312*^9, 3.835471944667301*^9}, {3.835472133568591*^9, 
   3.835472146458373*^9}, {3.8354722398527403`*^9, 3.8354722684825993`*^9}, {
   3.835472327189047*^9, 3.835472441444352*^9}, {3.835472477954636*^9, 
   3.8354725307475023`*^9}, {3.8354725873848658`*^9, 3.835472589858736*^9}, {
   3.835472677067501*^9, 3.835472677382126*^9}, {3.835472727006681*^9, 
   3.835472739451055*^9}, {3.835472788537559*^9, 3.835472803024736*^9}, {
   3.83547286249133*^9, 3.835472862651248*^9}, {3.8354729843857927`*^9, 
   3.835473003457418*^9}, {3.835473033608646*^9, 3.835473068149043*^9}, {
   3.83547330823114*^9, 3.835473311573303*^9}, {3.835473451426152*^9, 
   3.835473453654476*^9}, {3.835473491704762*^9, 3.8354734938963327`*^9}, {
   3.835473553080357*^9, 3.835473560735557*^9}, {3.835479086588212*^9, 
   3.835479121584784*^9}, {3.835479170024287*^9, 3.8354791786793613`*^9}, {
   3.835479451447424*^9, 3.83547945959903*^9}, {3.8354799750360327`*^9, 
   3.835480115178803*^9}, {3.835480174479991*^9, 3.835480286107811*^9}, {
   3.835480349910143*^9, 3.8354806066934767`*^9}, {3.8354807107012663`*^9, 
   3.835480803537777*^9}, {3.8354812902265244`*^9, 3.835481336218375*^9}, {
   3.8356939541238613`*^9, 3.835693960941977*^9}, {3.835694086037374*^9, 
   3.835694134340847*^9}, {3.8356941698249493`*^9, 3.835694231661469*^9}, {
   3.835694842649637*^9, 3.835694890402657*^9}, {3.835694984003853*^9, 
   3.835694997662994*^9}, {3.835695047950551*^9, 3.835695051447279*^9}, {
   3.835695095382547*^9, 3.835695116692688*^9}, {3.8356952452073927`*^9, 
   3.835695276050478*^9}, {3.835695315652401*^9, 3.835695331464056*^9}, {
   3.8356965536750593`*^9, 3.835696604284286*^9}, {3.8356966589138603`*^9, 
   3.835696661218536*^9}, {3.8356967113013268`*^9, 3.835696732079521*^9}, {
   3.835696783326808*^9, 3.8356967933364267`*^9}, {3.835696898555019*^9, 
   3.835696973046053*^9}, {3.8356972267731543`*^9, 3.835697312232521*^9}, 
   3.835697591279818*^9, {3.835698562349609*^9, 3.835698616797779*^9}, {
   3.835699142240032*^9, 3.835699216053561*^9}, 3.835699266938794*^9, 
   3.83570007047959*^9, {3.836378161311626*^9, 3.836378239342669*^9}, {
   3.836378333129324*^9, 3.836378335452346*^9}, 3.836378786123816*^9, 
   3.836378823281918*^9, 3.8363790362626963`*^9, {3.836379263141631*^9, 
   3.836379303840784*^9}, {3.836383746400208*^9, 3.836383747537849*^9}, 
   3.836554808321061*^9, 3.83655582787652*^9, {3.837157057149569*^9, 
   3.837157059544064*^9}, 3.837864640259159*^9},
 CellLabel->
  "In[433]:=",ExpressionUUID->"c757d0e1-012f-413b-951c-083767d8eef5"],

Cell[BoxData[
 RowBox[{"4", " ", 
  SqrtBox["2"]}]], "Output",
 CellChangeTimes->{3.837864641424535*^9, 3.837866976708911*^9, 
  3.838075838496498*^9, 3.838097550649115*^9, 3.838105179661561*^9, 
  3.8381096314041767`*^9, 3.838109907144813*^9, 3.838112618425046*^9, 
  3.8381129147392683`*^9, 3.838113278587818*^9, 3.838114628830847*^9, 
  3.838116407832329*^9, 3.8381166596171017`*^9, 3.838117365266058*^9, 
  3.8381184706647177`*^9, 3.838119045156095*^9, 3.8381358363081827`*^9, 
  3.838196542076193*^9, 3.838208172337509*^9, 3.838239067358452*^9, 
  3.838239232051914*^9, 3.838239500500536*^9, 3.838239797489135*^9, 
  3.838243632087592*^9, 3.8382803459979887`*^9, 3.8382805097082663`*^9, 
  3.838290480357918*^9, 3.8382923419207783`*^9, 3.838398722746541*^9, 
  3.8387017476807833`*^9, 3.838801469246336*^9, 3.839572297138073*^9, 
  3.839573956450572*^9},
 CellLabel->
  "Out[433]=",ExpressionUUID->"9c2e26af-78dd-4c81-b610-dc67e2824a78"]
}, Open  ]],

Cell[TextData[{
 "This is enough to calculate ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "1"], TraditionalForm]],
  FormatType->TraditionalForm,ExpressionUUID->
  "ca4c505d-97b8-494c-baf8-d84a5abdd0b1"],
 " and also the first order approximation of C (second line)"
}], "Text",
 CellChangeTimes->{{3.8354530449808693`*^9, 3.83545318172403*^9}, {
   3.835466684290997*^9, 3.8354669789223957`*^9}, 3.8354670192738953`*^9, {
   3.835467141655026*^9, 3.8354672329366827`*^9}, {3.835467266130377*^9, 
   3.835467490623328*^9}, {3.835467568513277*^9, 3.835467609264051*^9}, {
   3.835467845884493*^9, 3.835467859621139*^9}, 3.8354678931728287`*^9, 
   3.835467964683958*^9, {3.835468128981934*^9, 3.835468149404161*^9}, {
   3.835468346394278*^9, 3.835468364263399*^9}, {3.835468432382189*^9, 
   3.835468438329624*^9}, {3.8354688906661577`*^9, 3.8354688940224953`*^9}, {
   3.835468953723577*^9, 3.83546895416929*^9}, {3.835469010652217*^9, 
   3.8354690113065767`*^9}, {3.835471864869014*^9, 3.835471876518332*^9}, {
   3.835471944040312*^9, 3.835471944667301*^9}, {3.835472133568591*^9, 
   3.835472146458373*^9}, {3.8354722398527403`*^9, 3.8354722684825993`*^9}, {
   3.835472327189047*^9, 3.835472441444352*^9}, {3.835472477954636*^9, 
   3.8354725307475023`*^9}, {3.8354725873848658`*^9, 3.835472589858736*^9}, {
   3.835472677067501*^9, 3.835472677382126*^9}, {3.835472727006681*^9, 
   3.835472739451055*^9}, {3.835472788537559*^9, 3.835472803024736*^9}, {
   3.83547286249133*^9, 3.835472862651248*^9}, {3.8354729843857927`*^9, 
   3.835473003457418*^9}, {3.835473033608646*^9, 3.835473068149043*^9}, {
   3.83547330823114*^9, 3.835473311573303*^9}, {3.835473451426152*^9, 
   3.835473453654476*^9}, {3.835473491704762*^9, 3.8354734938963327`*^9}, {
   3.835473553080357*^9, 3.835473560735557*^9}, {3.835479086588212*^9, 
   3.835479121584784*^9}, {3.835479170024287*^9, 3.8354791786793613`*^9}, {
   3.835479451447424*^9, 3.83547945959903*^9}, {3.8354799750360327`*^9, 
   3.835480115178803*^9}, {3.835480174479991*^9, 3.835480286107811*^9}, {
   3.835480349910143*^9, 3.8354806066934767`*^9}, {3.8354807107012663`*^9, 
   3.835480803537777*^9}, {3.8354812902265244`*^9, 3.835481336218375*^9}, {
   3.8356939541238613`*^9, 3.835693960941977*^9}, {3.835694086037374*^9, 
   3.835694134340847*^9}, {3.8356941698249493`*^9, 3.835694231661469*^9}, {
   3.835694842649637*^9, 3.835694890402657*^9}, {3.835694984003853*^9, 
   3.835694997662994*^9}, {3.835695047950551*^9, 3.835695051447279*^9}, {
   3.835695095382547*^9, 3.835695116692688*^9}, {3.8356952452073927`*^9, 
   3.835695276050478*^9}, {3.835695315652401*^9, 3.835695331464056*^9}, {
   3.8356965536750593`*^9, 3.835696604284286*^9}, {3.8356966589138603`*^9, 
   3.835696661218536*^9}, {3.8356967113013268`*^9, 3.835696732079521*^9}, {
   3.835696783326808*^9, 3.8356967933364267`*^9}, {3.835696898555019*^9, 
   3.835696973046053*^9}, {3.8356972267731543`*^9, 3.835697312232521*^9}, 
   3.835697591279818*^9, {3.835698562349609*^9, 3.835698616797779*^9}, {
   3.835699142240032*^9, 3.835699216053561*^9}, 3.835699266938794*^9, 
   3.83570007047959*^9, {3.836378161311626*^9, 3.836378239342669*^9}, {
   3.836378333129324*^9, 3.836378335452346*^9}, 3.836378786123816*^9, 
   3.836378823281918*^9, 3.8363790362626963`*^9, {3.836379263141631*^9, 
   3.836379328452557*^9}, {3.83638004727252*^9, 3.836380067630355*^9}, {
   3.8363803389880257`*^9, 3.836380391384088*^9}, {3.8363804287178926`*^9, 
   3.8363804316077547`*^9}},ExpressionUUID->"75b648bd-ac89-4adb-bd1e-\
75d1d957badd"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["C", "1"], " ", "=", " ", 
   RowBox[{"-", 
    RowBox[{"Log", "[", "Int10", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  RowBox[{
   SubscriptBox["C", "1"], " ", "+", " ", 
   RowBox[{"d", "*", 
    RowBox[{"Log", "[", "d", "]"}]}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.8363793425378523`*^9, 3.836379368616913*^9}, {
   3.836379734897592*^9, 3.836379768973117*^9}, {3.836379807430863*^9, 
   3.836379822737446*^9}, 3.8363800394024754`*^9, {3.83638007243192*^9, 
   3.836380101381691*^9}, {3.836380405367229*^9, 3.836380411933976*^9}, {
   3.836381105369935*^9, 3.8363811074684887`*^9}, {3.836383335276701*^9, 
   3.836383347860034*^9}, {3.8365547946788197`*^9, 3.8365548066064663`*^9}},
 CellLabel->
  "In[434]:=",ExpressionUUID->"70bb00f4-1d36-4691-a0de-c96cb8cc4bc6"],

Cell[BoxData["1.5629689146044659`"], "Output",
 CellChangeTimes->{
  3.83638041359511*^9, 3.836380758676515*^9, 3.836381117660006*^9, 
   3.836383506916724*^9, 3.8363837672554398`*^9, {3.836383801813901*^9, 
   3.836383829139979*^9}, 3.836384814967176*^9, 3.8365455723119297`*^9, 
   3.836555831199955*^9, 3.836829743371141*^9, 3.836942409330461*^9, 
   3.837000908573763*^9, 3.837157067523567*^9, 3.8371642523815002`*^9, 
   3.837348408413522*^9, 3.8373500469147377`*^9, 3.837350620487225*^9, 
   3.8373554078585978`*^9, 3.837356904545061*^9, 3.837508800475416*^9, 
   3.837509673412846*^9, 3.8375098576979713`*^9, 3.837759261835998*^9, {
   3.837767209094831*^9, 3.8377672230409927`*^9}, 3.837768127719103*^9, 
   3.837768360815436*^9, 3.837768462002862*^9, 3.837768525234685*^9, 
   3.83777177351099*^9, 3.83786086868478*^9, 3.837861211128234*^9, 
   3.837866978782144*^9, 3.838075841125657*^9, 3.838097553646899*^9, 
   3.838109632728397*^9, 3.838109907939974*^9, 3.838112620245216*^9, 
   3.838112916554942*^9, 3.8381132807814207`*^9, 3.8381146302361937`*^9, 
   3.838116409388208*^9, 3.8381166609921293`*^9, 3.838117366821458*^9, 
   3.838118471431546*^9, 3.838119045347066*^9, 3.838135838556707*^9, 
   3.838196543899486*^9, 3.8382081746641493`*^9, 3.8382390697398787`*^9, 
   3.838239234527729*^9, 3.8382395020005817`*^9, 3.838239800010791*^9, 
   3.8382436346512003`*^9, 3.8382803489223137`*^9, 3.838280511811817*^9, 
   3.838290483239243*^9, 3.8382923444358053`*^9, 3.838398724033265*^9, 
   3.838701750858718*^9, 3.8388014705355043`*^9, 3.839573958265484*^9},
 CellLabel->
  "Out[435]=",ExpressionUUID->"e1073f11-9568-4e1e-9ee0-0eb67703a781"]
}, Open  ]],

Cell[TextData[{
 "The value for C here aligns perfectly with that calculated by Rudd, et al., \
J. Chem. Phys. (1968). Here, we note  however, that there is an oddity in the \
transcription of values of C, wherein the free volume term includes the \
dLog[d] term in their tables. This is nevertheless resolved as they write out \
their expansions. \n\nThe calculation of D1 is straightforward. It requires \
first the evaluation of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["I", "1", 
    RowBox[{"(", "1", ")"}]], TraditionalForm]],
  FormatType->TraditionalForm,ExpressionUUID->
  "ed4f70e7-5a14-4fe0-aa3c-c68ce2a3cd05"],
 ", here called Int11, which contains a sum of functions evaluated on the \
surface of the cell"
}], "Text",
 CellChangeTimes->{{3.8363793425378523`*^9, 3.836379368616913*^9}, {
   3.836379734897592*^9, 3.836379768973117*^9}, {3.836379807430863*^9, 
   3.836379822737446*^9}, 3.8363800394024754`*^9, {3.83638007243192*^9, 
   3.836380156488763*^9}, {3.8363801904763517`*^9, 3.836380201962661*^9}, {
   3.836380248341538*^9, 3.836380251041834*^9}, {3.83638054171278*^9, 
   3.836380564069642*^9}, {3.836380614827322*^9, 3.83638066777075*^9}, {
   3.836380786663465*^9, 3.836380814666253*^9}, {3.8363810260715427`*^9, 
   3.83638108976266*^9}, {3.836383522428784*^9, 3.836383548422894*^9}, {
   3.8363838417038393`*^9, 
   3.836383848084209*^9}},ExpressionUUID->"523d62d6-2e5a-4278-a2d4-\
75e1524d1756"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"deltaIntegral", "[", 
   RowBox[{
   "polyIn_", ",", "ineqIn_", ",", "eqIn_", ",", "varsIn_", ",", 
    "indexPos_"}], "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"solution", " ", "=", " ", 
     RowBox[{"Solve", "[", 
      RowBox[{
       RowBox[{"eqIn", "[", 
        RowBox[{"[", "indexPos", "]"}], "]"}], ",", "Reals"}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Length", "[", "solution", "]"}], "==", "0"}], ",", 
      RowBox[{"Return", "[", "0", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
    
    RowBox[{"subs", " ", "=", "  ", 
     RowBox[{"solution", "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"intVar", " ", "=", " ", 
     RowBox[{"subs", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"coeff", " ", "=", " ", 
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"CoefficientArrays", "[", 
        RowBox[{
         RowBox[{"eqIn", "[", 
          RowBox[{"[", "indexPos", "]"}], "]"}], ",", "intVar"}], "]"}], 
       "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"newVars", "=", 
     RowBox[{"Delete", "[", 
      RowBox[{"varsIn", ",", 
       RowBox[{"Position", "[", 
        RowBox[{"varsIn", ",", "intVar"}], "]"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"newIneq", "=", 
     RowBox[{"ineqIn", "//.", "subs"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"newPoly", " ", "=", " ", 
     RowBox[{
      FractionBox["polyIn", 
       RowBox[{"Abs", "[", "coeff", "]"}]], "//.", "subs"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Return", "[", 
     RowBox[{"Integrate", "[", 
      RowBox[{
       RowBox[{"FullSimplify", "[", "newPoly", "]"}], ",", 
       RowBox[{
        RowBox[{"Evaluate", "[", "newVars", "]"}], "\[Element]", 
        RowBox[{"ImplicitRegion", "[", 
         RowBox[{
          RowBox[{"And", "@@", "newIneq"}], ",", 
          RowBox[{"Evaluate", "[", "newVars", "]"}]}], "]"}]}]}], "]"}], 
     "]"}], ";"}], "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
 RowBox[{"Int11", " ", "=", " ", 
  RowBox[{"Total", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"deltaIntegral", "[", 
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], "*", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Total", "[", 
           SuperscriptBox["z", "2"], "]"}], " ", "-", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"w1", "[", 
              RowBox[{"[", "j", "]"}], "]"}], ".", "z"}], ")"}], "2"]}], 
         ")"}]}], ",", "InequalityList", ",", "EqualityList", ",", "z", ",", 
       "j"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJwdzk0og3EABvCZj0TyNVzI1Cw0qVFzmbyXmZlsSyLR0tRI06aRHV61dhiN
1pqMlslqFOW9TNHUYj5au1gulpR8tNabLFwkeZ//4el3eXp6GiZmdZN8Ho9X
xwV66j6dmnWWyitjVuBAZdsRHA6YiJnHcBT6n8cuYVQQTJJeavAOqoKpfC1n
y+peAQwKp7Q6TutwlkgrRMrmDZaqOJzrg5mq0kiWU6jUEped+y9QsFjyCmvs
tfFGH0u1iRQJ6Cqj7+D21dcDvGUMT9BxnHyDPe7IO5TkRH/gyknsD0baC8vF
6F2pq6GRX9QE/aH+VmhQ6oi2jpgU7oaSxNoluxx6QhoNDM/0DsLvgGQE5sUu
xqFSKtPDOPNrgENptwkKcs8W4Ol5ijj/UU9Dd1y/BH/NAZuLs2v0hhhRpxNb
2Kuk76HYZzb6Oe0q2TQUb/J+oEVuJXYXeyev8S/zTHSYdmSWA5Zi1ryd8B9e
sevd
  "],ExpressionUUID->"21f53e28-e625-4ab3-8fd2-a2f763bfef1a"],

Cell[BoxData[
 FractionBox["3", 
  SqrtBox["2"]]], "Output",
 CellChangeTimes->{{3.8378665972660513`*^9, 3.8378666278091307`*^9}, {
   3.837866680687613*^9, 3.837866698391239*^9}, {3.837866745254961*^9, 
   3.83786675442268*^9}, 3.8378668754916*^9, {3.837866956826968*^9, 
   3.837866994681301*^9}, {3.838075847271265*^9, 3.838075863181534*^9}, 
   3.838097557013117*^9, 3.838105185198801*^9, {3.8381052275812902`*^9, 
   3.8381052926229877`*^9}, 3.8381054824703197`*^9, {3.8381096393198357`*^9, 
   3.8381096523335743`*^9}, 3.838109913464663*^9, 3.838112627265162*^9, 
   3.838112922085169*^9, 3.838113287962759*^9, 3.838114636145505*^9, 
   3.838116415290173*^9, 3.8381166665896387`*^9, 3.83811737305296*^9, 
   3.838118477984664*^9, 3.838119049953199*^9, 3.838135843618662*^9, 
   3.838196549496751*^9, 3.838208178819998*^9, 3.8382390751011*^9, 
   3.838239239066897*^9, 3.838239507673513*^9, 3.83823980325813*^9, 
   3.8382436385998173`*^9, 3.838280517831665*^9, 3.8382904893959*^9, 
   3.838292349333701*^9, 3.838303168332025*^9, 3.8383987283962297`*^9, 
   3.838701757798213*^9, 3.838801474186561*^9},
 CellLabel->
  "Out[186]=",ExpressionUUID->"2318c518-f08c-4865-b74b-af090999c8ae"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  SubscriptBox["D", "1"], " ", "=", " ", 
  RowBox[{
   RowBox[{"-", "Int11"}], "/", "Int10"}]}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  RowBox[{
   FractionBox[
    SubscriptBox["D", "1"], "d"], "+", 
   FractionBox[
    RowBox[{"(", 
     RowBox[{"d", "-", "1"}], ")"}], "2"]}], "]"}]}], "Input",
 CellChangeTimes->{{3.8363802315973597`*^9, 3.836380245501955*^9}, {
  3.836380537471107*^9, 3.836380539456951*^9}, {3.836380677683461*^9, 
  3.8363807149340343`*^9}, {3.836380821096283*^9, 3.8363809522528343`*^9}, {
  3.8363833195111437`*^9, 3.8363833286656857`*^9}},
 CellLabel->
  "In[187]:=",ExpressionUUID->"9434cd58-094d-43be-993d-b036c028967d"],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["3", "8"]}]], "Output",
 CellChangeTimes->{
  3.836380865226961*^9, {3.83638095413165*^9, 3.8363809732896757`*^9}, 
   3.836381120901733*^9, 3.8363812457214746`*^9, 3.8363835814875917`*^9, 
   3.836383896681538*^9, 3.83638482062386*^9, 3.836545582854424*^9, 
   3.83655583753398*^9, 3.8368297484643717`*^9, 3.836942416409052*^9, 
   3.837157076574606*^9, 3.837348451722135*^9, 3.837350052060892*^9, 
   3.8373554138639927`*^9, 3.837356910689098*^9, 3.8375088331058702`*^9, 
   3.837509681787509*^9, 3.837509866687954*^9, 3.837759275875227*^9, 
   3.837767228773929*^9, 3.83776813324963*^9, 3.83776836715794*^9, 
   3.8377684678289747`*^9, 3.837768531218664*^9, 3.837771779491207*^9, 
   3.8378608804746532`*^9, 3.837861217145343*^9, 3.837867225876005*^9, {
   3.83807585189074*^9, 3.8380758655278397`*^9}, 3.838097558613303*^9, 
   3.838109654503748*^9, 3.838109915139298*^9, 3.838112630105337*^9, 
   3.8381129243128157`*^9, 3.838113289897244*^9, 3.838114647831053*^9, 
   3.838116418048801*^9, 3.8381166679697742`*^9, 3.838117376669475*^9, 
   3.838118481007062*^9, 3.83811905224716*^9, 3.83813584476858*^9, 
   3.838208185003091*^9, 3.83823907684969*^9, 3.8382392428010397`*^9, 
   3.8382395121085043`*^9, 3.838239805741321*^9, 3.8382436399745493`*^9, 
   3.8382805210346212`*^9, 3.83829049275552*^9, 3.838292351774638*^9, 
   3.838398731304016*^9, 3.8387017602095623`*^9, 3.838801476921875*^9},
 CellLabel->
  "Out[187]=",ExpressionUUID->"1907f97b-cd48-4684-9e5e-e9d70a5f49d3"],

Cell[BoxData["0.875`"], "Output",
 CellChangeTimes->{
  3.836380865226961*^9, {3.83638095413165*^9, 3.8363809732896757`*^9}, 
   3.836381120901733*^9, 3.8363812457214746`*^9, 3.8363835814875917`*^9, 
   3.836383896681538*^9, 3.83638482062386*^9, 3.836545582854424*^9, 
   3.83655583753398*^9, 3.8368297484643717`*^9, 3.836942416409052*^9, 
   3.837157076574606*^9, 3.837348451722135*^9, 3.837350052060892*^9, 
   3.8373554138639927`*^9, 3.837356910689098*^9, 3.8375088331058702`*^9, 
   3.837509681787509*^9, 3.837509866687954*^9, 3.837759275875227*^9, 
   3.837767228773929*^9, 3.83776813324963*^9, 3.83776836715794*^9, 
   3.8377684678289747`*^9, 3.837768531218664*^9, 3.837771779491207*^9, 
   3.8378608804746532`*^9, 3.837861217145343*^9, 3.837867225876005*^9, {
   3.83807585189074*^9, 3.8380758655278397`*^9}, 3.838097558613303*^9, 
   3.838109654503748*^9, 3.838109915139298*^9, 3.838112630105337*^9, 
   3.8381129243128157`*^9, 3.838113289897244*^9, 3.838114647831053*^9, 
   3.838116418048801*^9, 3.8381166679697742`*^9, 3.838117376669475*^9, 
   3.838118481007062*^9, 3.83811905224716*^9, 3.83813584476858*^9, 
   3.838208185003091*^9, 3.83823907684969*^9, 3.8382392428010397`*^9, 
   3.8382395121085043`*^9, 3.838239805741321*^9, 3.8382436399745493`*^9, 
   3.8382805210346212`*^9, 3.83829049275552*^9, 3.838292351774638*^9, 
   3.838398731304016*^9, 3.8387017602095623`*^9, 3.838801476993431*^9},
 CellLabel->
  "Out[188]=",ExpressionUUID->"85d4e7aa-bb6b-4778-8a38-b50f822b5d41"]
}, Open  ]],

Cell["\<\
Again, these are exactly the values reported in Rudd, et al., J. Chem. Phys. \
(1968). This concludes the known calculations to first order. It is also \
worth noting that simplifications can be made, such as the substitution \
z.w[[j]] = -1\
\>", "Text",
 CellChangeTimes->{{3.836381128856174*^9, 3.836381201686969*^9}, {
  3.836381264647275*^9, 
  3.8363813000671473`*^9}},ExpressionUUID->"9d4b628d-8664-447f-8950-\
39051f6e1924"],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{
   RowBox[{"Int11", " ", "=", " ", 
    RowBox[{"Total", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Integrate", "[", 
        RowBox[{
         RowBox[{
          FractionBox["1", "2"], "*", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Total", "[", 
             SuperscriptBox["z", "2"], "]"}], "-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"z", ".", 
               RowBox[{"w1", "[", 
                RowBox[{"[", "j", "]"}], "]"}]}], ")"}], "2"]}], ")"}]}], ",", 
         RowBox[{
          RowBox[{"Evaluate", "[", "z", "]"}], "\[Element]", 
          RowBox[{"RegionIntersection", "[", 
           RowBox[{"reg", ",", 
            RowBox[{"ImplicitRegion", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"1", "+", 
                RowBox[{"z", ".", 
                 RowBox[{"w1", "[", 
                  RowBox[{"[", "j", "]"}], "]"}]}]}], "\[Equal]", "0"}], ",", 
              
              RowBox[{"Evaluate", "[", "z", "]"}]}], "]"}]}], "]"}]}]}], 
        "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}], "]"}]}], ";", 
   "\[IndentingNewLine]", 
   RowBox[{
    SubscriptBox["D", "1"], " ", "=", " ", 
    RowBox[{
     RowBox[{"-", "Int11"}], "/", "Int10"}]}]}], "*)"}]], "Input",
 CellChangeTimes->{{3.836381128856174*^9, 3.836381201686969*^9}, {
   3.836381264647275*^9, 3.83638135781781*^9}, {3.836383359717579*^9, 
   3.8363833627809258`*^9}, {3.836383905762403*^9, 3.836383910124175*^9}, {
   3.837348456282859*^9, 3.8373484773816853`*^9}, 3.8381096622023*^9},
 CellLabel->"In[23]:=",ExpressionUUID->"a51883d4-daf5-47a4-8572-0c332014f1d4"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "oneSide", "]"}], ";", 
  RowBox[{
   RowBox[{"oneSide", "[", "e_", "]"}], ":=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"#", "-", 
        RowBox[{"Last", "[", "e", "]"}]}], ")"}], "&"}], ",", "e"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"integrationPartsDoubleDelta", "[", 
   RowBox[{"j_", ",", "k_"}], "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"deltaCoeffIn", "[", "ineq_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"solution", " ", "=", " ", 
        RowBox[{"Solve", "[", 
         RowBox[{"ineq", ",", "Reals"}], "]"}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "solution", "]"}], "==", "0"}], ",", 
         RowBox[{"Return", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", "}"}], ",", 
            RowBox[{"{", "}"}], ",", "0"}], "}"}], "]"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"subs", " ", "=", " ", 
        RowBox[{"solution", "[", 
         RowBox[{"[", 
          RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"intVar", " ", "=", " ", 
        RowBox[{"subs", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"coeffs", " ", "=", " ", 
        RowBox[{
         RowBox[{"Normal", "[", 
          RowBox[{"CoefficientArrays", "[", 
           RowBox[{"ineq", ",", "intVar"}], "]"}], "]"}], "[", 
         RowBox[{"[", 
          RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"subs", ",", "intVar", ",", "coeffs"}], "}"}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"sub", ",", "inVar", ",", "coeff"}], "}"}], " ", "=", " ", 
     RowBox[{"deltaCoeffIn", "[", 
      RowBox[{"EqualityList", "[", 
       RowBox[{"[", "j", "]"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"EqualityList", "[", 
          RowBox[{"[", "k", "]"}], "]"}], "//.", "sub"}], ")"}], "===", 
       "False"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"{", "}"}], ",", "InequalityList"}], "}"}], "]"}]}], 
     "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"sub2", ",", "inVar2", ",", "coeff2"}], "}"}], " ", "=", " ", 
     RowBox[{"deltaCoeffIn", "[", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"EqualityList", "[", 
         RowBox[{"[", "k", "]"}], "]"}], "//.", "sub"}], ")"}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"coeff2", "==", "0"}], ",", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"{", "}"}], ",", "InequalityList"}], "}"}], "]"}]}], "]"}], 
    ";", "\[IndentingNewLine]", 
    RowBox[{"poly", " ", "=", " ", 
     RowBox[{
      FractionBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           FractionBox["1", "2"], "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Total", "[", 
              SuperscriptBox["z", "2"], "]"}], " ", "-", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{"w1", "[", 
                 RowBox[{"[", "j", "]"}], "]"}], ".", "z"}], ")"}], "2"]}], 
            ")"}]}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{
           FractionBox["1", "2"], "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Total", "[", 
              SuperscriptBox["z", "2"], "]"}], " ", "-", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{"w1", "[", 
                 RowBox[{"[", "k", "]"}], "]"}], ".", "z"}], ")"}], "2"]}], 
            ")"}]}], ")"}]}], ")"}], 
       RowBox[{
        RowBox[{"Abs", "[", "coeff", "]"}], "*", 
        RowBox[{"Abs", "[", "coeff2", "]"}]}]], "//.", 
      RowBox[{"{", 
       RowBox[{"sub", ",", "sub2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"vars", "=", 
     RowBox[{"Delete", "[", 
      RowBox[{"z", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"Position", "[", 
           RowBox[{"z", ",", "inVar"}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Position", "[", 
           RowBox[{"z", ",", "inVar2"}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], "}"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"ineqList", " ", "=", " ", 
     RowBox[{"InequalityList", "//.", 
      RowBox[{"{", 
       RowBox[{"sub", ",", "sub2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vars", "]"}], ">", "0"}], " ", "&&", " ", 
       RowBox[{"!", 
        RowBox[{"ineqList", "===", "0"}]}], " ", "&&", " ", 
       RowBox[{
        RowBox[{"RegionDimension", "[", 
         RowBox[{"ImplicitRegion", "[", 
          RowBox[{"ineqList", ",", 
           RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}], "]"}], ">", 
        "0"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"poly", ",", "vars", ",", "ineqList"}], "}"}], "]"}], ";"}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"0", ",", 
          RowBox[{"{", "}"}], ",", "ineqList"}], "}"}], "]"}], ";"}]}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
   ")"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"eachEDoubleDeltaInt", "[", 
   RowBox[{"j_", ",", "k_"}], "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{"poly", ",", "vars", ",", "ineqList"}], "}"}], " ", "=", " ", 
     RowBox[{"integrationPartsDoubleDelta", "[", 
      RowBox[{"j", ",", "k"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", " ", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vars", "]"}], ">", "0"}], " ", "&&", " ", 
       RowBox[{
        RowBox[{"Length", "[", "ineqList", "]"}], ">", "0"}], " ", "&&", " ", 
       
       RowBox[{
        RowBox[{"RegionDimension", "[", 
         RowBox[{"ImplicitRegion", "[", 
          RowBox[{"ineqList", ",", 
           RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}], "]"}], "==", 
        RowBox[{"d", "-", "2"}]}]}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Return", "[", 
        RowBox[{"Integrate", "[", 
         RowBox[{"poly", ",", 
          RowBox[{"vars", "\[Element]", 
           RowBox[{"ImplicitRegion", "[", 
            RowBox[{
             RowBox[{"And", "@@", "ineqList"}], ",", 
             RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}]}]}], "]"}], 
        "]"}], ";"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "0", "]"}]}], "]"}], ";"}], 
   "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"integrationPartsSingleDelta", "[", "j_", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"solution", " ", "=", " ", 
     RowBox[{
      RowBox[{"Solve", "[", 
       RowBox[{
        RowBox[{"EqualityList", "[", 
         RowBox[{"[", "j", "]"}], "]"}], ",", 
        RowBox[{"z", "[", 
         RowBox[{"[", 
          RowBox[{"deltaDir1", "[", "j", "]"}], "]"}], "]"}], ",", "Reals"}], 
       "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"poly", " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"w1", "[", 
             RowBox[{"[", "j", "]"}], "]"}], ".", "z"}]}], "*", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Total", "[", 
             SuperscriptBox["z", "2"], "]"}], " ", "-", " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{"w1", "[", 
                RowBox[{"[", "j", "]"}], "]"}], ".", "z"}], ")"}], "2"]}], 
           ")"}]}], 
         RowBox[{"Abs", "[", 
          RowBox[{"w1", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", 
             RowBox[{"deltaDir1", "[", "j", "]"}]}], "]"}], "]"}], "]"}]], 
        "-", 
        FractionBox[
         RowBox[{"D", "[", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FractionBox["1", "2"], "*", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"Total", "[", 
                 SuperscriptBox["z", "2"], "]"}], " ", "-", " ", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"w1", "[", 
                    RowBox[{"[", "j", "]"}], "]"}], ".", "z"}], ")"}], "2"]}],
                ")"}]}], ")"}], "2"], ",", 
           RowBox[{"z", "[", 
            RowBox[{"[", 
             RowBox[{"deltaDir1", "[", "j", "]"}], "]"}], "]"}]}], "]"}], 
         RowBox[{"Abs", "[", 
          RowBox[{"w1", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", 
             RowBox[{"deltaDir1", "[", "j", "]"}]}], "]"}], "]"}], "]"}]]}], 
       ")"}], "/.", "solution"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"vars", " ", "=", " ", 
     RowBox[{"Delete", "[", 
      RowBox[{"z", ",", 
       RowBox[{"deltaDir1", "[", "j", "]"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"ineqList", " ", "=", " ", 
     RowBox[{
      RowBox[{"InequalityList", "[", 
       RowBox[{"[", 
        RowBox[{"Delete", "[", 
         RowBox[{
          RowBox[{"Range", "[", 
           RowBox[{"1", ",", "sites"}], "]"}], ",", "j"}], "]"}], "]"}], 
       "]"}], "/.", "solution"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"Return", "[", 
     RowBox[{"{", 
      RowBox[{"poly", ",", "vars", ",", "ineqList"}], "}"}], "]"}], ";"}], 
   "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"eachESingleDeltaInt", "[", "j_", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{"poly", ",", "vars", ",", "ineqList"}], "}"}], "=", 
     RowBox[{"integrationPartsSingleDelta", "[", "j", "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"If", " ", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vars", "]"}], ">", "0"}], " ", "&&", " ", 
       RowBox[{
        RowBox[{"Length", "[", "ineqList", "]"}], ">", "0"}], " ", "&&", " ", 
       
       RowBox[{
        RowBox[{"RegionDimension", "[", 
         RowBox[{"ImplicitRegion", "[", 
          RowBox[{"ineqList", ",", 
           RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}], "]"}], "==", 
        RowBox[{"d", "-", "1"}]}]}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Return", "[", 
        RowBox[{"Integrate", "[", 
         RowBox[{"poly", ",", 
          RowBox[{"vars", "\[Element]", 
           RowBox[{"ImplicitRegion", "[", 
            RowBox[{
             RowBox[{"And", "@@", 
              RowBox[{"(", "ineqList", ")"}]}], ",", 
             RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}]}]}], "]"}], 
        "]"}], ";"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "0", "]"}]}], "]"}], ";"}], 
   "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"subInequality", "[", "ineq_", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"toDelete", " ", "=", " ", 
     RowBox[{"Position", "[", 
      RowBox[{"ineq", ",", "True"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"Return", "[", 
     RowBox[{"Delete", "[", 
      RowBox[{"ineq", ",", "toDelete"}], "]"}], "]"}], ";"}], 
   "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"HRepFile", " ", "[", 
    RowBox[{"ineqList_", ",", "vars_"}], "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vars", "]"}], ">", "0"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"coeffs", " ", "=", " ", 
         RowBox[{"Normal", "[", 
          RowBox[{"getLinearForm", "[", 
           RowBox[{
            RowBox[{"subInequality", "[", "ineqList", "]"}], ",", 
            RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", 
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"Transpose", "[", 
            RowBox[{"{", 
             RowBox[{"coeffs", "[", 
              RowBox[{"[", "2", "]"}], "]"}], "}"}], "]"}], ",", 
           RowBox[{"-", 
            RowBox[{"coeffs", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], ",", "2"}], "]"}], "]"}], 
        ";"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{"{", "}"}], "]"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
    ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"polyExp", "[", 
    RowBox[{"poly_", ",", "vars_"}], "]"}], ":=", 
   RowBox[{"CoefficientRules", "[", 
    RowBox[{"poly", ",", "vars"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"evenPoly", "[", 
    RowBox[{"p_", ",", "var_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"p", "/.", 
        RowBox[{"var", "->", "x"}]}], ")"}], "-", 
      RowBox[{"(", 
       RowBox[{"p", "/.", 
        RowBox[{"var", "->", 
         RowBox[{"-", "x"}]}]}], ")"}]}], "]"}], "===", "0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"polyStringWrite", "[", 
     RowBox[{"poly_", ",", "vars_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"polySet", " ", "=", " ", 
       RowBox[{"polyExp", "[", 
        RowBox[{"poly", ",", "vars"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"negSet", " ", "=", " ", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"polySet", "[", 
             RowBox[{"[", 
              RowBox[{";;", ",", "2"}], "]"}], "]"}], "//", "N"}], ")"}], ",", 
          RowBox[{"_", "?", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"#", "<", "0"}], "&"}], ")"}]}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"posSet", " ", "=", " ", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"polySet", "[", 
             RowBox[{"[", 
              RowBox[{";;", ",", "2"}], "]"}], "]"}], "//", "N"}], ")"}], ",", 
          RowBox[{"_", "?", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"#", ">=", "0"}], "&"}], ")"}]}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "posSet", "]"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"posSet", " ", "=", " ", 
          RowBox[{"polySet", "[", 
           RowBox[{"[", "posSet", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         
         RowBox[{"posString", " ", "=", " ", "\"\<[\>\""}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"m", "=", "1"}], ",", 
           RowBox[{"m", "<=", 
            RowBox[{"Length", "[", "posSet", "]"}]}], ",", 
           RowBox[{"m", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"posString", " ", "=", " ", 
             RowBox[{"posString", "<>", "\"\<[\>\"", "<>", 
              RowBox[{"ToString", "[", 
               RowBox[{"Round", "[", 
                RowBox[{
                 SuperscriptBox["10", "10"], "*", 
                 RowBox[{"posSet", "[", 
                  RowBox[{"[", 
                   RowBox[{"m", ",", "2"}], "]"}], "]"}]}], "]"}], "]"}], 
              "<>", "\"\<,[\>\"", "<>", 
              RowBox[{"StringRiffle", "[", 
               RowBox[{
                RowBox[{"posSet", "[", 
                 RowBox[{"[", 
                  RowBox[{"m", ",", "1"}], "]"}], "]"}], ",", "\"\<,\>\""}], 
               "]"}], "<>", "\"\<]],\>\""}]}], ";"}]}], "\[IndentingNewLine]",
           "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"posString", " ", "=", " ", 
          RowBox[{
           RowBox[{"StringDrop", "[", 
            RowBox[{"posString", ",", 
             RowBox[{"-", "1"}]}], "]"}], "<>", "\"\<]\>\""}]}], ";"}], 
        "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"posString", "=", "\"\<\>\""}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "negSet", "]"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"negSet", " ", "=", " ", 
          RowBox[{"polySet", "[", 
           RowBox[{"[", "negSet", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         
         RowBox[{"negString", " ", "=", " ", "\"\<[\>\""}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"m", "=", "1"}], ",", 
           RowBox[{"m", "<=", 
            RowBox[{"Length", "[", "negSet", "]"}]}], ",", 
           RowBox[{"m", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"negString", " ", "=", " ", 
             RowBox[{"negString", "<>", "\"\<[\>\"", "<>", 
              RowBox[{"ToString", "[", 
               RowBox[{"Round", "[", 
                RowBox[{
                 RowBox[{"-", 
                  SuperscriptBox["10", "10"]}], "*", 
                 RowBox[{"negSet", "[", 
                  RowBox[{"[", 
                   RowBox[{"m", ",", "2"}], "]"}], "]"}]}], "]"}], "]"}], 
              "<>", "\"\<,[\>\"", "<>", 
              RowBox[{"StringRiffle", "[", 
               RowBox[{
                RowBox[{"negSet", "[", 
                 RowBox[{"[", 
                  RowBox[{"m", ",", "1"}], "]"}], "]"}], ",", "\"\<,\>\""}], 
               "]"}], "<>", "\"\<]],\>\""}]}], ";"}]}], "\[IndentingNewLine]",
           "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"negString", " ", "=", " ", 
          RowBox[{
           RowBox[{"StringDrop", "[", 
            RowBox[{"negString", ",", 
             RowBox[{"-", "1"}]}], "]"}], "<>", "\"\<]\>\""}]}], ";"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"negString", "=", "\"\<\>\""}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{"posString", ",", "negString"}], "}"}], "]"}], ";"}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"reflectionRows", "[", 
    RowBox[{"H_", ",", "p_", ",", "vars_", ",", "subspace_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"added", "=", 
        RowBox[{"{", "}"}]}], ",", "k"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"sortedHRep", " ", "=", " ", 
       RowBox[{"Sort", "[", "H", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", " ", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "H", "]"}], "==", "0"}], ",", 
        RowBox[{"Return", "[", 
         RowBox[{"{", "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"k", "=", "2"}], ",", 
        RowBox[{"k", "<=", 
         RowBox[{
          RowBox[{"Dimensions", "[", "H", "]"}], "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", 
        RowBox[{"k", "++"}], ",", 
        RowBox[{
         RowBox[{"test", "=", " ", "H"}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"test", "[", 
           RowBox[{"[", 
            RowBox[{";;", ",", "k"}], "]"}], "]"}], "=", 
          RowBox[{"-", 
           RowBox[{"test", "[", 
            RowBox[{"[", 
             RowBox[{";;", ",", "k"}], "]"}], "]"}]}]}], ";", 
         RowBox[{"vect", " ", "=", " ", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"0", ",", 
            RowBox[{
             RowBox[{"Dimensions", "[", "H", "]"}], "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"vect", "[", 
           RowBox[{"[", "k", "]"}], "]"}], " ", "=", " ", "1"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Sort", "[", "test", "]"}], "===", "sortedHRep"}], " ", "&&",
             " ", 
            RowBox[{"evenPoly", "[", 
             RowBox[{"p", ",", 
              RowBox[{"subspace", "[", 
               RowBox[{"[", 
                RowBox[{"k", "-", "1"}], "]"}], "]"}]}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"added", " ", "=", " ", 
            RowBox[{"Join", "[", 
             RowBox[{"added", ",", 
              RowBox[{"{", "vect", "}"}], ",", "1"}], "]"}]}]}], 
          "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "added", "]"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"poly1", ",", "vars1", ",", "ineq1"}], "}"}], "=", 
   RowBox[{"integrationPartsSingleDelta", "[", "7", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"H1", " ", "=", " ", 
   RowBox[{"HRepFile", "[", 
    RowBox[{"ineq1", ",", "vars1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"For", "[", 
    RowBox[{
     RowBox[{"j", "=", "1"}], ",", 
     RowBox[{"j", "<=", "sites"}], ",", 
     RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{"poly", ",", "vars", ",", "ineq"}], "}"}], "=", 
       RowBox[{"integrationPartsSingleDelta", "[", "j", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ineq", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"oneSide", "/@", 
          RowBox[{"FullSimplify", "[", "ineq", "]"}]}], "//", 
         "DeleteDuplicates"}], "//", 
        RowBox[{"DeleteCases", "[", "True", "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"H", " ", "=", " ", 
       RowBox[{"HRepFile", "[", 
        RowBox[{"ineq", ",", "vars"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Export", "[", 
       RowBox[{
        RowBox[{
        "\"\<basedir/latteIntegrals/MathematicaFormat/E1/\>\"", "<>", 
         RowBox[{"ToString", "[", "d", "]"}], "<>", "\"\</HRepSingle-\>\"", "<>", 
         RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Dimensions", "[", "H", "]"}], "}"}], ",", 
          RowBox[{"Round", "[", 
           RowBox[{
            SuperscriptBox["10", "14"], "*", "H"}], "]"}]}], "]"}], ",", 
        "\"\<Table\>\"", ",", 
        RowBox[{"\"\<FieldSeparators\>\"", "->", "\"\< \>\""}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"polySet", " ", "=", " ", 
       RowBox[{"polyExp", "[", 
        RowBox[{"poly", ",", "vars"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"posString", ",", "negString"}], "}"}], "=", 
       RowBox[{"polyStringWrite", "[", 
        RowBox[{"poly", ",", "vars"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"StringLength", "[", "posString", "]"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Export", "[", 
          RowBox[{
           RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/E1/\>\"", "<>", 
            RowBox[{"ToString", "[", "d", "]"}], "<>", 
            "\"\</PolynomialSinglePos-\>\"", "<>", 
            RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
           "posString"}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"Export", "[", 
          RowBox[{
           RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/E1/\>\"", "<>", 
            RowBox[{"ToString", "[", "d", "]"}], "<>", 
            "\"\</CoefficientSinglePos-\>\"", "<>", 
            RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
           
           RowBox[{"ToString", "[", 
            RowBox[{
             RowBox[{"AccountingForm", "[", "1*^-10", "]"}], "//", "N"}], 
            "]"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"StringLength", "[", "negString", "]"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Export", "[", 
          RowBox[{
           RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/E1/\>\"", "<>", 
            RowBox[{"ToString", "[", "d", "]"}], "<>", 
            "\"\</PolynomialSingleNeg-\>\"", "<>", 
            RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
           "negString"}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"Export", "[", 
          RowBox[{
           RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/E1/\>\"", "<>", 
            RowBox[{"ToString", "[", "d", "]"}], "<>", 
            "\"\</CoefficientSingleNeg-\>\"", "<>", 
            RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
           
           RowBox[{"\"\<-\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{
              RowBox[{"AccountingForm", "[", "1*^-10", "]"}], "//", "N"}], 
             "]"}]}]}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}],
     "\[IndentingNewLine]", "]"}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"j", "=", "1"}], ",", 
    RowBox[{"j", "<=", "sites"}], ",", 
    RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{"For", "[", 
     RowBox[{
      RowBox[{"k", "=", 
       RowBox[{"j", "+", "1"}]}], ",", 
      RowBox[{"k", "<=", "sites"}], ",", 
      RowBox[{"k", "++"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"poly", ",", "vars", ",", "ineq"}], "}"}], "=", 
        RowBox[{"integrationPartsDoubleDelta", "[", 
         RowBox[{"j", ",", "k"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"ineq", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"oneSide", "/@", 
           RowBox[{"FullSimplify", "[", "ineq", "]"}]}], "//", 
          "DeleteDuplicates"}], "//", 
         RowBox[{"DeleteCases", "[", "True", "]"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"H", " ", "=", " ", 
        RowBox[{"HRepFile", "[", 
         RowBox[{"ineq", ",", "vars"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"H", " ", "=", " ", 
          RowBox[{"Join", "[", 
           RowBox[{"H", ",", 
            RowBox[{"reflectionRows", "[", 
             RowBox[{"H", ",", "poly", ",", "vars", ",", "ineq"}], "]"}]}], 
           "]"}]}], ";"}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "H", "]"}], ">", "0"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"overallCoeff", " ", "=", "1"}], ";", 
          RowBox[{"(*", " ", 
           RowBox[{
            RowBox[{"2", "^", 
             RowBox[{
              RowBox[{"Dimensions", "[", 
               RowBox[{"JustReflections", "[", "j", "]"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{"Export", "[", 
           RowBox[{
            RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/E1/\>\"", "<>", 
             RowBox[{"ToString", "[", "d", "]"}], "<>", 
             "\"\</HRepDouble-\>\"", "<>", 
             RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
             RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<.dat\>\""}], ",", 
            RowBox[{"Join", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"Dimensions", "[", "H", "]"}], "}"}], ",", 
              RowBox[{"Round", "[", 
               RowBox[{
                SuperscriptBox["10", "14"], "*", "H"}], "]"}]}], "]"}], ",", 
            "\"\<Table\>\"", ",", 
            RowBox[{"\"\<FieldSeparators\>\"", "->", "\"\< \>\""}]}], "]"}], 
          ";", "\[IndentingNewLine]", 
          RowBox[{"polySet", " ", "=", " ", 
           RowBox[{"polyExp", "[", 
            RowBox[{"poly", ",", "vars"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"posString", ",", "negString"}], "}"}], "=", 
           RowBox[{"polyStringWrite", "[", 
            RowBox[{"poly", ",", "vars"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"StringLength", "[", "posString", "]"}], ">", "0"}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"Export", "[", 
               RowBox[{
                RowBox[{
                "\"\<basedir/latteIntegrals/MathematicaFormat/E1/\>\"", "<>", 
                 
                 RowBox[{"ToString", "[", "d", "]"}], "<>", 
                 "\"\</PolynomialDoublePos-\>\"", "<>", 
                 RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
                 RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<.dat\>\""}], 
                ",", "posString"}], "]"}], "\[IndentingNewLine]", 
              RowBox[{"Export", "[", 
               RowBox[{
                RowBox[{
                "\"\<basedir/latteIntegrals/MathematicaFormat/E1/\>\"", "<>", 
                 
                 RowBox[{"ToString", "[", "d", "]"}], "<>", 
                 "\"\</CoefficientDoublePos-\>\"", "<>", 
                 RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
                 RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<.dat\>\""}], 
                ",", 
                RowBox[{"ToString", "[", 
                 RowBox[{
                  RowBox[{"AccountingForm", "[", "1*^-10", "]"}], "//", "N"}],
                  "]"}]}], "]"}]}], ";"}]}], "\[IndentingNewLine]", "]"}], 
          ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"StringLength", "[", "negString", "]"}], ">", "0"}], ",", 
            RowBox[{
             RowBox[{"Export", "[", 
              RowBox[{
               RowBox[{
               "\"\<basedir/latteIntegrals/MathematicaFormat/E1/\>\"", "<>", 
                RowBox[{"ToString", "[", "d", "]"}], "<>", 
                "\"\</PolynomialDoubleNeg-\>\"", "<>", 
                RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
                
                RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<.dat\>\""}], 
               ",", "negString"}], "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"Export", "[", 
              RowBox[{
               RowBox[{
               "\"\<basedir/latteIntegrals/MathematicaFormat/E1/\>\"", "<>", 
                RowBox[{"ToString", "[", "d", "]"}], "<>", 
                "\"\</CoefficientDoubleNeg-\>\"", "<>", 
                RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
                
                RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<.dat\>\""}], 
               ",", 
               RowBox[{"\"\<-\>\"", "<>", 
                RowBox[{"ToString", "[", 
                 RowBox[{
                  RowBox[{"AccountingForm", "[", "1*^-10", "]"}], "//", "N"}],
                  "]"}]}]}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}]}],
         "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
   "\[IndentingNewLine]", "]"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"Reduce", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"0", "-", "x2"}], ">=", "0"}], ",", 
    RowBox[{
     RowBox[{"2", "+", 
      RowBox[{
       RowBox[{"Sqrt", "[", "2", "]"}], "*", "x2"}]}], ">=", "0"}], ",", 
    RowBox[{
     RowBox[{"1", "+", 
      RowBox[{
       RowBox[{"Sqrt", "[", "2", "]"}], "*", "x2"}]}], ">=", "0"}], ",", 
    RowBox[{
     RowBox[{"1", "-", 
      RowBox[{
       RowBox[{"Sqrt", "[", "2", "]"}], "*", "x2"}]}], ">=", "0"}]}], "}"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"poly", ",", "vars", ",", "ineq"}], "}"}], "=", 
   RowBox[{"integrationPartsDoubleDelta", "[", 
    RowBox[{"1", ",", "8"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{"Expand", "[", "poly", "]"}], ",", "16"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Reduce", "[", "ineq", "]"}], "\[IndentingNewLine]", 
 RowBox[{"Integrate", "[", 
  RowBox[{
   RowBox[{"N", "[", 
    RowBox[{
     RowBox[{"Expand", "[", "poly", "]"}], ",", "16"}], "]"}], ",", 
   RowBox[{"x2", "\[Element]", 
    RowBox[{"ImplicitRegion", "[", 
     RowBox[{
      RowBox[{"0", "<=", "x2", "<=", 
       RowBox[{"1", "/", 
        RowBox[{"Sqrt", "[", "2", "]"}]}]}], ",", "x2"}], "]"}]}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{"N", "[", 
     RowBox[{
      RowBox[{"Expand", "[", "poly", "]"}], ",", "16"}], "]"}], ",", 
    RowBox[{"x2", "\[Element]", 
     RowBox[{"ImplicitRegion", "[", 
      RowBox[{
       RowBox[{"Reduce", "[", "ineq", "]"}], ",", "x2"}], "]"}]}]}], "]"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"oneSide", "/@", 
    RowBox[{"FullSimplify", "[", "ineq", "]"}]}], "//", "DeleteDuplicates"}], 
  "//", 
  RowBox[{"DeleteCases", "[", "True", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"oneSide", "/@", "ineq"}], "//", "DeleteDuplicates"}], "//", 
  RowBox[{"DeleteCases", "[", "True", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"HRepFile", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"oneSide", "/@", 
      RowBox[{"FullSimplify", "[", "ineq", "]"}]}], "//", 
     "DeleteDuplicates"}], "//", 
    RowBox[{"DeleteCases", "[", "True", "]"}]}], ",", "vars"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"HRepFile", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"oneSide", "/@", "ineq"}], "//", "DeleteDuplicates"}], "//", 
    RowBox[{"DeleteCases", "[", "True", "]"}]}], ",", "vars"}], 
  "]"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.837759051812975*^9, 3.837759071591836*^9}, {
   3.837759217318631*^9, 3.837759241400175*^9}, {3.837759320301201*^9, 
   3.83775936716477*^9}, {3.837759456303917*^9, 3.837759642000123*^9}, {
   3.837759816966599*^9, 3.83775991155928*^9}, {3.837759956835602*^9, 
   3.8377599789804487`*^9}, {3.837760016652355*^9, 3.837760096420809*^9}, {
   3.837760140717391*^9, 3.837760207546588*^9}, {3.8377602481030903`*^9, 
   3.8377602740980864`*^9}, {3.837760333242087*^9, 3.837760427623968*^9}, {
   3.8377604636283073`*^9, 3.8377607453888807`*^9}, {3.837760787735127*^9, 
   3.837760793341056*^9}, {3.837761056684053*^9, 3.837761100969037*^9}, {
   3.8377615513819017`*^9, 3.83776158670927*^9}, {3.837761621261744*^9, 
   3.8377616531407127`*^9}, {3.837761693987907*^9, 3.837761729898768*^9}, {
   3.837761844581964*^9, 3.837761970006235*^9}, {3.837762073595648*^9, 
   3.837762096290968*^9}, {3.837762142678879*^9, 3.8377622375066757`*^9}, {
   3.8377622979149218`*^9, 3.837762405543975*^9}, {3.83776251457353*^9, 
   3.837762566781784*^9}, {3.837762618932023*^9, 3.8377626637366657`*^9}, {
   3.837763509203718*^9, 3.837763591558241*^9}, {3.83776364159385*^9, 
   3.83776367456262*^9}, {3.837763759909724*^9, 3.8377638110846853`*^9}, {
   3.8377638908917513`*^9, 3.8377639078480997`*^9}, {3.837763980843951*^9, 
   3.837764177471188*^9}, {3.8377642380320673`*^9, 3.837764360041239*^9}, {
   3.837764442964953*^9, 3.837764502687722*^9}, {3.83776455233293*^9, 
   3.8377646335315657`*^9}, {3.837764699266444*^9, 3.8377647272120543`*^9}, {
   3.837764769287674*^9, 3.837765008276999*^9}, {3.837765052599174*^9, 
   3.837765057200159*^9}, {3.837765092612886*^9, 3.837765334368432*^9}, {
   3.8377654308449802`*^9, 3.8377655319772167`*^9}, {3.837765568969681*^9, 
   3.837765661871271*^9}, {3.837765791941186*^9, 3.837765912988414*^9}, {
   3.8377660145740337`*^9, 3.8377661776828814`*^9}, {3.837766237263646*^9, 
   3.837766278805069*^9}, {3.8377663100115967`*^9, 3.837766329618359*^9}, {
   3.837766657496814*^9, 3.837766657731106*^9}, {3.83776715547604*^9, 
   3.8377671565101347`*^9}, {3.8377674335426807`*^9, 3.8377674351976757`*^9}, 
   3.837768112932695*^9, {3.837768376845517*^9, 3.837768449341558*^9}, 
   3.837768484719595*^9, {3.837768542966751*^9, 3.837768557770595*^9}, {
   3.8377686335259457`*^9, 3.837768639317741*^9}, {3.837769274391078*^9, 
   3.8377693524395447`*^9}, {3.837769497420038*^9, 3.8377695037909527`*^9}, {
   3.837769596022748*^9, 3.8377696087576*^9}, {3.837769647025366*^9, 
   3.837769761571034*^9}, {3.83776983151989*^9, 3.8377699867015038`*^9}, {
   3.837770952577222*^9, 3.837771031706276*^9}, {3.837771104943639*^9, 
   3.837771165144917*^9}, 3.8377712134152813`*^9, {3.837771255405746*^9, 
   3.837771342336483*^9}, {3.837771373782195*^9, 3.837771500315425*^9}, {
   3.8377715318583117`*^9, 3.837771603465426*^9}, 3.83777165452458*^9, {
   3.837771689129313*^9, 3.8377717429592857`*^9}, {3.8377717947541027`*^9, 
   3.837772161043878*^9}, {3.837772205636837*^9, 3.837772226479508*^9}, {
   3.8377722800771313`*^9, 3.8377722844518147`*^9}, {3.837772316184485*^9, 
   3.837772330120789*^9}, {3.8377723601984863`*^9, 3.837772389931653*^9}, {
   3.837772440094946*^9, 3.8377724717356987`*^9}, {3.837772505324279*^9, 
   3.837772849864531*^9}, {3.837772896947722*^9, 3.837773036687307*^9}, {
   3.8377731147022753`*^9, 3.837773227864345*^9}, {3.837773275727414*^9, 
   3.837773298163156*^9}, {3.837773347035487*^9, 3.83777335522159*^9}, {
   3.837773412888567*^9, 3.837773475740651*^9}, {3.837773530112762*^9, 
   3.8377735931395397`*^9}, {3.837773630868861*^9, 3.837773822688457*^9}, {
   3.837773878077608*^9, 3.8377738872682123`*^9}, {3.837773919005477*^9, 
   3.837774000967046*^9}, {3.8377740453237867`*^9, 3.837774049124507*^9}, {
   3.837774085611842*^9, 3.837774223315514*^9}, 3.837774263547697*^9, {
   3.8377743121800003`*^9, 3.8377743126575537`*^9}, {3.837774496089884*^9, 
   3.837774503826468*^9}, {3.8377745574267807`*^9, 3.837774895068288*^9}, {
   3.837774940831119*^9, 3.8377750491080093`*^9}, {3.8377751090428333`*^9, 
   3.837775144213842*^9}, {3.8377751951433*^9, 3.837775305357629*^9}, {
   3.837775364894342*^9, 3.837775496742807*^9}, {3.8377755338131523`*^9, 
   3.8377755753185453`*^9}, {3.8377756277767887`*^9, 3.837775726961532*^9}, {
   3.8377757758014393`*^9, 3.837775839367248*^9}, {3.837775880521937*^9, 
   3.837775911455641*^9}, {3.837775942504963*^9, 3.837775964469473*^9}, {
   3.837775996413103*^9, 3.837776188288731*^9}, {3.837776234433379*^9, 
   3.837776350471487*^9}, {3.837776493128147*^9, 3.837776540687028*^9}, 
   3.8377766164973783`*^9, {3.837776652131136*^9, 3.8377766581985807`*^9}, {
   3.837776701059732*^9, 3.837776709453355*^9}, {3.83777678203025*^9, 
   3.83777678709515*^9}, {3.837776820532475*^9, 3.8377768331493797`*^9}, {
   3.837776956286186*^9, 3.837777064590246*^9}, {3.837777101706354*^9, 
   3.837777205573284*^9}, {3.837777272769291*^9, 3.837777673512064*^9}, {
   3.837777724691207*^9, 3.837777748824847*^9}, {3.837777879292939*^9, 
   3.837777898982292*^9}, {3.837777937064353*^9, 3.837777973922823*^9}, 
   3.837778019767487*^9, {3.837778050456066*^9, 3.837778147495962*^9}, {
   3.8377795716200027`*^9, 3.837779600678067*^9}, 3.837779651879637*^9, {
   3.8377796862719593`*^9, 3.837779717452261*^9}, {3.8377797614372*^9, 
   3.837779775519079*^9}, {3.837779972163884*^9, 3.8377799740534887`*^9}, {
   3.837780017640114*^9, 3.837780024681024*^9}, {3.837780057444573*^9, 
   3.837780080083229*^9}, 3.8377803804440327`*^9, {3.8377805107291203`*^9, 
   3.837780566161954*^9}, {3.837780637776647*^9, 3.837780822613832*^9}, {
   3.837780872466659*^9, 3.8377809327605352`*^9}, {3.837780992367846*^9, 
   3.837781005509432*^9}, {3.8377810726487713`*^9, 3.837781088451127*^9}, 
   3.8377811295651197`*^9, {3.837781739704506*^9, 3.837781750649578*^9}, {
   3.837781898408183*^9, 3.8377819488022537`*^9}, {3.8377819864007177`*^9, 
   3.837781992270015*^9}, {3.837782024170294*^9, 3.8377821019794483`*^9}, {
   3.837782142737905*^9, 3.837782162036997*^9}, {3.837782354378776*^9, 
   3.837782448482431*^9}, {3.8377848682062187`*^9, 3.8377848913130713`*^9}, {
   3.837784930076548*^9, 3.8377849415859213`*^9}, {3.837785645716629*^9, 
   3.837785678588539*^9}, {3.837785728736228*^9, 3.837785763713437*^9}, 
   3.837786034233245*^9, {3.837786066746749*^9, 3.837786116126554*^9}, {
   3.837861251941223*^9, 3.83786128903496*^9}, {3.837861349162579*^9, 
   3.83786137538763*^9}, {3.837861441354437*^9, 3.8378614479760733`*^9}, {
   3.83786148029738*^9, 3.837861505582625*^9}, 3.837861752973679*^9, {
   3.837862755674859*^9, 3.837862763059555*^9}, {3.8380758896068077`*^9, 
   3.8380759297842407`*^9}, {3.8380975980153093`*^9, 3.838097663482284*^9}, {
   3.8380977510926313`*^9, 3.838097788509506*^9}, {3.8380978601000767`*^9, 
   3.8380978646323233`*^9}, {3.8380980026732683`*^9, 
   3.8380980036038847`*^9}, {3.838098036220108*^9, 3.838098064053454*^9}, {
   3.838112651834959*^9, 3.8381126815227118`*^9}, 3.838114667604498*^9, {
   3.8381157640608788`*^9, 3.838115813481683*^9}, {3.838115854554874*^9, 
   3.838115890462085*^9}, {3.8381159232655077`*^9, 3.83811621862961*^9}, 
   3.838116376202277*^9, {3.838116502668528*^9, 3.838116528605359*^9}, {
   3.838116559555479*^9, 3.838116567161934*^9}, {3.8381167114662952`*^9, 
   3.83811672720665*^9}, {3.838116871218288*^9, 3.838116928407834*^9}, {
   3.8381169723037233`*^9, 3.838117176849574*^9}, {3.838117211210527*^9, 
   3.838117337049486*^9}, {3.838117400110519*^9, 3.8381174104287243`*^9}, {
   3.838117804814328*^9, 3.838117811366661*^9}, {3.838117971393291*^9, 
   3.8381180172717133`*^9}, {3.8381180573208237`*^9, 3.838118114782769*^9}, {
   3.8381181468442497`*^9, 3.838118259837858*^9}, {3.8381183198373623`*^9, 
   3.83811839171646*^9}, {3.838118495332131*^9, 3.838118510806614*^9}, {
   3.838118549071732*^9, 3.838118671093519*^9}, {3.838118727371719*^9, 
   3.838118729575013*^9}, {3.838118802904285*^9, 3.838119004428368*^9}, {
   3.838119069326445*^9, 3.838119100184208*^9}, {3.838119132017015*^9, 
   3.8381191324033747`*^9}, {3.838119357982761*^9, 3.838119359264317*^9}, {
   3.838119397391301*^9, 3.8381193979146833`*^9}, {3.838119434377028*^9, 
   3.838119793141184*^9}, {3.838119852912465*^9, 3.83811986850945*^9}, {
   3.838120588456049*^9, 3.8381206931623774`*^9}, {3.8381207285414057`*^9, 
   3.8381208119394093`*^9}, {3.8381208549039583`*^9, 
   3.8381209295082197`*^9}, {3.8381209621077623`*^9, 3.838121211124579*^9}, {
   3.838121274501198*^9, 3.838121430586789*^9}, {3.838121488790436*^9, 
   3.838121501607415*^9}, {3.8381226720274067`*^9, 3.838122698728516*^9}, {
   3.838124396057765*^9, 3.8381243961994123`*^9}, {3.8381321490254507`*^9, 
   3.8381322069763193`*^9}, {3.838132258492202*^9, 3.838132265912795*^9}, {
   3.8381323277398653`*^9, 3.838132537541498*^9}, {3.838133288998493*^9, 
   3.838133471070714*^9}, {3.8381337160067*^9, 3.838133780246677*^9}, {
   3.83813588350189*^9, 3.838135950251145*^9}, {3.838136027408605*^9, 
   3.8381361909936037`*^9}, {3.838136316967559*^9, 3.838136333768537*^9}, {
   3.8381364051098146`*^9, 3.838136468974766*^9}, {3.838138284810422*^9, 
   3.838138287971339*^9}, {3.838138446009407*^9, 3.83813848974603*^9}, {
   3.838138600077479*^9, 3.83813861376338*^9}, {3.838192809669958*^9, 
   3.838192823412847*^9}, {3.838193462382653*^9, 3.838193463311808*^9}, {
   3.838193517496935*^9, 3.8381936261992607`*^9}, {3.838193681831151*^9, 
   3.8381936835272903`*^9}, {3.8381937789211617`*^9, 3.838193814649061*^9}, {
   3.838193860044997*^9, 3.838193863279777*^9}, 3.8381941879749928`*^9, {
   3.8381947873871307`*^9, 3.838194787926078*^9}, {3.838195098425609*^9, 
   3.838195149315035*^9}, {3.838195218759927*^9, 3.838195219667021*^9}, {
   3.83819525925213*^9, 3.838195334819067*^9}, {3.838195372819125*^9, 
   3.838195376876358*^9}, {3.8381954381242523`*^9, 3.838195458961376*^9}, {
   3.838195489350226*^9, 3.838195509066133*^9}, {3.838195573389518*^9, 
   3.838195654438189*^9}, {3.838196826400489*^9, 3.838196830581263*^9}, {
   3.838196948659704*^9, 3.838196956537139*^9}, 3.838197043039798*^9, {
   3.838197078037409*^9, 3.838197083025579*^9}, {3.838197353820166*^9, 
   3.8381973990323353`*^9}, {3.838197444616848*^9, 3.83819746836843*^9}, {
   3.838197499992804*^9, 3.838197500149294*^9}, {3.838197580281456*^9, 
   3.8381976271037407`*^9}, {3.838197755901965*^9, 3.83819775640872*^9}, {
   3.838199496671093*^9, 3.838199536907127*^9}, {3.8381995766229362`*^9, 
   3.8381996231000977`*^9}, {3.838200041907902*^9, 3.8382000549270163`*^9}, {
   3.8382003233725863`*^9, 3.838200327800591*^9}, {3.838200365058825*^9, 
   3.8382004318613234`*^9}, {3.838200482019011*^9, 3.838200493740061*^9}, {
   3.838200563390181*^9, 3.8382005658238373`*^9}, {3.838200643060609*^9, 
   3.838200657165009*^9}, {3.838200765393807*^9, 3.838200824425879*^9}, {
   3.8382009866018953`*^9, 3.838201076654128*^9}, {3.83820113182444*^9, 
   3.8382011977382812`*^9}, {3.8382012543099413`*^9, 3.838201318408381*^9}, {
   3.838201357052339*^9, 3.838201357481707*^9}, {3.8382014135977087`*^9, 
   3.838201438494611*^9}, {3.8382018279644537`*^9, 3.838201864091898*^9}, 
   3.838202462998166*^9, 3.838204517534644*^9, {3.838239536977901*^9, 
   3.838239546552473*^9}, {3.838239758656314*^9, 3.838239781431768*^9}, {
   3.838243402882976*^9, 3.83824358480553*^9}, {3.838243726729574*^9, 
   3.838243728935431*^9}, {3.838243835343155*^9, 3.838243913631022*^9}, {
   3.838243951289035*^9, 3.83824402249965*^9}, {3.8382441256817217`*^9, 
   3.838244288609477*^9}, {3.838244929281444*^9, 3.838245014941049*^9}, {
   3.838245072952002*^9, 3.838245095359233*^9}, {3.838245162348489*^9, 
   3.8382451749714327`*^9}, {3.838245227568116*^9, 3.8382452452858467`*^9}, {
   3.838245284024765*^9, 3.8382452948969517`*^9}, 3.838245411585197*^9, {
   3.838246188268137*^9, 3.8382462719556227`*^9}, {3.838246303414322*^9, 
   3.838246307317048*^9}, {3.838246345779402*^9, 3.838246347527246*^9}, {
   3.838246434866619*^9, 3.8382464991992826`*^9}, {3.838246564210376*^9, 
   3.838246586049138*^9}, {3.838246619973543*^9, 3.8382466201744413`*^9}, {
   3.838248108801785*^9, 3.8382481365512943`*^9}, {3.838248182268852*^9, 
   3.838248257498809*^9}, {3.838248372450782*^9, 3.838248441185956*^9}, {
   3.8382484770352983`*^9, 3.8382484876107187`*^9}, {3.838248525352385*^9, 
   3.8382486329610167`*^9}, {3.838248675276849*^9, 3.8382486889701777`*^9}, {
   3.838248732173944*^9, 3.838248740809209*^9}, {3.8382503579474688`*^9, 
   3.838250374497404*^9}, {3.8382504121682577`*^9, 3.8382505487156143`*^9}, {
   3.838250580628149*^9, 3.83825060657659*^9}, {3.8382507972247763`*^9, 
   3.838250797773428*^9}, {3.838250830615451*^9, 3.838250935735551*^9}, {
   3.838251061243134*^9, 3.8382511198144407`*^9}, {3.838252038885927*^9, 
   3.838252067220907*^9}, 3.838252530669641*^9, {3.838274238280954*^9, 
   3.838274251294813*^9}, {3.838274283531253*^9, 3.8382743127786293`*^9}, 
   3.8382743796102047`*^9, {3.838275093816903*^9, 3.838275100859033*^9}, {
   3.838275140293757*^9, 3.838275180457425*^9}, {3.83827528631536*^9, 
   3.8382753625275993`*^9}, {3.8382754589149303`*^9, 3.838275503693738*^9}, 
   3.8382756532890177`*^9, {3.838276061437256*^9, 3.838276199071059*^9}, {
   3.838276409152915*^9, 3.838276485539034*^9}, {3.838280186837373*^9, 
   3.838280195716236*^9}, {3.838292100656541*^9, 3.838292154951517*^9}, {
   3.83829231164585*^9, 3.838292316379764*^9}, 3.838701769997694*^9, 
   3.838801507840971*^9, {3.8479834766158733`*^9, 3.8479835185154343`*^9}, {
   3.847984037582533*^9, 3.847984037631011*^9}, 
   3.84798407464692*^9},ExpressionUUID->"ee0adbbb-781a-4989-b46a-\
86f9b1aad08b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", 
    SqrtBox["2"]]}], "\[LessEqual]", "x2", "\[LessEqual]", "0"}]], "Output",
 CellChangeTimes->{
  3.838135855284397*^9, 3.838135935307404*^9, 3.838135987630686*^9, 
   3.838136079962658*^9, 3.8381361558364697`*^9, 3.8381362142647333`*^9, 
   3.838136345509656*^9, 3.838136417987011*^9, {3.8381364537661333`*^9, 
   3.838136476321108*^9}, 3.8381382910158577`*^9, {3.838138452598276*^9, 
   3.838138492567541*^9}, {3.838138604607637*^9, 3.8381386167721033`*^9}, 
   3.8381928263462467`*^9, 3.8381934663838053`*^9, {3.838193522883288*^9, 
   3.8381935681527147`*^9}, {3.838193599430602*^9, 3.838193629220193*^9}, 
   3.8381936882508783`*^9, {3.838193796809009*^9, 3.8381938185430107`*^9}, 
   3.838193866097094*^9, 3.838194191029058*^9, 3.838194791090049*^9, 
   3.838195118658203*^9, 3.838195152202137*^9, 3.838195223208246*^9, {
   3.838195321701625*^9, 3.8381953376093082`*^9}, 3.838195379655159*^9, {
   3.838195445071327*^9, 3.838195461955699*^9}, {3.8381954955702353`*^9, 
   3.8381955121655188`*^9}, 3.838195598626733*^9, {3.838195628666795*^9, 
   3.8381956580540648`*^9}, 3.838196592195594*^9, {3.838196817124673*^9, 
   3.83819683393499*^9}, 3.8381969594094048`*^9, 3.83819704735888*^9, 
   3.83819708733041*^9, {3.83819736012643*^9, 3.83819740238835*^9}, {
   3.838197454616816*^9, 3.838197471253503*^9}, 3.8381975030543127`*^9, 
   3.8381975879979362`*^9, 3.838197630199223*^9, 3.8381977594094877`*^9, {
   3.838199516979061*^9, 3.8381995409289923`*^9}, {3.838199585941297*^9, 
   3.838199626978765*^9}, 3.8382001929688396`*^9, 3.838200233427125*^9, 
   3.838200332052126*^9, {3.8382004001240187`*^9, 3.838200434020404*^9}, 
   3.8382004971617317`*^9, 3.8382005667855597`*^9, 3.838200658025077*^9, {
   3.8382007712345333`*^9, 3.838200789961648*^9}, 3.838200824989113*^9, {
   3.838200988561356*^9, 3.838201028673361*^9}, 3.838201078242505*^9, {
   3.8382011430217113`*^9, 3.838201201316042*^9}, {3.838201260133481*^9, 
   3.838201278943379*^9}, {3.838201309745639*^9, 3.838201319001542*^9}, 
   3.83820135851289*^9, 3.8382014462060137`*^9, {3.838201853911603*^9, 
   3.838201867761775*^9}, 3.8382024675638857`*^9, 3.838204525932125*^9, 
   3.8382052629022617`*^9, 3.8382081903776407`*^9, 3.838239084298512*^9, {
   3.838239219948361*^9, 3.838239249250147*^9}, {3.838239529016798*^9, 
   3.838239549815136*^9}, 3.838239810192692*^9, 3.83824359845688*^9, 
   3.838243650547364*^9, 3.838243731575823*^9, 3.838243868032824*^9, {
   3.838243899763178*^9, 3.8382439163109407`*^9}, {3.838243954781808*^9, 
   3.8382440260526943`*^9}, 3.838245032176836*^9, 3.838245098972673*^9, 
   3.838245182091054*^9, 3.8382452484492073`*^9, 3.838245297611104*^9, 
   3.838245415500739*^9, 3.838245446111436*^9, 3.838246355068489*^9, 
   3.838246503154478*^9, 3.838246589405971*^9, {3.83824662507314*^9, 
   3.838246651501807*^9}, 3.8382486965885572`*^9, 3.838250618563526*^9, 
   3.8382509383063927`*^9, 3.8382511245976257`*^9, 3.838251160512944*^9, 
   3.8382520710228453`*^9, 3.8382525394049263`*^9, 3.8382742548772783`*^9, {
   3.838274288173587*^9, 3.8382743160545673`*^9}, 3.838274411170527*^9, 
   3.8382751035065203`*^9, {3.8382751647677727`*^9, 3.8382751826597958`*^9}, 
   3.838275295997991*^9, 3.838275364870254*^9, 3.838275464962778*^9, 
   3.8382755073576527`*^9, 3.838275667551221*^9, 3.838276165497401*^9, 
   3.838276201673314*^9, {3.83827641718303*^9, 3.838276440897884*^9}, 
   3.838276488090859*^9, 3.8382805284765873`*^9, 3.8382905004614964`*^9, 
   3.8382923580082407`*^9, 3.8383031741500473`*^9, 3.838398738541813*^9, 
   3.838701774531659*^9, 3.838801482781333*^9, 3.838801516881137*^9},
 CellLabel->
  "Out[238]=",ExpressionUUID->"e20888d3-8c5d-4904-9b16-275938c65530"],

Cell[BoxData[
 RowBox[{"0.5`16.", "+", 
  RowBox[{"2.82842712474619009760337744841939615714`16.", " ", "x2"}], "+", 
  RowBox[{"7.`16.", " ", 
   SuperscriptBox["x2", "2"]}], "+", 
  RowBox[{"8.48528137423857029281013234525818847141`16.", " ", 
   SuperscriptBox["x2", "3"]}], "+", 
  RowBox[{"4.5`16.", " ", 
   SuperscriptBox["x2", "4"]}]}]], "Output",
 CellChangeTimes->{
  3.838135855284397*^9, 3.838135935307404*^9, 3.838135987630686*^9, 
   3.838136079962658*^9, 3.8381361558364697`*^9, 3.8381362142647333`*^9, 
   3.838136345509656*^9, 3.838136417987011*^9, {3.8381364537661333`*^9, 
   3.838136476321108*^9}, 3.8381382910158577`*^9, {3.838138452598276*^9, 
   3.838138492567541*^9}, {3.838138604607637*^9, 3.8381386167721033`*^9}, 
   3.8381928263462467`*^9, 3.8381934663838053`*^9, {3.838193522883288*^9, 
   3.8381935681527147`*^9}, {3.838193599430602*^9, 3.838193629220193*^9}, 
   3.8381936882508783`*^9, {3.838193796809009*^9, 3.8381938185430107`*^9}, 
   3.838193866097094*^9, 3.838194191029058*^9, 3.838194791090049*^9, 
   3.838195118658203*^9, 3.838195152202137*^9, 3.838195223208246*^9, {
   3.838195321701625*^9, 3.8381953376093082`*^9}, 3.838195379655159*^9, {
   3.838195445071327*^9, 3.838195461955699*^9}, {3.8381954955702353`*^9, 
   3.8381955121655188`*^9}, 3.838195598626733*^9, {3.838195628666795*^9, 
   3.8381956580540648`*^9}, 3.838196592195594*^9, {3.838196817124673*^9, 
   3.83819683393499*^9}, 3.8381969594094048`*^9, 3.83819704735888*^9, 
   3.83819708733041*^9, {3.83819736012643*^9, 3.83819740238835*^9}, {
   3.838197454616816*^9, 3.838197471253503*^9}, 3.8381975030543127`*^9, 
   3.8381975879979362`*^9, 3.838197630199223*^9, 3.8381977594094877`*^9, {
   3.838199516979061*^9, 3.8381995409289923`*^9}, {3.838199585941297*^9, 
   3.838199626978765*^9}, 3.8382001929688396`*^9, 3.838200233427125*^9, 
   3.838200332052126*^9, {3.8382004001240187`*^9, 3.838200434020404*^9}, 
   3.8382004971617317`*^9, 3.8382005667855597`*^9, 3.838200658025077*^9, {
   3.8382007712345333`*^9, 3.838200789961648*^9}, 3.838200824989113*^9, {
   3.838200988561356*^9, 3.838201028673361*^9}, 3.838201078242505*^9, {
   3.8382011430217113`*^9, 3.838201201316042*^9}, {3.838201260133481*^9, 
   3.838201278943379*^9}, {3.838201309745639*^9, 3.838201319001542*^9}, 
   3.83820135851289*^9, 3.8382014462060137`*^9, {3.838201853911603*^9, 
   3.838201867761775*^9}, 3.8382024675638857`*^9, 3.838204525932125*^9, 
   3.8382052629022617`*^9, 3.8382081903776407`*^9, 3.838239084298512*^9, {
   3.838239219948361*^9, 3.838239249250147*^9}, {3.838239529016798*^9, 
   3.838239549815136*^9}, 3.838239810192692*^9, 3.83824359845688*^9, 
   3.838243650547364*^9, 3.838243731575823*^9, 3.838243868032824*^9, {
   3.838243899763178*^9, 3.8382439163109407`*^9}, {3.838243954781808*^9, 
   3.8382440260526943`*^9}, 3.838245032176836*^9, 3.838245098972673*^9, 
   3.838245182091054*^9, 3.8382452484492073`*^9, 3.838245297611104*^9, 
   3.838245415500739*^9, 3.838245446111436*^9, 3.838246355068489*^9, 
   3.838246503154478*^9, 3.838246589405971*^9, {3.83824662507314*^9, 
   3.838246651501807*^9}, 3.8382486965885572`*^9, 3.838250618563526*^9, 
   3.8382509383063927`*^9, 3.8382511245976257`*^9, 3.838251160512944*^9, 
   3.8382520710228453`*^9, 3.8382525394049263`*^9, 3.8382742548772783`*^9, {
   3.838274288173587*^9, 3.8382743160545673`*^9}, 3.838274411170527*^9, 
   3.8382751035065203`*^9, {3.8382751647677727`*^9, 3.8382751826597958`*^9}, 
   3.838275295997991*^9, 3.838275364870254*^9, 3.838275464962778*^9, 
   3.8382755073576527`*^9, 3.838275667551221*^9, 3.838276165497401*^9, 
   3.838276201673314*^9, {3.83827641718303*^9, 3.838276440897884*^9}, 
   3.838276488090859*^9, 3.8382805284765873`*^9, 3.8382905004614964`*^9, 
   3.8382923580082407`*^9, 3.8383031741500473`*^9, 3.838398738541813*^9, 
   3.838701774531659*^9, 3.838801482781333*^9, 3.838801516958802*^9},
 CellLabel->
  "Out[240]=",ExpressionUUID->"a526958d-68fc-4f5f-ba86-9fad417243f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", 
    SqrtBox["2"]]}], "\[LessEqual]", "x2", "\[LessEqual]", "0"}]], "Output",
 CellChangeTimes->{
  3.838135855284397*^9, 3.838135935307404*^9, 3.838135987630686*^9, 
   3.838136079962658*^9, 3.8381361558364697`*^9, 3.8381362142647333`*^9, 
   3.838136345509656*^9, 3.838136417987011*^9, {3.8381364537661333`*^9, 
   3.838136476321108*^9}, 3.8381382910158577`*^9, {3.838138452598276*^9, 
   3.838138492567541*^9}, {3.838138604607637*^9, 3.8381386167721033`*^9}, 
   3.8381928263462467`*^9, 3.8381934663838053`*^9, {3.838193522883288*^9, 
   3.8381935681527147`*^9}, {3.838193599430602*^9, 3.838193629220193*^9}, 
   3.8381936882508783`*^9, {3.838193796809009*^9, 3.8381938185430107`*^9}, 
   3.838193866097094*^9, 3.838194191029058*^9, 3.838194791090049*^9, 
   3.838195118658203*^9, 3.838195152202137*^9, 3.838195223208246*^9, {
   3.838195321701625*^9, 3.8381953376093082`*^9}, 3.838195379655159*^9, {
   3.838195445071327*^9, 3.838195461955699*^9}, {3.8381954955702353`*^9, 
   3.8381955121655188`*^9}, 3.838195598626733*^9, {3.838195628666795*^9, 
   3.8381956580540648`*^9}, 3.838196592195594*^9, {3.838196817124673*^9, 
   3.83819683393499*^9}, 3.8381969594094048`*^9, 3.83819704735888*^9, 
   3.83819708733041*^9, {3.83819736012643*^9, 3.83819740238835*^9}, {
   3.838197454616816*^9, 3.838197471253503*^9}, 3.8381975030543127`*^9, 
   3.8381975879979362`*^9, 3.838197630199223*^9, 3.8381977594094877`*^9, {
   3.838199516979061*^9, 3.8381995409289923`*^9}, {3.838199585941297*^9, 
   3.838199626978765*^9}, 3.8382001929688396`*^9, 3.838200233427125*^9, 
   3.838200332052126*^9, {3.8382004001240187`*^9, 3.838200434020404*^9}, 
   3.8382004971617317`*^9, 3.8382005667855597`*^9, 3.838200658025077*^9, {
   3.8382007712345333`*^9, 3.838200789961648*^9}, 3.838200824989113*^9, {
   3.838200988561356*^9, 3.838201028673361*^9}, 3.838201078242505*^9, {
   3.8382011430217113`*^9, 3.838201201316042*^9}, {3.838201260133481*^9, 
   3.838201278943379*^9}, {3.838201309745639*^9, 3.838201319001542*^9}, 
   3.83820135851289*^9, 3.8382014462060137`*^9, {3.838201853911603*^9, 
   3.838201867761775*^9}, 3.8382024675638857`*^9, 3.838204525932125*^9, 
   3.8382052629022617`*^9, 3.8382081903776407`*^9, 3.838239084298512*^9, {
   3.838239219948361*^9, 3.838239249250147*^9}, {3.838239529016798*^9, 
   3.838239549815136*^9}, 3.838239810192692*^9, 3.83824359845688*^9, 
   3.838243650547364*^9, 3.838243731575823*^9, 3.838243868032824*^9, {
   3.838243899763178*^9, 3.8382439163109407`*^9}, {3.838243954781808*^9, 
   3.8382440260526943`*^9}, 3.838245032176836*^9, 3.838245098972673*^9, 
   3.838245182091054*^9, 3.8382452484492073`*^9, 3.838245297611104*^9, 
   3.838245415500739*^9, 3.838245446111436*^9, 3.838246355068489*^9, 
   3.838246503154478*^9, 3.838246589405971*^9, {3.83824662507314*^9, 
   3.838246651501807*^9}, 3.8382486965885572`*^9, 3.838250618563526*^9, 
   3.8382509383063927`*^9, 3.8382511245976257`*^9, 3.838251160512944*^9, 
   3.8382520710228453`*^9, 3.8382525394049263`*^9, 3.8382742548772783`*^9, {
   3.838274288173587*^9, 3.8382743160545673`*^9}, 3.838274411170527*^9, 
   3.8382751035065203`*^9, {3.8382751647677727`*^9, 3.8382751826597958`*^9}, 
   3.838275295997991*^9, 3.838275364870254*^9, 3.838275464962778*^9, 
   3.8382755073576527`*^9, 3.838275667551221*^9, 3.838276165497401*^9, 
   3.838276201673314*^9, {3.83827641718303*^9, 3.838276440897884*^9}, 
   3.838276488090859*^9, 3.8382805284765873`*^9, 3.8382905004614964`*^9, 
   3.8382923580082407`*^9, 3.8383031741500473`*^9, 3.838398738541813*^9, 
   3.838701774531659*^9, 3.838801482781333*^9, 3.838801517012026*^9},
 CellLabel->
  "Out[241]=",ExpressionUUID->"896b6f37-6ca5-43af-9674-c9d6e1747b8c"],

Cell[BoxData[
 RowBox[{"{", "2.57504719482101056804178999592328626032`16.", "}"}]], "Output",\

 CellChangeTimes->{
  3.838135855284397*^9, 3.838135935307404*^9, 3.838135987630686*^9, 
   3.838136079962658*^9, 3.8381361558364697`*^9, 3.8381362142647333`*^9, 
   3.838136345509656*^9, 3.838136417987011*^9, {3.8381364537661333`*^9, 
   3.838136476321108*^9}, 3.8381382910158577`*^9, {3.838138452598276*^9, 
   3.838138492567541*^9}, {3.838138604607637*^9, 3.8381386167721033`*^9}, 
   3.8381928263462467`*^9, 3.8381934663838053`*^9, {3.838193522883288*^9, 
   3.8381935681527147`*^9}, {3.838193599430602*^9, 3.838193629220193*^9}, 
   3.8381936882508783`*^9, {3.838193796809009*^9, 3.8381938185430107`*^9}, 
   3.838193866097094*^9, 3.838194191029058*^9, 3.838194791090049*^9, 
   3.838195118658203*^9, 3.838195152202137*^9, 3.838195223208246*^9, {
   3.838195321701625*^9, 3.8381953376093082`*^9}, 3.838195379655159*^9, {
   3.838195445071327*^9, 3.838195461955699*^9}, {3.8381954955702353`*^9, 
   3.8381955121655188`*^9}, 3.838195598626733*^9, {3.838195628666795*^9, 
   3.8381956580540648`*^9}, 3.838196592195594*^9, {3.838196817124673*^9, 
   3.83819683393499*^9}, 3.8381969594094048`*^9, 3.83819704735888*^9, 
   3.83819708733041*^9, {3.83819736012643*^9, 3.83819740238835*^9}, {
   3.838197454616816*^9, 3.838197471253503*^9}, 3.8381975030543127`*^9, 
   3.8381975879979362`*^9, 3.838197630199223*^9, 3.8381977594094877`*^9, {
   3.838199516979061*^9, 3.8381995409289923`*^9}, {3.838199585941297*^9, 
   3.838199626978765*^9}, 3.8382001929688396`*^9, 3.838200233427125*^9, 
   3.838200332052126*^9, {3.8382004001240187`*^9, 3.838200434020404*^9}, 
   3.8382004971617317`*^9, 3.8382005667855597`*^9, 3.838200658025077*^9, {
   3.8382007712345333`*^9, 3.838200789961648*^9}, 3.838200824989113*^9, {
   3.838200988561356*^9, 3.838201028673361*^9}, 3.838201078242505*^9, {
   3.8382011430217113`*^9, 3.838201201316042*^9}, {3.838201260133481*^9, 
   3.838201278943379*^9}, {3.838201309745639*^9, 3.838201319001542*^9}, 
   3.83820135851289*^9, 3.8382014462060137`*^9, {3.838201853911603*^9, 
   3.838201867761775*^9}, 3.8382024675638857`*^9, 3.838204525932125*^9, 
   3.8382052629022617`*^9, 3.8382081903776407`*^9, 3.838239084298512*^9, {
   3.838239219948361*^9, 3.838239249250147*^9}, {3.838239529016798*^9, 
   3.838239549815136*^9}, 3.838239810192692*^9, 3.83824359845688*^9, 
   3.838243650547364*^9, 3.838243731575823*^9, 3.838243868032824*^9, {
   3.838243899763178*^9, 3.8382439163109407`*^9}, {3.838243954781808*^9, 
   3.8382440260526943`*^9}, 3.838245032176836*^9, 3.838245098972673*^9, 
   3.838245182091054*^9, 3.8382452484492073`*^9, 3.838245297611104*^9, 
   3.838245415500739*^9, 3.838245446111436*^9, 3.838246355068489*^9, 
   3.838246503154478*^9, 3.838246589405971*^9, {3.83824662507314*^9, 
   3.838246651501807*^9}, 3.8382486965885572`*^9, 3.838250618563526*^9, 
   3.8382509383063927`*^9, 3.8382511245976257`*^9, 3.838251160512944*^9, 
   3.8382520710228453`*^9, 3.8382525394049263`*^9, 3.8382742548772783`*^9, {
   3.838274288173587*^9, 3.8382743160545673`*^9}, 3.838274411170527*^9, 
   3.8382751035065203`*^9, {3.8382751647677727`*^9, 3.8382751826597958`*^9}, 
   3.838275295997991*^9, 3.838275364870254*^9, 3.838275464962778*^9, 
   3.8382755073576527`*^9, 3.838275667551221*^9, 3.838276165497401*^9, 
   3.838276201673314*^9, {3.83827641718303*^9, 3.838276440897884*^9}, 
   3.838276488090859*^9, 3.8382805284765873`*^9, 3.8382905004614964`*^9, 
   3.8382923580082407`*^9, 3.8383031741500473`*^9, 3.838398738541813*^9, 
   3.838701774531659*^9, 3.838801482781333*^9, 3.83880151708053*^9},
 CellLabel->
  "Out[242]=",ExpressionUUID->"e3cab884-99c2-4d1c-ab30-1a1205eefe82"],

Cell[BoxData[
 RowBox[{"{", "0.10017346066809423262402382608865581611`16.", "}"}]], "Output",\

 CellChangeTimes->{
  3.838135855284397*^9, 3.838135935307404*^9, 3.838135987630686*^9, 
   3.838136079962658*^9, 3.8381361558364697`*^9, 3.8381362142647333`*^9, 
   3.838136345509656*^9, 3.838136417987011*^9, {3.8381364537661333`*^9, 
   3.838136476321108*^9}, 3.8381382910158577`*^9, {3.838138452598276*^9, 
   3.838138492567541*^9}, {3.838138604607637*^9, 3.8381386167721033`*^9}, 
   3.8381928263462467`*^9, 3.8381934663838053`*^9, {3.838193522883288*^9, 
   3.8381935681527147`*^9}, {3.838193599430602*^9, 3.838193629220193*^9}, 
   3.8381936882508783`*^9, {3.838193796809009*^9, 3.8381938185430107`*^9}, 
   3.838193866097094*^9, 3.838194191029058*^9, 3.838194791090049*^9, 
   3.838195118658203*^9, 3.838195152202137*^9, 3.838195223208246*^9, {
   3.838195321701625*^9, 3.8381953376093082`*^9}, 3.838195379655159*^9, {
   3.838195445071327*^9, 3.838195461955699*^9}, {3.8381954955702353`*^9, 
   3.8381955121655188`*^9}, 3.838195598626733*^9, {3.838195628666795*^9, 
   3.8381956580540648`*^9}, 3.838196592195594*^9, {3.838196817124673*^9, 
   3.83819683393499*^9}, 3.8381969594094048`*^9, 3.83819704735888*^9, 
   3.83819708733041*^9, {3.83819736012643*^9, 3.83819740238835*^9}, {
   3.838197454616816*^9, 3.838197471253503*^9}, 3.8381975030543127`*^9, 
   3.8381975879979362`*^9, 3.838197630199223*^9, 3.8381977594094877`*^9, {
   3.838199516979061*^9, 3.8381995409289923`*^9}, {3.838199585941297*^9, 
   3.838199626978765*^9}, 3.8382001929688396`*^9, 3.838200233427125*^9, 
   3.838200332052126*^9, {3.8382004001240187`*^9, 3.838200434020404*^9}, 
   3.8382004971617317`*^9, 3.8382005667855597`*^9, 3.838200658025077*^9, {
   3.8382007712345333`*^9, 3.838200789961648*^9}, 3.838200824989113*^9, {
   3.838200988561356*^9, 3.838201028673361*^9}, 3.838201078242505*^9, {
   3.8382011430217113`*^9, 3.838201201316042*^9}, {3.838201260133481*^9, 
   3.838201278943379*^9}, {3.838201309745639*^9, 3.838201319001542*^9}, 
   3.83820135851289*^9, 3.8382014462060137`*^9, {3.838201853911603*^9, 
   3.838201867761775*^9}, 3.8382024675638857`*^9, 3.838204525932125*^9, 
   3.8382052629022617`*^9, 3.8382081903776407`*^9, 3.838239084298512*^9, {
   3.838239219948361*^9, 3.838239249250147*^9}, {3.838239529016798*^9, 
   3.838239549815136*^9}, 3.838239810192692*^9, 3.83824359845688*^9, 
   3.838243650547364*^9, 3.838243731575823*^9, 3.838243868032824*^9, {
   3.838243899763178*^9, 3.8382439163109407`*^9}, {3.838243954781808*^9, 
   3.8382440260526943`*^9}, 3.838245032176836*^9, 3.838245098972673*^9, 
   3.838245182091054*^9, 3.8382452484492073`*^9, 3.838245297611104*^9, 
   3.838245415500739*^9, 3.838245446111436*^9, 3.838246355068489*^9, 
   3.838246503154478*^9, 3.838246589405971*^9, {3.83824662507314*^9, 
   3.838246651501807*^9}, 3.8382486965885572`*^9, 3.838250618563526*^9, 
   3.8382509383063927`*^9, 3.8382511245976257`*^9, 3.838251160512944*^9, 
   3.8382520710228453`*^9, 3.8382525394049263`*^9, 3.8382742548772783`*^9, {
   3.838274288173587*^9, 3.8382743160545673`*^9}, 3.838274411170527*^9, 
   3.8382751035065203`*^9, {3.8382751647677727`*^9, 3.8382751826597958`*^9}, 
   3.838275295997991*^9, 3.838275364870254*^9, 3.838275464962778*^9, 
   3.8382755073576527`*^9, 3.838275667551221*^9, 3.838276165497401*^9, 
   3.838276201673314*^9, {3.83827641718303*^9, 3.838276440897884*^9}, 
   3.838276488090859*^9, 3.8382805284765873`*^9, 3.8382905004614964`*^9, 
   3.8382923580082407`*^9, 3.8383031741500473`*^9, 3.838398738541813*^9, 
   3.838701774531659*^9, 3.838801482781333*^9, 3.838801517151835*^9},
 CellLabel->
  "Out[243]=",ExpressionUUID->"2ccfce55-5efe-48e9-a653-0ab8545055de"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"x2", "\[GreaterEqual]", "0"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], "+", 
     RowBox[{
      SqrtBox["2"], " ", "x2"}]}], "\[LessEqual]", "0"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     RowBox[{
      SqrtBox["2"], " ", "x2"}]}], "\[LessEqual]", "0"}], ",", 
   RowBox[{
    RowBox[{"1", "+", 
     RowBox[{
      SqrtBox["2"], " ", "x2"}]}], "\[GreaterEqual]", "0"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.838135855284397*^9, 3.838135935307404*^9, 3.838135987630686*^9, 
   3.838136079962658*^9, 3.8381361558364697`*^9, 3.8381362142647333`*^9, 
   3.838136345509656*^9, 3.838136417987011*^9, {3.8381364537661333`*^9, 
   3.838136476321108*^9}, 3.8381382910158577`*^9, {3.838138452598276*^9, 
   3.838138492567541*^9}, {3.838138604607637*^9, 3.8381386167721033`*^9}, 
   3.8381928263462467`*^9, 3.8381934663838053`*^9, {3.838193522883288*^9, 
   3.8381935681527147`*^9}, {3.838193599430602*^9, 3.838193629220193*^9}, 
   3.8381936882508783`*^9, {3.838193796809009*^9, 3.8381938185430107`*^9}, 
   3.838193866097094*^9, 3.838194191029058*^9, 3.838194791090049*^9, 
   3.838195118658203*^9, 3.838195152202137*^9, 3.838195223208246*^9, {
   3.838195321701625*^9, 3.8381953376093082`*^9}, 3.838195379655159*^9, {
   3.838195445071327*^9, 3.838195461955699*^9}, {3.8381954955702353`*^9, 
   3.8381955121655188`*^9}, 3.838195598626733*^9, {3.838195628666795*^9, 
   3.8381956580540648`*^9}, 3.838196592195594*^9, {3.838196817124673*^9, 
   3.83819683393499*^9}, 3.8381969594094048`*^9, 3.83819704735888*^9, 
   3.83819708733041*^9, {3.83819736012643*^9, 3.83819740238835*^9}, {
   3.838197454616816*^9, 3.838197471253503*^9}, 3.8381975030543127`*^9, 
   3.8381975879979362`*^9, 3.838197630199223*^9, 3.8381977594094877`*^9, {
   3.838199516979061*^9, 3.8381995409289923`*^9}, {3.838199585941297*^9, 
   3.838199626978765*^9}, 3.8382001929688396`*^9, 3.838200233427125*^9, 
   3.838200332052126*^9, {3.8382004001240187`*^9, 3.838200434020404*^9}, 
   3.8382004971617317`*^9, 3.8382005667855597`*^9, 3.838200658025077*^9, {
   3.8382007712345333`*^9, 3.838200789961648*^9}, 3.838200824989113*^9, {
   3.838200988561356*^9, 3.838201028673361*^9}, 3.838201078242505*^9, {
   3.8382011430217113`*^9, 3.838201201316042*^9}, {3.838201260133481*^9, 
   3.838201278943379*^9}, {3.838201309745639*^9, 3.838201319001542*^9}, 
   3.83820135851289*^9, 3.8382014462060137`*^9, {3.838201853911603*^9, 
   3.838201867761775*^9}, 3.8382024675638857`*^9, 3.838204525932125*^9, 
   3.8382052629022617`*^9, 3.8382081903776407`*^9, 3.838239084298512*^9, {
   3.838239219948361*^9, 3.838239249250147*^9}, {3.838239529016798*^9, 
   3.838239549815136*^9}, 3.838239810192692*^9, 3.83824359845688*^9, 
   3.838243650547364*^9, 3.838243731575823*^9, 3.838243868032824*^9, {
   3.838243899763178*^9, 3.8382439163109407`*^9}, {3.838243954781808*^9, 
   3.8382440260526943`*^9}, 3.838245032176836*^9, 3.838245098972673*^9, 
   3.838245182091054*^9, 3.8382452484492073`*^9, 3.838245297611104*^9, 
   3.838245415500739*^9, 3.838245446111436*^9, 3.838246355068489*^9, 
   3.838246503154478*^9, 3.838246589405971*^9, {3.83824662507314*^9, 
   3.838246651501807*^9}, 3.8382486965885572`*^9, 3.838250618563526*^9, 
   3.8382509383063927`*^9, 3.8382511245976257`*^9, 3.838251160512944*^9, 
   3.8382520710228453`*^9, 3.8382525394049263`*^9, 3.8382742548772783`*^9, {
   3.838274288173587*^9, 3.8382743160545673`*^9}, 3.838274411170527*^9, 
   3.8382751035065203`*^9, {3.8382751647677727`*^9, 3.8382751826597958`*^9}, 
   3.838275295997991*^9, 3.838275364870254*^9, 3.838275464962778*^9, 
   3.8382755073576527`*^9, 3.838275667551221*^9, 3.838276165497401*^9, 
   3.838276201673314*^9, {3.83827641718303*^9, 3.838276440897884*^9}, 
   3.838276488090859*^9, 3.8382805284765873`*^9, 3.8382905004614964`*^9, 
   3.8382923580082407`*^9, 3.8383031741500473`*^9, 3.838398738541813*^9, 
   3.838701774531659*^9, 3.838801482781333*^9, 3.838801517228179*^9},
 CellLabel->
  "Out[245]=",ExpressionUUID->"06c0abba-de8b-4361-b980-ff0223dc329a"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     SqrtBox["2"], " ", "x2"}], "\[GreaterEqual]", "0"}], ",", 
   RowBox[{
    RowBox[{"2", "-", 
     RowBox[{
      SqrtBox["2"], " ", "x2"}]}], "\[GreaterEqual]", "0"}], ",", 
   RowBox[{
    RowBox[{
     FractionBox["1", "2"], "-", 
     FractionBox["x2", 
      SqrtBox["2"]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{
         SqrtBox["2"], " ", "x2"}]}], ")"}]}]}], "\[GreaterEqual]", "0"}], 
   ",", 
   RowBox[{
    RowBox[{
     FractionBox["1", "2"], "-", 
     FractionBox["x2", 
      SqrtBox["2"]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", 
        RowBox[{
         SqrtBox["2"], " ", "x2"}]}], ")"}]}]}], "\[GreaterEqual]", "0"}], 
   ",", 
   RowBox[{
    RowBox[{
     FractionBox["1", "2"], "+", 
     FractionBox["x2", 
      SqrtBox["2"]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", 
        RowBox[{
         SqrtBox["2"], " ", "x2"}]}], ")"}]}]}], "\[GreaterEqual]", "0"}], 
   ",", 
   RowBox[{
    RowBox[{
     FractionBox["1", "2"], "+", 
     FractionBox["x2", 
      SqrtBox["2"]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{
         SqrtBox["2"], " ", "x2"}]}], ")"}]}]}], "\[GreaterEqual]", "0"}], 
   ",", 
   RowBox[{
    RowBox[{
     FractionBox["3", "2"], "-", 
     FractionBox["x2", 
      SqrtBox["2"]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{
         SqrtBox["2"], " ", "x2"}]}], ")"}]}]}], "\[GreaterEqual]", "0"}], 
   ",", 
   RowBox[{
    RowBox[{
     FractionBox["3", "2"], "-", 
     FractionBox["x2", 
      SqrtBox["2"]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", 
        RowBox[{
         SqrtBox["2"], " ", "x2"}]}], ")"}]}]}], "\[GreaterEqual]", "0"}], 
   ",", 
   RowBox[{
    RowBox[{
     FractionBox["3", "2"], "+", 
     FractionBox["x2", 
      SqrtBox["2"]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{
         SqrtBox["2"], " ", "x2"}]}], ")"}]}]}], "\[GreaterEqual]", "0"}], 
   ",", 
   RowBox[{
    RowBox[{
     FractionBox["3", "2"], "+", 
     FractionBox["x2", 
      SqrtBox["2"]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", 
        RowBox[{
         SqrtBox["2"], " ", "x2"}]}], ")"}]}]}], "\[GreaterEqual]", "0"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.838135855284397*^9, 3.838135935307404*^9, 3.838135987630686*^9, 
   3.838136079962658*^9, 3.8381361558364697`*^9, 3.8381362142647333`*^9, 
   3.838136345509656*^9, 3.838136417987011*^9, {3.8381364537661333`*^9, 
   3.838136476321108*^9}, 3.8381382910158577`*^9, {3.838138452598276*^9, 
   3.838138492567541*^9}, {3.838138604607637*^9, 3.8381386167721033`*^9}, 
   3.8381928263462467`*^9, 3.8381934663838053`*^9, {3.838193522883288*^9, 
   3.8381935681527147`*^9}, {3.838193599430602*^9, 3.838193629220193*^9}, 
   3.8381936882508783`*^9, {3.838193796809009*^9, 3.8381938185430107`*^9}, 
   3.838193866097094*^9, 3.838194191029058*^9, 3.838194791090049*^9, 
   3.838195118658203*^9, 3.838195152202137*^9, 3.838195223208246*^9, {
   3.838195321701625*^9, 3.8381953376093082`*^9}, 3.838195379655159*^9, {
   3.838195445071327*^9, 3.838195461955699*^9}, {3.8381954955702353`*^9, 
   3.8381955121655188`*^9}, 3.838195598626733*^9, {3.838195628666795*^9, 
   3.8381956580540648`*^9}, 3.838196592195594*^9, {3.838196817124673*^9, 
   3.83819683393499*^9}, 3.8381969594094048`*^9, 3.83819704735888*^9, 
   3.83819708733041*^9, {3.83819736012643*^9, 3.83819740238835*^9}, {
   3.838197454616816*^9, 3.838197471253503*^9}, 3.8381975030543127`*^9, 
   3.8381975879979362`*^9, 3.838197630199223*^9, 3.8381977594094877`*^9, {
   3.838199516979061*^9, 3.8381995409289923`*^9}, {3.838199585941297*^9, 
   3.838199626978765*^9}, 3.8382001929688396`*^9, 3.838200233427125*^9, 
   3.838200332052126*^9, {3.8382004001240187`*^9, 3.838200434020404*^9}, 
   3.8382004971617317`*^9, 3.8382005667855597`*^9, 3.838200658025077*^9, {
   3.8382007712345333`*^9, 3.838200789961648*^9}, 3.838200824989113*^9, {
   3.838200988561356*^9, 3.838201028673361*^9}, 3.838201078242505*^9, {
   3.8382011430217113`*^9, 3.838201201316042*^9}, {3.838201260133481*^9, 
   3.838201278943379*^9}, {3.838201309745639*^9, 3.838201319001542*^9}, 
   3.83820135851289*^9, 3.8382014462060137`*^9, {3.838201853911603*^9, 
   3.838201867761775*^9}, 3.8382024675638857`*^9, 3.838204525932125*^9, 
   3.8382052629022617`*^9, 3.8382081903776407`*^9, 3.838239084298512*^9, {
   3.838239219948361*^9, 3.838239249250147*^9}, {3.838239529016798*^9, 
   3.838239549815136*^9}, 3.838239810192692*^9, 3.83824359845688*^9, 
   3.838243650547364*^9, 3.838243731575823*^9, 3.838243868032824*^9, {
   3.838243899763178*^9, 3.8382439163109407`*^9}, {3.838243954781808*^9, 
   3.8382440260526943`*^9}, 3.838245032176836*^9, 3.838245098972673*^9, 
   3.838245182091054*^9, 3.8382452484492073`*^9, 3.838245297611104*^9, 
   3.838245415500739*^9, 3.838245446111436*^9, 3.838246355068489*^9, 
   3.838246503154478*^9, 3.838246589405971*^9, {3.83824662507314*^9, 
   3.838246651501807*^9}, 3.8382486965885572`*^9, 3.838250618563526*^9, 
   3.8382509383063927`*^9, 3.8382511245976257`*^9, 3.838251160512944*^9, 
   3.8382520710228453`*^9, 3.8382525394049263`*^9, 3.8382742548772783`*^9, {
   3.838274288173587*^9, 3.8382743160545673`*^9}, 3.838274411170527*^9, 
   3.8382751035065203`*^9, {3.8382751647677727`*^9, 3.8382751826597958`*^9}, 
   3.838275295997991*^9, 3.838275364870254*^9, 3.838275464962778*^9, 
   3.8382755073576527`*^9, 3.838275667551221*^9, 3.838276165497401*^9, 
   3.838276201673314*^9, {3.83827641718303*^9, 3.838276440897884*^9}, 
   3.838276488090859*^9, 3.8382805284765873`*^9, 3.8382905004614964`*^9, 
   3.8382923580082407`*^9, 3.8383031741500473`*^9, 3.838398738541813*^9, 
   3.838701774531659*^9, 3.838801482781333*^9, 3.8388015173185997`*^9},
 CellLabel->
  "Out[246]=",ExpressionUUID->"8c63861f-1c05-4a86-9c12-1410ca1ce0e4"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", 
     RowBox[{"-", 
      SqrtBox["2"]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     RowBox[{"-", 
      SqrtBox["2"]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     SqrtBox["2"]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.838135855284397*^9, 3.838135935307404*^9, 3.838135987630686*^9, 
   3.838136079962658*^9, 3.8381361558364697`*^9, 3.8381362142647333`*^9, 
   3.838136345509656*^9, 3.838136417987011*^9, {3.8381364537661333`*^9, 
   3.838136476321108*^9}, 3.8381382910158577`*^9, {3.838138452598276*^9, 
   3.838138492567541*^9}, {3.838138604607637*^9, 3.8381386167721033`*^9}, 
   3.8381928263462467`*^9, 3.8381934663838053`*^9, {3.838193522883288*^9, 
   3.8381935681527147`*^9}, {3.838193599430602*^9, 3.838193629220193*^9}, 
   3.8381936882508783`*^9, {3.838193796809009*^9, 3.8381938185430107`*^9}, 
   3.838193866097094*^9, 3.838194191029058*^9, 3.838194791090049*^9, 
   3.838195118658203*^9, 3.838195152202137*^9, 3.838195223208246*^9, {
   3.838195321701625*^9, 3.8381953376093082`*^9}, 3.838195379655159*^9, {
   3.838195445071327*^9, 3.838195461955699*^9}, {3.8381954955702353`*^9, 
   3.8381955121655188`*^9}, 3.838195598626733*^9, {3.838195628666795*^9, 
   3.8381956580540648`*^9}, 3.838196592195594*^9, {3.838196817124673*^9, 
   3.83819683393499*^9}, 3.8381969594094048`*^9, 3.83819704735888*^9, 
   3.83819708733041*^9, {3.83819736012643*^9, 3.83819740238835*^9}, {
   3.838197454616816*^9, 3.838197471253503*^9}, 3.8381975030543127`*^9, 
   3.8381975879979362`*^9, 3.838197630199223*^9, 3.8381977594094877`*^9, {
   3.838199516979061*^9, 3.8381995409289923`*^9}, {3.838199585941297*^9, 
   3.838199626978765*^9}, 3.8382001929688396`*^9, 3.838200233427125*^9, 
   3.838200332052126*^9, {3.8382004001240187`*^9, 3.838200434020404*^9}, 
   3.8382004971617317`*^9, 3.8382005667855597`*^9, 3.838200658025077*^9, {
   3.8382007712345333`*^9, 3.838200789961648*^9}, 3.838200824989113*^9, {
   3.838200988561356*^9, 3.838201028673361*^9}, 3.838201078242505*^9, {
   3.8382011430217113`*^9, 3.838201201316042*^9}, {3.838201260133481*^9, 
   3.838201278943379*^9}, {3.838201309745639*^9, 3.838201319001542*^9}, 
   3.83820135851289*^9, 3.8382014462060137`*^9, {3.838201853911603*^9, 
   3.838201867761775*^9}, 3.8382024675638857`*^9, 3.838204525932125*^9, 
   3.8382052629022617`*^9, 3.8382081903776407`*^9, 3.838239084298512*^9, {
   3.838239219948361*^9, 3.838239249250147*^9}, {3.838239529016798*^9, 
   3.838239549815136*^9}, 3.838239810192692*^9, 3.83824359845688*^9, 
   3.838243650547364*^9, 3.838243731575823*^9, 3.838243868032824*^9, {
   3.838243899763178*^9, 3.8382439163109407`*^9}, {3.838243954781808*^9, 
   3.8382440260526943`*^9}, 3.838245032176836*^9, 3.838245098972673*^9, 
   3.838245182091054*^9, 3.8382452484492073`*^9, 3.838245297611104*^9, 
   3.838245415500739*^9, 3.838245446111436*^9, 3.838246355068489*^9, 
   3.838246503154478*^9, 3.838246589405971*^9, {3.83824662507314*^9, 
   3.838246651501807*^9}, 3.8382486965885572`*^9, 3.838250618563526*^9, 
   3.8382509383063927`*^9, 3.8382511245976257`*^9, 3.838251160512944*^9, 
   3.8382520710228453`*^9, 3.8382525394049263`*^9, 3.8382742548772783`*^9, {
   3.838274288173587*^9, 3.8382743160545673`*^9}, 3.838274411170527*^9, 
   3.8382751035065203`*^9, {3.8382751647677727`*^9, 3.8382751826597958`*^9}, 
   3.838275295997991*^9, 3.838275364870254*^9, 3.838275464962778*^9, 
   3.8382755073576527`*^9, 3.838275667551221*^9, 3.838276165497401*^9, 
   3.838276201673314*^9, {3.83827641718303*^9, 3.838276440897884*^9}, 
   3.838276488090859*^9, 3.8382805284765873`*^9, 3.8382905004614964`*^9, 
   3.8382923580082407`*^9, 3.8383031741500473`*^9, 3.838398738541813*^9, 
   3.838701774531659*^9, 3.838801482781333*^9, 3.8388015173901052`*^9},
 CellLabel->
  "Out[247]=",ExpressionUUID->"968ef69c-c648-4961-af70-68b2e38c8227"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     SqrtBox["2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", 
     RowBox[{"-", 
      SqrtBox["2"]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     RowBox[{"-", 
      SqrtBox["2"]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     SqrtBox["2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", 
     RowBox[{"-", 
      SqrtBox["2"]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     SqrtBox["2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.838135855284397*^9, 3.838135935307404*^9, 3.838135987630686*^9, 
   3.838136079962658*^9, 3.8381361558364697`*^9, 3.8381362142647333`*^9, 
   3.838136345509656*^9, 3.838136417987011*^9, {3.8381364537661333`*^9, 
   3.838136476321108*^9}, 3.8381382910158577`*^9, {3.838138452598276*^9, 
   3.838138492567541*^9}, {3.838138604607637*^9, 3.8381386167721033`*^9}, 
   3.8381928263462467`*^9, 3.8381934663838053`*^9, {3.838193522883288*^9, 
   3.8381935681527147`*^9}, {3.838193599430602*^9, 3.838193629220193*^9}, 
   3.8381936882508783`*^9, {3.838193796809009*^9, 3.8381938185430107`*^9}, 
   3.838193866097094*^9, 3.838194191029058*^9, 3.838194791090049*^9, 
   3.838195118658203*^9, 3.838195152202137*^9, 3.838195223208246*^9, {
   3.838195321701625*^9, 3.8381953376093082`*^9}, 3.838195379655159*^9, {
   3.838195445071327*^9, 3.838195461955699*^9}, {3.8381954955702353`*^9, 
   3.8381955121655188`*^9}, 3.838195598626733*^9, {3.838195628666795*^9, 
   3.8381956580540648`*^9}, 3.838196592195594*^9, {3.838196817124673*^9, 
   3.83819683393499*^9}, 3.8381969594094048`*^9, 3.83819704735888*^9, 
   3.83819708733041*^9, {3.83819736012643*^9, 3.83819740238835*^9}, {
   3.838197454616816*^9, 3.838197471253503*^9}, 3.8381975030543127`*^9, 
   3.8381975879979362`*^9, 3.838197630199223*^9, 3.8381977594094877`*^9, {
   3.838199516979061*^9, 3.8381995409289923`*^9}, {3.838199585941297*^9, 
   3.838199626978765*^9}, 3.8382001929688396`*^9, 3.838200233427125*^9, 
   3.838200332052126*^9, {3.8382004001240187`*^9, 3.838200434020404*^9}, 
   3.8382004971617317`*^9, 3.8382005667855597`*^9, 3.838200658025077*^9, {
   3.8382007712345333`*^9, 3.838200789961648*^9}, 3.838200824989113*^9, {
   3.838200988561356*^9, 3.838201028673361*^9}, 3.838201078242505*^9, {
   3.8382011430217113`*^9, 3.838201201316042*^9}, {3.838201260133481*^9, 
   3.838201278943379*^9}, {3.838201309745639*^9, 3.838201319001542*^9}, 
   3.83820135851289*^9, 3.8382014462060137`*^9, {3.838201853911603*^9, 
   3.838201867761775*^9}, 3.8382024675638857`*^9, 3.838204525932125*^9, 
   3.8382052629022617`*^9, 3.8382081903776407`*^9, 3.838239084298512*^9, {
   3.838239219948361*^9, 3.838239249250147*^9}, {3.838239529016798*^9, 
   3.838239549815136*^9}, 3.838239810192692*^9, 3.83824359845688*^9, 
   3.838243650547364*^9, 3.838243731575823*^9, 3.838243868032824*^9, {
   3.838243899763178*^9, 3.8382439163109407`*^9}, {3.838243954781808*^9, 
   3.8382440260526943`*^9}, 3.838245032176836*^9, 3.838245098972673*^9, 
   3.838245182091054*^9, 3.8382452484492073`*^9, 3.838245297611104*^9, 
   3.838245415500739*^9, 3.838245446111436*^9, 3.838246355068489*^9, 
   3.838246503154478*^9, 3.838246589405971*^9, {3.83824662507314*^9, 
   3.838246651501807*^9}, 3.8382486965885572`*^9, 3.838250618563526*^9, 
   3.8382509383063927`*^9, 3.8382511245976257`*^9, 3.838251160512944*^9, 
   3.8382520710228453`*^9, 3.8382525394049263`*^9, 3.8382742548772783`*^9, {
   3.838274288173587*^9, 3.8382743160545673`*^9}, 3.838274411170527*^9, 
   3.8382751035065203`*^9, {3.8382751647677727`*^9, 3.8382751826597958`*^9}, 
   3.838275295997991*^9, 3.838275364870254*^9, 3.838275464962778*^9, 
   3.8382755073576527`*^9, 3.838275667551221*^9, 3.838276165497401*^9, 
   3.838276201673314*^9, {3.83827641718303*^9, 3.838276440897884*^9}, 
   3.838276488090859*^9, 3.8382805284765873`*^9, 3.8382905004614964`*^9, 
   3.8382923580082407`*^9, 3.8383031741500473`*^9, 3.838398738541813*^9, 
   3.838701774531659*^9, 3.838801482781333*^9, 3.838801517455554*^9},
 CellLabel->
  "Out[248]=",ExpressionUUID->"324c58b5-587e-44e0-a2e3-482f53b3a63a"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"N", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"eachESingleDeltaInt", "[", "j", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}], "+", 
      RowBox[{"Total", "[", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"eachEDoubleDeltaInt", "[", 
           RowBox[{"j", ",", "k"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", "1", ",", "sites"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", 
            RowBox[{"j", "+", "1"}], ",", "sites"}], "}"}]}], "]"}], ",", 
        RowBox[{"{", "2", "}"}]}], "]"}]}], ",", "16"}], "]"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"eachEDoubleDeltaInt", "[", 
      RowBox[{"j", ",", "k"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "sites"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"j", "+", "1"}], ",", "sites"}], "}"}]}], "]"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"eachESingleDeltaInt", "[", "j", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Int12", " ", "=", " ", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{"Total", "[", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"eachESingleDeltaInt", "[", "j", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}], "+", 
       RowBox[{"Total", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"eachEDoubleDeltaInt", "[", 
            RowBox[{"j", ",", "k"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"j", ",", "1", ",", "sites"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", 
             RowBox[{"j", "+", "1"}], ",", "sites"}], "}"}]}], "]"}], ",", 
         RowBox[{"{", "2", "}"}]}], "]"}]}], "]"}], "]"}]}], 
   "\[IndentingNewLine]", 
   RowBox[{"N", "[", 
    RowBox[{"Int12", ",", "16"}], "]"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     SubscriptBox["E", "1"], "=", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"Int12", "/", "Int10"}], ")"}]}], "+", 
      RowBox[{
       FractionBox["1", "2"], "*", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"Int11", "/", "Int10"}], ")"}], "2"]}]}]}], " ", ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"N", "[", 
    SubscriptBox["E", "1"], "]"}], "\[IndentingNewLine]", 
   RowBox[{"N", "[", 
    RowBox[{
     FractionBox[
      RowBox[{
       SuperscriptBox["d", "2"], "-", "1"}], "12"], "-", 
     RowBox[{"d", "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           SubscriptBox["D", "1"], "d"], "+", 
          FractionBox[
           RowBox[{"(", 
            RowBox[{"d", "-", "1"}], ")"}], "2"]}], ")"}], "+", 
        RowBox[{"2", "*", 
         RowBox[{"(", 
          RowBox[{
           FractionBox[
            SubscriptBox["E", "1"], 
            SuperscriptBox["d", "2"]], "-", 
           RowBox[{
            FractionBox[
             RowBox[{"d", "+", "1"}], 
             RowBox[{"2", "*", 
              SuperscriptBox["d", "2"]}]], "*", 
            SubscriptBox["D", "1"]}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"5", "*", "d"}], "-", "1"}], ")"}], "*", 
             RowBox[{"(", 
              RowBox[{"1", "-", "d"}], ")"}]}], 
            RowBox[{"24", "*", "d"}]]}], ")"}]}]}], ")"}]}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.837759051812975*^9, 3.837759071591836*^9}, {
   3.837759217318631*^9, 3.837759241400175*^9}, {3.837759320301201*^9, 
   3.83775936716477*^9}, {3.837759456303917*^9, 3.837759642000123*^9}, {
   3.837759816966599*^9, 3.83775991155928*^9}, {3.837759956835602*^9, 
   3.8377599789804487`*^9}, {3.837760016652355*^9, 3.837760096420809*^9}, {
   3.837760140717391*^9, 3.837760207546588*^9}, {3.8377602481030903`*^9, 
   3.8377602740980864`*^9}, {3.837760333242087*^9, 3.837760427623968*^9}, {
   3.8377604636283073`*^9, 3.8377607453888807`*^9}, {3.837760787735127*^9, 
   3.837760793341056*^9}, {3.837761056684053*^9, 3.837761100969037*^9}, {
   3.8377615513819017`*^9, 3.83776158670927*^9}, {3.837761621261744*^9, 
   3.8377616531407127`*^9}, {3.837761693987907*^9, 3.837761729898768*^9}, {
   3.837761844581964*^9, 3.837761970006235*^9}, {3.837762073595648*^9, 
   3.837762096290968*^9}, {3.837762142678879*^9, 3.8377622375066757`*^9}, {
   3.8377622979149218`*^9, 3.837762405543975*^9}, {3.83776251457353*^9, 
   3.837762566781784*^9}, {3.837762618932023*^9, 3.8377626637366657`*^9}, {
   3.837763509203718*^9, 3.837763591558241*^9}, {3.83776364159385*^9, 
   3.83776367456262*^9}, {3.837763759909724*^9, 3.8377638110846853`*^9}, {
   3.8377638908917513`*^9, 3.8377639078480997`*^9}, {3.837763980843951*^9, 
   3.837764177471188*^9}, {3.8377642380320673`*^9, 3.837764360041239*^9}, {
   3.837764442964953*^9, 3.837764502687722*^9}, {3.83776455233293*^9, 
   3.8377646335315657`*^9}, {3.837764699266444*^9, 3.8377647272120543`*^9}, {
   3.837764769287674*^9, 3.837765008276999*^9}, {3.837765052599174*^9, 
   3.837765057200159*^9}, {3.837765092612886*^9, 3.837765334368432*^9}, {
   3.8377654308449802`*^9, 3.8377655319772167`*^9}, {3.837765568969681*^9, 
   3.837765661871271*^9}, {3.837765791941186*^9, 3.837765912988414*^9}, {
   3.8377660145740337`*^9, 3.8377661776828814`*^9}, {3.837766237263646*^9, 
   3.837766278805069*^9}, {3.8377663100115967`*^9, 3.837766329618359*^9}, {
   3.837766657496814*^9, 3.837766657731106*^9}, {3.83776715547604*^9, 
   3.8377671565101347`*^9}, {3.8377674335426807`*^9, 3.8377674351976757`*^9}, 
   3.837768112932695*^9, {3.837768376845517*^9, 3.837768449341558*^9}, 
   3.837768484719595*^9, {3.837768542966751*^9, 3.837768557770595*^9}, {
   3.8377686335259457`*^9, 3.837768639317741*^9}, {3.837769274391078*^9, 
   3.8377693524395447`*^9}, {3.837769497420038*^9, 3.8377695037909527`*^9}, {
   3.837769596022748*^9, 3.8377696087576*^9}, {3.837769647025366*^9, 
   3.837769761571034*^9}, {3.83776983151989*^9, 3.8377699867015038`*^9}, {
   3.837770952577222*^9, 3.837771031706276*^9}, {3.837771104943639*^9, 
   3.837771165144917*^9}, 3.8377712134152813`*^9, {3.837771255405746*^9, 
   3.837771342336483*^9}, {3.837771373782195*^9, 3.837771500315425*^9}, {
   3.8377715318583117`*^9, 3.837771603465426*^9}, 3.83777165452458*^9, {
   3.837771689129313*^9, 3.8377717429592857`*^9}, {3.8377717947541027`*^9, 
   3.837772161043878*^9}, {3.837772205636837*^9, 3.837772226479508*^9}, {
   3.8377722800771313`*^9, 3.8377722844518147`*^9}, {3.837772316184485*^9, 
   3.837772330120789*^9}, {3.8377723601984863`*^9, 3.837772389931653*^9}, {
   3.837772440094946*^9, 3.8377724717356987`*^9}, {3.837772505324279*^9, 
   3.837772849864531*^9}, {3.837772896947722*^9, 3.837773036687307*^9}, {
   3.8377731147022753`*^9, 3.837773227864345*^9}, {3.837773275727414*^9, 
   3.837773298163156*^9}, {3.837773347035487*^9, 3.83777335522159*^9}, {
   3.837773412888567*^9, 3.837773475740651*^9}, {3.837773530112762*^9, 
   3.8377735931395397`*^9}, {3.837773630868861*^9, 3.837773822688457*^9}, {
   3.837773878077608*^9, 3.8377738872682123`*^9}, {3.837773919005477*^9, 
   3.837774000967046*^9}, {3.8377740453237867`*^9, 3.837774049124507*^9}, {
   3.837774085611842*^9, 3.837774223315514*^9}, 3.837774263547697*^9, {
   3.8377743121800003`*^9, 3.8377743126575537`*^9}, {3.837774496089884*^9, 
   3.837774503826468*^9}, {3.8377745574267807`*^9, 3.837774895068288*^9}, {
   3.837774940831119*^9, 3.8377750491080093`*^9}, {3.8377751090428333`*^9, 
   3.837775144213842*^9}, {3.8377751951433*^9, 3.837775305357629*^9}, {
   3.837775364894342*^9, 3.837775496742807*^9}, {3.8377755338131523`*^9, 
   3.8377755753185453`*^9}, {3.8377756277767887`*^9, 3.837775726961532*^9}, {
   3.8377757758014393`*^9, 3.837775839367248*^9}, {3.837775880521937*^9, 
   3.837775911455641*^9}, {3.837775942504963*^9, 3.837775964469473*^9}, {
   3.837775996413103*^9, 3.837776188288731*^9}, {3.837776234433379*^9, 
   3.837776350471487*^9}, {3.837776493128147*^9, 3.837776540687028*^9}, 
   3.8377766164973783`*^9, {3.837776652131136*^9, 3.8377766581985807`*^9}, {
   3.837776701059732*^9, 3.837776709453355*^9}, {3.83777678203025*^9, 
   3.83777678709515*^9}, {3.837776820532475*^9, 3.8377768331493797`*^9}, {
   3.837776956286186*^9, 3.837777064590246*^9}, {3.837777101706354*^9, 
   3.837777205573284*^9}, {3.837777272769291*^9, 3.837777673512064*^9}, {
   3.837777724691207*^9, 3.837777748824847*^9}, {3.837777879292939*^9, 
   3.837777898982292*^9}, {3.837777937064353*^9, 3.837777973922823*^9}, 
   3.837778019767487*^9, {3.837778050456066*^9, 3.837778147495962*^9}, {
   3.8377795716200027`*^9, 3.837779600678067*^9}, 3.837779651879637*^9, {
   3.8377796862719593`*^9, 3.837779717452261*^9}, {3.8377797614372*^9, 
   3.837779775519079*^9}, {3.837779972163884*^9, 3.8377799740534887`*^9}, {
   3.837780017640114*^9, 3.837780024681024*^9}, {3.837780057444573*^9, 
   3.837780080083229*^9}, 3.8377803804440327`*^9, {3.8377805107291203`*^9, 
   3.837780566161954*^9}, {3.837780637776647*^9, 3.837780822613832*^9}, {
   3.837780872466659*^9, 3.8377809327605352`*^9}, {3.837780992367846*^9, 
   3.837781005509432*^9}, {3.8377810726487713`*^9, 3.837781088451127*^9}, 
   3.8377811295651197`*^9, {3.837781739704506*^9, 3.837781750649578*^9}, {
   3.837781898408183*^9, 3.8377819488022537`*^9}, {3.8377819864007177`*^9, 
   3.837781992270015*^9}, {3.837782024170294*^9, 3.8377821019794483`*^9}, {
   3.837782142737905*^9, 3.837782162036997*^9}, {3.837782354378776*^9, 
   3.837782448482431*^9}, {3.8377848682062187`*^9, 3.8377848913130713`*^9}, {
   3.837784930076548*^9, 3.8377849415859213`*^9}, {3.837785645716629*^9, 
   3.837785678588539*^9}, {3.837785728736228*^9, 3.837785763713437*^9}, 
   3.837786034233245*^9, {3.837786066746749*^9, 3.837786116126554*^9}, {
   3.837861251941223*^9, 3.83786128903496*^9}, {3.837861349162579*^9, 
   3.83786137538763*^9}, {3.837861441354437*^9, 3.8378614479760733`*^9}, {
   3.83786148029738*^9, 3.837861505582625*^9}, 3.837861752973679*^9, {
   3.837862755674859*^9, 3.837862763059555*^9}, {3.8380758896068077`*^9, 
   3.8380759297842407`*^9}, {3.8380975980153093`*^9, 3.838097663482284*^9}, {
   3.8380977510926313`*^9, 3.838097788509506*^9}, {3.8380978601000767`*^9, 
   3.8380978646323233`*^9}, {3.8380980026732683`*^9, 
   3.8380980036038847`*^9}, {3.838098036220108*^9, 3.838098064053454*^9}, {
   3.838112651834959*^9, 3.8381126815227118`*^9}, 3.838114667604498*^9, {
   3.8381157640608788`*^9, 3.838115813481683*^9}, {3.838115854554874*^9, 
   3.838115890462085*^9}, {3.8381159232655077`*^9, 3.83811621862961*^9}, 
   3.838116376202277*^9, {3.838116502668528*^9, 3.838116528605359*^9}, {
   3.838116559555479*^9, 3.838116567161934*^9}, {3.8381167114662952`*^9, 
   3.83811672720665*^9}, {3.838116871218288*^9, 3.838116928407834*^9}, {
   3.8381169723037233`*^9, 3.838117176849574*^9}, {3.838117211210527*^9, 
   3.838117337049486*^9}, {3.838117400110519*^9, 3.8381174104287243`*^9}, {
   3.838117804814328*^9, 3.838117811366661*^9}, 3.838117971393291*^9, {
   3.8381227180652122`*^9, 3.8381227498561077`*^9}, {3.838124102605196*^9, 
   3.838124206619174*^9}, {3.838124275844899*^9, 3.838124275966592*^9}, {
   3.838124325369768*^9, 3.838124344329311*^9}, {3.838130777937604*^9, 
   3.838130778012775*^9}, {3.83813082658917*^9, 3.8381308481075706`*^9}, {
   3.838130928486267*^9, 3.838130940324314*^9}, {3.838202755908938*^9, 
   3.838202801330179*^9}, {3.838239290506819*^9, 3.83823933393834*^9}, {
   3.8382394002920723`*^9, 3.8382394350566483`*^9}, {3.8382396271973047`*^9, 
   3.838239641862195*^9}, {3.838254307647674*^9, 3.8382543328823*^9}, {
   3.838254577030506*^9, 3.8382546088446693`*^9}, 3.838254837430193*^9, {
   3.838398788180336*^9, 3.8383987943818703`*^9}, {3.838398828585643*^9, 
   3.838398861138983*^9}},
 CellLabel->
  "In[249]:=",ExpressionUUID->"6c19dc58-4b71-4df9-837b-97f66171c8c6"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.75424723326565069269423398624517230857`16.", ",", 
   "0.75424723326565069269423398624517230857`16.", ",", 
   "0.75424723326565069269423398624517230857`16.", ",", 
   "0.75424723326565069269423398624517230857`16.", ",", 
   "0.55390031192946222744732808364879841411`16.", ",", 
   "0.45372685126136799482387513235061146687`16.", ",", 
   "0.55390031192946222744732808364879841411`16.", ",", 
   "0.45372685126136799482387513235061146687`16.", ",", 
   "0.45372685126136799482387513235061146687`16.", ",", 
   "0.35355339059327376220042218105242451964`16.", ",", 
   "0.45372685126136799482387513235061146687`16.", ",", 
   "0.35355339059327376220042218105242451964`16."}], "}"}]], "Output",
 CellChangeTimes->{
  3.8381146728726797`*^9, 3.838116234664516*^9, 3.8381164247953653`*^9, {
   3.838116541558178*^9, 3.8381165708372107`*^9}, 3.838116675727318*^9, 
   3.8381167305415916`*^9, {3.83811688163279*^9, 3.8381168955644617`*^9}, 
   3.838116932344596*^9, 3.83811727578645*^9, 3.838117346211269*^9, 
   3.838117383722617*^9, 3.838118502541823*^9, {3.838122724339356*^9, 
   3.838122752800151*^9}, {3.838124097435034*^9, 3.838124209341752*^9}, 
   3.838124278917487*^9, {3.838124332277605*^9, 3.8381243475367413`*^9}, {
   3.838130835385419*^9, 3.8381308515365257`*^9}, 3.838130943357945*^9, 
   3.838191082956909*^9, 3.838202803143911*^9, 3.838208197712644*^9, 
   3.838239111139832*^9, 3.838239254699751*^9, {3.8382392942383413`*^9, 
   3.838239334979061*^9}, 3.838239436541318*^9, 3.838239558338705*^9, 
   3.838239644852519*^9, 3.838239819953383*^9, {3.838254322688583*^9, 
   3.838254335885709*^9}, {3.838254591195594*^9, 3.838254611634644*^9}, 
   3.838254840519187*^9, 3.838280536787595*^9, 3.8382905181763353`*^9, 
   3.838292365907517*^9, 3.8383987509598*^9, 3.838398797277648*^9, {
   3.8383988374801893`*^9, 3.838398864800226*^9}, 3.83870179118069*^9, 
   3.838801490076439*^9, 3.838801525177953*^9},
 CellLabel->
  "Out[249]=",ExpressionUUID->"9c670320-d0c1-4415-8cdb-3e3b8ffd1965"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0", ",", "0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0", ",", "0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0", ",", "0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0", ",", "0", ",", "0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", 
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     FractionBox["17", 
      RowBox[{"120", " ", 
       SqrtBox["2"]}]], ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    FractionBox["17", 
     RowBox[{"120", " ", 
      SqrtBox["2"]}]], "}"}], ",", 
   RowBox[{"{", "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.8381146728726797`*^9, 3.838116234664516*^9, 3.8381164247953653`*^9, {
   3.838116541558178*^9, 3.8381165708372107`*^9}, 3.838116675727318*^9, 
   3.8381167305415916`*^9, {3.83811688163279*^9, 3.8381168955644617`*^9}, 
   3.838116932344596*^9, 3.83811727578645*^9, 3.838117346211269*^9, 
   3.838117383722617*^9, 3.838118502541823*^9, {3.838122724339356*^9, 
   3.838122752800151*^9}, {3.838124097435034*^9, 3.838124209341752*^9}, 
   3.838124278917487*^9, {3.838124332277605*^9, 3.8381243475367413`*^9}, {
   3.838130835385419*^9, 3.8381308515365257`*^9}, 3.838130943357945*^9, 
   3.838191082956909*^9, 3.838202803143911*^9, 3.838208197712644*^9, 
   3.838239111139832*^9, 3.838239254699751*^9, {3.8382392942383413`*^9, 
   3.838239334979061*^9}, 3.838239436541318*^9, 3.838239558338705*^9, 
   3.838239644852519*^9, 3.838239819953383*^9, {3.838254322688583*^9, 
   3.838254335885709*^9}, {3.838254591195594*^9, 3.838254611634644*^9}, 
   3.838254840519187*^9, 3.838280536787595*^9, 3.8382905181763353`*^9, 
   3.838292365907517*^9, 3.8383987509598*^9, 3.838398797277648*^9, {
   3.8383988374801893`*^9, 3.838398864800226*^9}, 3.83870179118069*^9, 
   3.838801490076439*^9, 3.838801526992445*^9},
 CellLabel->
  "Out[250]=",ExpressionUUID->"39085576-dca5-485f-93bf-34442b9f2c26"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]], ",", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]]}], "}"}]], "Output",
 CellChangeTimes->{
  3.8381146728726797`*^9, 3.838116234664516*^9, 3.8381164247953653`*^9, {
   3.838116541558178*^9, 3.8381165708372107`*^9}, 3.838116675727318*^9, 
   3.8381167305415916`*^9, {3.83811688163279*^9, 3.8381168955644617`*^9}, 
   3.838116932344596*^9, 3.83811727578645*^9, 3.838117346211269*^9, 
   3.838117383722617*^9, 3.838118502541823*^9, {3.838122724339356*^9, 
   3.838122752800151*^9}, {3.838124097435034*^9, 3.838124209341752*^9}, 
   3.838124278917487*^9, {3.838124332277605*^9, 3.8381243475367413`*^9}, {
   3.838130835385419*^9, 3.8381308515365257`*^9}, 3.838130943357945*^9, 
   3.838191082956909*^9, 3.838202803143911*^9, 3.838208197712644*^9, 
   3.838239111139832*^9, 3.838239254699751*^9, {3.8382392942383413`*^9, 
   3.838239334979061*^9}, 3.838239436541318*^9, 3.838239558338705*^9, 
   3.838239644852519*^9, 3.838239819953383*^9, {3.838254322688583*^9, 
   3.838254335885709*^9}, {3.838254591195594*^9, 3.838254611634644*^9}, 
   3.838254840519187*^9, 3.838280536787595*^9, 3.8382905181763353`*^9, 
   3.838292365907517*^9, 3.8383987509598*^9, 3.838398797277648*^9, {
   3.8383988374801893`*^9, 3.838398864800226*^9}, 3.83870179118069*^9, 
   3.838801490076439*^9, 3.838801527431316*^9},
 CellLabel->
  "Out[251]=",ExpressionUUID->"67924d66-79b7-43ad-9484-fb357b9fd95f"],

Cell[BoxData[
 FractionBox["47", 
  RowBox[{"5", " ", 
   SqrtBox["2"]}]]], "Output",
 CellChangeTimes->{
  3.8381146728726797`*^9, 3.838116234664516*^9, 3.8381164247953653`*^9, {
   3.838116541558178*^9, 3.8381165708372107`*^9}, 3.838116675727318*^9, 
   3.8381167305415916`*^9, {3.83811688163279*^9, 3.8381168955644617`*^9}, 
   3.838116932344596*^9, 3.83811727578645*^9, 3.838117346211269*^9, 
   3.838117383722617*^9, 3.838118502541823*^9, {3.838122724339356*^9, 
   3.838122752800151*^9}, {3.838124097435034*^9, 3.838124209341752*^9}, 
   3.838124278917487*^9, {3.838124332277605*^9, 3.8381243475367413`*^9}, {
   3.838130835385419*^9, 3.8381308515365257`*^9}, 3.838130943357945*^9, 
   3.838191082956909*^9, 3.838202803143911*^9, 3.838208197712644*^9, 
   3.838239111139832*^9, 3.838239254699751*^9, {3.8382392942383413`*^9, 
   3.838239334979061*^9}, 3.838239436541318*^9, 3.838239558338705*^9, 
   3.838239644852519*^9, 3.838239819953383*^9, {3.838254322688583*^9, 
   3.838254335885709*^9}, {3.838254591195594*^9, 3.838254611634644*^9}, 
   3.838254840519187*^9, 3.838280536787595*^9, 3.8382905181763353`*^9, 
   3.838292365907517*^9, 3.8383987509598*^9, 3.838398797277648*^9, {
   3.8383988374801893`*^9, 3.838398864800226*^9}, 3.83870179118069*^9, 
   3.838801490076439*^9, 3.8388015298102207`*^9},
 CellLabel->
  "Out[252]=",ExpressionUUID->"f282da51-0bba-4034-97ba-d593a7bb67f6"],

Cell[BoxData["6.64680374315354672936793700378558096927`16."], "Output",
 CellChangeTimes->{
  3.8381146728726797`*^9, 3.838116234664516*^9, 3.8381164247953653`*^9, {
   3.838116541558178*^9, 3.8381165708372107`*^9}, 3.838116675727318*^9, 
   3.8381167305415916`*^9, {3.83811688163279*^9, 3.8381168955644617`*^9}, 
   3.838116932344596*^9, 3.83811727578645*^9, 3.838117346211269*^9, 
   3.838117383722617*^9, 3.838118502541823*^9, {3.838122724339356*^9, 
   3.838122752800151*^9}, {3.838124097435034*^9, 3.838124209341752*^9}, 
   3.838124278917487*^9, {3.838124332277605*^9, 3.8381243475367413`*^9}, {
   3.838130835385419*^9, 3.8381308515365257`*^9}, 3.838130943357945*^9, 
   3.838191082956909*^9, 3.838202803143911*^9, 3.838208197712644*^9, 
   3.838239111139832*^9, 3.838239254699751*^9, {3.8382392942383413`*^9, 
   3.838239334979061*^9}, 3.838239436541318*^9, 3.838239558338705*^9, 
   3.838239644852519*^9, 3.838239819953383*^9, {3.838254322688583*^9, 
   3.838254335885709*^9}, {3.838254591195594*^9, 3.838254611634644*^9}, 
   3.838254840519187*^9, 3.838280536787595*^9, 3.8382905181763353`*^9, 
   3.838292365907517*^9, 3.8383987509598*^9, 3.838398797277648*^9, {
   3.8383988374801893`*^9, 3.838398864800226*^9}, 3.83870179118069*^9, 
   3.838801490076439*^9, 3.838801529905891*^9},
 CellLabel->
  "Out[253]=",ExpressionUUID->"2f0c85c1-e835-4587-b464-f03eabcc6c6e"],

Cell[BoxData[
 RowBox[{"-", "1.1046875`"}]], "Output",
 CellChangeTimes->{
  3.8381146728726797`*^9, 3.838116234664516*^9, 3.8381164247953653`*^9, {
   3.838116541558178*^9, 3.8381165708372107`*^9}, 3.838116675727318*^9, 
   3.8381167305415916`*^9, {3.83811688163279*^9, 3.8381168955644617`*^9}, 
   3.838116932344596*^9, 3.83811727578645*^9, 3.838117346211269*^9, 
   3.838117383722617*^9, 3.838118502541823*^9, {3.838122724339356*^9, 
   3.838122752800151*^9}, {3.838124097435034*^9, 3.838124209341752*^9}, 
   3.838124278917487*^9, {3.838124332277605*^9, 3.8381243475367413`*^9}, {
   3.838130835385419*^9, 3.8381308515365257`*^9}, 3.838130943357945*^9, 
   3.838191082956909*^9, 3.838202803143911*^9, 3.838208197712644*^9, 
   3.838239111139832*^9, 3.838239254699751*^9, {3.8382392942383413`*^9, 
   3.838239334979061*^9}, 3.838239436541318*^9, 3.838239558338705*^9, 
   3.838239644852519*^9, 3.838239819953383*^9, {3.838254322688583*^9, 
   3.838254335885709*^9}, {3.838254591195594*^9, 3.838254611634644*^9}, 
   3.838254840519187*^9, 3.838280536787595*^9, 3.8382905181763353`*^9, 
   3.838292365907517*^9, 3.8383987509598*^9, 3.838398797277648*^9, {
   3.8383988374801893`*^9, 3.838398864800226*^9}, 3.83870179118069*^9, 
   3.838801490076439*^9, 3.838801529974131*^9},
 CellLabel->
  "Out[255]=",ExpressionUUID->"43f44c3a-1fb8-44a3-9e85-284de1277718"],

Cell[BoxData["0.6114583333333333`"], "Output",
 CellChangeTimes->{
  3.8381146728726797`*^9, 3.838116234664516*^9, 3.8381164247953653`*^9, {
   3.838116541558178*^9, 3.8381165708372107`*^9}, 3.838116675727318*^9, 
   3.8381167305415916`*^9, {3.83811688163279*^9, 3.8381168955644617`*^9}, 
   3.838116932344596*^9, 3.83811727578645*^9, 3.838117346211269*^9, 
   3.838117383722617*^9, 3.838118502541823*^9, {3.838122724339356*^9, 
   3.838122752800151*^9}, {3.838124097435034*^9, 3.838124209341752*^9}, 
   3.838124278917487*^9, {3.838124332277605*^9, 3.8381243475367413`*^9}, {
   3.838130835385419*^9, 3.8381308515365257`*^9}, 3.838130943357945*^9, 
   3.838191082956909*^9, 3.838202803143911*^9, 3.838208197712644*^9, 
   3.838239111139832*^9, 3.838239254699751*^9, {3.8382392942383413`*^9, 
   3.838239334979061*^9}, 3.838239436541318*^9, 3.838239558338705*^9, 
   3.838239644852519*^9, 3.838239819953383*^9, {3.838254322688583*^9, 
   3.838254335885709*^9}, {3.838254591195594*^9, 3.838254611634644*^9}, 
   3.838254840519187*^9, 3.838280536787595*^9, 3.8382905181763353`*^9, 
   3.838292365907517*^9, 3.8383987509598*^9, 3.838398797277648*^9, {
   3.8383988374801893`*^9, 3.838398864800226*^9}, 3.83870179118069*^9, 
   3.838801490076439*^9, 3.8388015300447273`*^9},
 CellLabel->
  "Out[256]=",ExpressionUUID->"bee2b919-af19-4b21-80a8-84dbaeca1179"]
}, Open  ]],

Cell[BoxData[" "], "Input",
 CellChangeTimes->{
  3.838239300388268*^9},ExpressionUUID->"f650c2d8-af76-412b-99b5-\
fa46e1a33a34"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.837759051812975*^9, 3.837759071591836*^9}, {
   3.837759217318631*^9, 3.837759241400175*^9}, {3.837759320301201*^9, 
   3.83775936716477*^9}, {3.837759456303917*^9, 3.837759642000123*^9}, {
   3.837759816966599*^9, 3.83775991155928*^9}, {3.837759956835602*^9, 
   3.8377599789804487`*^9}, {3.837760016652355*^9, 3.837760096420809*^9}, {
   3.837760140717391*^9, 3.837760207546588*^9}, {3.8377602481030903`*^9, 
   3.8377602740980864`*^9}, {3.837760333242087*^9, 3.837760427623968*^9}, {
   3.8377604636283073`*^9, 3.8377607453888807`*^9}, {3.837760787735127*^9, 
   3.837760793341056*^9}, {3.837761056684053*^9, 3.837761100969037*^9}, {
   3.8377615513819017`*^9, 3.83776158670927*^9}, {3.837761621261744*^9, 
   3.8377616531407127`*^9}, {3.837761693987907*^9, 3.837761729898768*^9}, {
   3.837761844581964*^9, 3.837761970006235*^9}, {3.837762073595648*^9, 
   3.837762096290968*^9}, {3.837762142678879*^9, 3.8377622375066757`*^9}, {
   3.8377622979149218`*^9, 3.837762405543975*^9}, {3.83776251457353*^9, 
   3.837762566781784*^9}, {3.837762618932023*^9, 3.8377626637366657`*^9}, {
   3.837763509203718*^9, 3.837763591558241*^9}, {3.83776364159385*^9, 
   3.83776367456262*^9}, {3.837763759909724*^9, 3.8377638110846853`*^9}, {
   3.8377638908917513`*^9, 3.8377639078480997`*^9}, {3.837763980843951*^9, 
   3.837764177471188*^9}, {3.8377642380320673`*^9, 3.837764360041239*^9}, {
   3.837764442964953*^9, 3.837764502687722*^9}, {3.83776455233293*^9, 
   3.8377646335315657`*^9}, {3.837764699266444*^9, 3.8377647272120543`*^9}, {
   3.837764769287674*^9, 3.837765008276999*^9}, {3.837765052599174*^9, 
   3.837765057200159*^9}, {3.837765092612886*^9, 3.837765334368432*^9}, {
   3.8377654308449802`*^9, 3.8377655319772167`*^9}, {3.837765568969681*^9, 
   3.837765661871271*^9}, {3.837765791941186*^9, 3.837765912988414*^9}, {
   3.8377660145740337`*^9, 3.8377661776828814`*^9}, {3.837766237263646*^9, 
   3.837766278805069*^9}, {3.8377663100115967`*^9, 3.837766329618359*^9}, {
   3.837766657496814*^9, 3.837766657731106*^9}, {3.83776715547604*^9, 
   3.8377671565101347`*^9}, {3.8377674335426807`*^9, 3.8377674351976757`*^9}, 
   3.837768112932695*^9, {3.837768376845517*^9, 3.837768449341558*^9}, 
   3.837768484719595*^9, {3.837768542966751*^9, 3.837768557770595*^9}, {
   3.8377686335259457`*^9, 3.837768639317741*^9}, {3.837769274391078*^9, 
   3.8377693524395447`*^9}, {3.837769497420038*^9, 3.8377695037909527`*^9}, {
   3.837769596022748*^9, 3.8377696087576*^9}, {3.837769647025366*^9, 
   3.837769761571034*^9}, {3.83776983151989*^9, 3.8377699867015038`*^9}, {
   3.837770952577222*^9, 3.837771031706276*^9}, {3.837771104943639*^9, 
   3.837771165144917*^9}, 3.8377712134152813`*^9, {3.837771255405746*^9, 
   3.837771342336483*^9}, {3.837771373782195*^9, 3.837771500315425*^9}, {
   3.8377715318583117`*^9, 3.837771603465426*^9}, 3.83777165452458*^9, {
   3.837771689129313*^9, 3.8377717429592857`*^9}, {3.8377717947541027`*^9, 
   3.837772161043878*^9}, {3.837772205636837*^9, 3.837772226479508*^9}, {
   3.8377722800771313`*^9, 3.8377722844518147`*^9}, {3.837772316184485*^9, 
   3.837772330120789*^9}, {3.8377723601984863`*^9, 3.837772389931653*^9}, {
   3.837772440094946*^9, 3.8377724717356987`*^9}, {3.837772505324279*^9, 
   3.837772849864531*^9}, {3.837772896947722*^9, 3.837773036687307*^9}, {
   3.8377731147022753`*^9, 3.837773227864345*^9}, {3.837773275727414*^9, 
   3.837773298163156*^9}, {3.837773347035487*^9, 3.83777335522159*^9}, {
   3.837773412888567*^9, 3.837773475740651*^9}, {3.837773530112762*^9, 
   3.8377735931395397`*^9}, {3.837773630868861*^9, 3.837773822688457*^9}, {
   3.837773878077608*^9, 3.8377738872682123`*^9}, {3.837773919005477*^9, 
   3.837774000967046*^9}, {3.8377740453237867`*^9, 3.837774049124507*^9}, {
   3.837774085611842*^9, 3.837774223315514*^9}, 3.837774263547697*^9, {
   3.8377743121800003`*^9, 3.8377743126575537`*^9}, {3.837774496089884*^9, 
   3.837774503826468*^9}, {3.8377745574267807`*^9, 3.837774895068288*^9}, {
   3.837774940831119*^9, 3.8377750491080093`*^9}, {3.8377751090428333`*^9, 
   3.837775144213842*^9}, {3.8377751951433*^9, 3.837775305357629*^9}, {
   3.837775364894342*^9, 3.837775496742807*^9}, {3.8377755338131523`*^9, 
   3.8377755753185453`*^9}, {3.8377756277767887`*^9, 3.837775726961532*^9}, {
   3.8377757758014393`*^9, 3.837775839367248*^9}, {3.837775880521937*^9, 
   3.837775911455641*^9}, {3.837775942504963*^9, 3.837775964469473*^9}, {
   3.837775996413103*^9, 3.837776188288731*^9}, {3.837776234433379*^9, 
   3.837776350471487*^9}, {3.837776493128147*^9, 3.837776540687028*^9}, 
   3.8377766164973783`*^9, {3.837776652131136*^9, 3.8377766581985807`*^9}, {
   3.837776701059732*^9, 3.837776709453355*^9}, {3.83777678203025*^9, 
   3.83777678709515*^9}, {3.837776820532475*^9, 3.8377768331493797`*^9}, {
   3.837776956286186*^9, 3.837777064590246*^9}, {3.837777101706354*^9, 
   3.837777205573284*^9}, {3.837777272769291*^9, 3.837777673512064*^9}, {
   3.837777724691207*^9, 3.837777748824847*^9}, {3.837777879292939*^9, 
   3.837777898982292*^9}, {3.837777937064353*^9, 3.837777973922823*^9}, 
   3.837778019767487*^9, {3.837778050456066*^9, 3.837778147495962*^9}, {
   3.8377795716200027`*^9, 3.837779600678067*^9}, 3.837779651879637*^9, {
   3.8377796862719593`*^9, 3.837779717452261*^9}, {3.8377797614372*^9, 
   3.837779775519079*^9}, {3.837779972163884*^9, 3.8377799740534887`*^9}, {
   3.837780017640114*^9, 3.837780024681024*^9}, {3.837780057444573*^9, 
   3.837780080083229*^9}, 3.8377803804440327`*^9, {3.8377805107291203`*^9, 
   3.837780566161954*^9}, {3.837780637776647*^9, 3.837780822613832*^9}, {
   3.837780872466659*^9, 3.8377809327605352`*^9}, {3.837780992367846*^9, 
   3.837781005509432*^9}, {3.8377810726487713`*^9, 3.837781088451127*^9}, 
   3.8377811295651197`*^9, {3.837781739704506*^9, 3.837781750649578*^9}, {
   3.837781898408183*^9, 3.8377819488022537`*^9}, {3.8377819864007177`*^9, 
   3.837781992270015*^9}, {3.837782024170294*^9, 3.8377821019794483`*^9}, {
   3.837782142737905*^9, 3.837782162036997*^9}, {3.837782354378776*^9, 
   3.837782448482431*^9}, {3.8377848682062187`*^9, 3.8377848913130713`*^9}, {
   3.837784930076548*^9, 3.8377849415859213`*^9}, {3.837785645716629*^9, 
   3.837785678588539*^9}, {3.837785728736228*^9, 3.837785763713437*^9}, 
   3.837786034233245*^9, {3.837786066746749*^9, 3.837786116126554*^9}, {
   3.837861251941223*^9, 3.83786128903496*^9}, {3.837861349162579*^9, 
   3.83786137538763*^9}, {3.837861441354437*^9, 3.8378614479760733`*^9}, {
   3.83786148029738*^9, 3.837861505582625*^9}, 3.837861752973679*^9, {
   3.837862755674859*^9, 3.837862763059555*^9}, {3.8380758896068077`*^9, 
   3.8380759297842407`*^9}, {3.8380975980153093`*^9, 3.838097663482284*^9}, {
   3.8380977510926313`*^9, 3.838097788509506*^9}, {3.8380978601000767`*^9, 
   3.8380978646323233`*^9}, {3.8380980026732683`*^9, 
   3.8380980036038847`*^9}, {3.838098036220108*^9, 3.838098064053454*^9}, {
   3.838112651834959*^9, 3.8381126815227118`*^9}, {3.838114667604498*^9, 
   3.838114692184175*^9}},ExpressionUUID->"19b95e8d-8177-45fc-8e99-\
2d06d6b7b9f6"],

Cell[TextData[{
 "To second order, we must now calculate ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["I", "2", 
    RowBox[{"(", "0", ")"}]], TraditionalForm]],ExpressionUUID->
  "5265942b-4fda-40b3-a169-16e93ed84b18"],
 "and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["I", "2", 
    RowBox[{"(", "1", ")"}]], TraditionalForm]],ExpressionUUID->
  "85efd963-9462-46b1-81a7-fcce8854e2c0"],
 ". These involve the unpinning of a second particle. We can choose our \
coordinates to place this particle opposite the first neighbor vector of the \
particle that we introduced previously. The new coordinates are now z = \
[x1,x2,x3,x4,x5,x6], which are all real. However, the unit vectors w are now \
modified. Most of these vectors only involve a plane defined by one of  the \
particles, and so they are easily written from above, substituting 0s for the \
uninvolved particle dimensions. The only major modification is in the first \
entry which couples two of the sides."
}], "Text",
 CellChangeTimes->{{3.836381128856174*^9, 3.836381201686969*^9}, {
  3.836381264647275*^9, 3.8363812940296993`*^9}, {3.836381373308559*^9, 
  3.836381511180917*^9}, {3.8363815567754602`*^9, 3.836381660610794*^9}, {
  3.836381697604307*^9, 3.836381705457198*^9}, {3.836381758487977*^9, 
  3.836381794432242*^9}, {3.83638182651686*^9, 3.83638201144018*^9}, {
  3.836383919708055*^9, 3.836383922197781*^9}, {3.83638527757804*^9, 
  3.836385280268815*^9}, {3.836385406578272*^9, 3.8363854329065742`*^9}, {
  3.8370007446945763`*^9, 
  3.837000746211*^9}},ExpressionUUID->"1ab9e84a-9032-4551-9d67-f55dbd635a4d"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"u", " ", "=", " ", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Join", "[", 
       RowBox[{"ConstantArray", "[", 
        RowBox[{"1", ",", 
         RowBox[{"2", "d"}]}], "]"}], "]"}], "}"}], ",", 
     RowBox[{"ConstantArray", "[", 
      RowBox[{
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"ConstantArray", "[", 
          RowBox[{"1", ",", "d"}], "]"}], ",", 
         RowBox[{"ConstantArray", "[", 
          RowBox[{"0", ",", "d"}], "]"}]}], "]"}], ",", 
       RowBox[{
        RowBox[{"Length", "[", "w1", "]"}], "-", "1"}]}], "]"}], ",", 
     RowBox[{"ConstantArray", "[", 
      RowBox[{
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"ConstantArray", "[", 
          RowBox[{"0", ",", "d"}], "]"}], ",", 
         RowBox[{"ConstantArray", "[", 
          RowBox[{"1", ",", "d"}], "]"}]}], "]"}], ",", 
       RowBox[{
        RowBox[{"Length", "[", "w1", "]"}], "-", "1"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"w", " ", "=", " ", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"w1", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", 
        RowBox[{"-", 
         RowBox[{"w1", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}], "}"}], ",", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"w1", "[", 
        RowBox[{"[", 
         RowBox[{"2", ";;"}], "]"}], "]"}], ",", 
       RowBox[{"ConstantArray", "[", 
        RowBox[{
         RowBox[{"ConstantArray", "[", 
          RowBox[{"0", ",", "d"}], "]"}], ",", 
         RowBox[{"Length", "[", 
          RowBox[{"w1", "[", 
           RowBox[{"[", 
            RowBox[{"2", ";;"}], "]"}], "]"}], "]"}]}], "]"}], ",", "2"}], 
      "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"ConstantArray", "[", 
         RowBox[{"0", ",", "d"}], "]"}], ",", 
        RowBox[{"w1", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "}"}], ",", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"ConstantArray", "[", 
        RowBox[{
         RowBox[{"ConstantArray", "[", 
          RowBox[{"0", ",", "d"}], "]"}], ",", 
         RowBox[{"Length", "[", 
          RowBox[{"w1", "[", 
           RowBox[{"[", 
            RowBox[{"3", ";;"}], "]"}], "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"w1", "[", 
        RowBox[{"[", 
         RowBox[{"3", ";;"}], "]"}], "]"}], ",", "2"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"z", " ", "=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Subscript", "[", 
      RowBox[{"x", ",", "i"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"2", "d"}]}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"$Assumptions", " ", "=", " ", 
   RowBox[{
    RowBox[{"Evaluate", "[", "z", "]"}], " ", "\[Element]", " ", "Reals"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sites", " ", "=", " ", 
   RowBox[{"Length", "[", "w", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"InequalityList", " ", "=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"1", "+", 
       RowBox[{"z", ".", 
        RowBox[{"w", "[", 
         RowBox[{"[", "j", "]"}], "]"}]}]}], ">=", "0"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"EqualityList", " ", "=", "  ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"1", "+", 
       RowBox[{"z", ".", 
        RowBox[{"w", "[", 
         RowBox[{"[", "j", "]"}], "]"}]}]}], "==", "0"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"reg", "=", " ", 
   RowBox[{"RegionIntersection", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"ImplicitRegion", "[", 
       RowBox[{
        RowBox[{"And", "@@", "InequalityList"}], ",", 
        RowBox[{"Evaluate", "[", "z", "]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Magz", "[", "j_", "]"}], ":=", 
  RowBox[{"Total", "[", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"z", "[", 
        RowBox[{"[", 
         RowBox[{"1", ";;", "d"}], "]"}], "]"}], "*", 
       RowBox[{"u", "[", 
        RowBox[{"[", 
         RowBox[{"j", ",", 
          RowBox[{"1", ";;", "d"}]}], "]"}], "]"}]}], " ", "-", " ", 
      RowBox[{
       RowBox[{"z", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"d", "+", "1"}], ")"}], ";;", 
          RowBox[{"2", "d"}]}], "]"}], "]"}], "*", 
       RowBox[{"u", "[", 
        RowBox[{"[", 
         RowBox[{"j", ",", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"d", "+", "1"}], ")"}], ";;", 
           RowBox[{"2", "d"}]}]}], "]"}], "]"}]}]}], ")"}], "2"], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HalfPlanes", "[", 
   RowBox[{"j_", ",", "k_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"And", "@@", 
     RowBox[{"InequalityList", "[", 
      RowBox[{"[", 
       RowBox[{"Delete", "[", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"1", ",", "sites"}], "]"}], ",", "j"}], "]"}], "]"}], 
      "]"}]}], ")"}], "/.", 
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{"EqualityList", "[", 
       RowBox[{"[", "j", "]"}], "]"}], ",", 
      RowBox[{"z", "[", 
       RowBox[{"[", "k", "]"}], "]"}], ",", "Reals"}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PolyIntegrand", " ", "[", 
   RowBox[{"j_", ",", "k_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", "2"], "*", 
     FractionBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Magz", "[", "j", "]"}], "-", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"z", ".", 
           RowBox[{"w", "[", 
            RowBox[{"[", "j", "]"}], "]"}]}], ")"}], "2"]}], ")"}], 
      SqrtBox[
       SuperscriptBox[
        RowBox[{"w", "[", 
         RowBox[{"[", 
          RowBox[{"j", ",", "k"}], "]"}], "]"}], "2"]]]}], ")"}], "/.", 
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{"EqualityList", "[", 
       RowBox[{"[", "j", "]"}], "]"}], ",", 
      RowBox[{"z", "[", 
       RowBox[{"[", "k", "]"}], "]"}], ",", "Reals"}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"deltaDir", "[", "j_", "]"}], ":=", 
  RowBox[{
   RowBox[{"FirstPosition", "[", 
    RowBox[{
     RowBox[{"w", "[", 
      RowBox[{"[", "j", "]"}], "]"}], ",", 
     RowBox[{"_", "?", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"#", " ", "!=", " ", "0"}], " ", "&"}], ")"}]}]}], "]"}], "[", 
   RowBox[{"[", "1", "]"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HRep", "[", "j_", "]"}], ":=", 
  RowBox[{"N", "[", 
   RowBox[{
    RowBox[{"Expand", "[", 
     RowBox[{"HalfPlanes", "[", 
      RowBox[{"j", ",", 
       RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], "]"}], ",", "2"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ReducedPoly", "[", "j_", "]"}], ":=", 
  RowBox[{"Expand", "[", 
   RowBox[{"PolyIntegrand", "[", 
    RowBox[{"j", ",", 
     RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"surfaceIntegral", "[", 
   RowBox[{"j_", ",", "k_"}], "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"RegionDimension", "[", 
        RowBox[{"ImplicitRegion", "[", 
         RowBox[{
          RowBox[{"HalfPlanes", "[", 
           RowBox[{"j", ",", "k"}], "]"}], ",", 
          RowBox[{"Evaluate", "[", 
           RowBox[{"Delete", "[", 
            RowBox[{"z", ",", "k"}], "]"}], "]"}]}], "]"}], "]"}], "<", 
       RowBox[{
        RowBox[{"2", "*", "d"}], "-", "1"}]}], ",", 
      RowBox[{
       RowBox[{"Return", "[", "0", "]"}], ";"}], ",", 
      RowBox[{"Return", "[", 
       RowBox[{"Integrate", "[", 
        RowBox[{
         RowBox[{"PolyIntegrand", "[", 
          RowBox[{"j", ",", "k"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Evaluate", "[", 
           RowBox[{"Delete", "[", 
            RowBox[{"z", ",", "k"}], "]"}], "]"}], "\[Element]", " ", 
          RowBox[{"ImplicitRegion", "[", 
           RowBox[{
            RowBox[{"HalfPlanes", "[", 
             RowBox[{"j", ",", "k"}], "]"}], ",", 
            RowBox[{"Evaluate", "[", 
             RowBox[{"Delete", "[", 
              RowBox[{"z", ",", "k"}], "]"}], "]"}]}], "]"}]}]}], "]"}], 
       "]"}]}], "]"}], ";"}], "\[IndentingNewLine]", ")"}]}]}], "Input",
 CellChangeTimes->{{3.8363816635194607`*^9, 3.836381687103815*^9}, 
   3.836382052279499*^9, {3.8363821405210533`*^9, 3.836382141053196*^9}, {
   3.83638405376974*^9, 3.836384056176593*^9}, {3.836384092130391*^9, 
   3.836384196346674*^9}, 3.8363843036510067`*^9, {3.83638458113338*^9, 
   3.836384741330194*^9}, {3.836384861930937*^9, 3.836384885731694*^9}, {
   3.836544573475802*^9, 3.836544676674941*^9}, {3.836544720034343*^9, 
   3.8365447613742*^9}, {3.836544889021159*^9, 3.8365448932223997`*^9}, {
   3.836545147060693*^9, 3.836545150465077*^9}, {3.836545604379964*^9, 
   3.8365457859528837`*^9}, 3.836829869415275*^9, 3.836829903162695*^9, {
   3.8368301639370127`*^9, 3.836830175872464*^9}, 3.836830251542295*^9, {
   3.836861687654317*^9, 3.836861687887755*^9}, 3.836861804819784*^9, {
   3.8369425633550777`*^9, 3.8369425715346212`*^9}, {3.83700073523041*^9, 
   3.8370007546902742`*^9}, {3.837348523555286*^9, 3.83734854766203*^9}, 
   3.8373506614387703`*^9, {3.837509733052663*^9, 3.837509740478406*^9}, {
   3.8381128858657513`*^9, 3.838112892301281*^9}, 3.83811324271276*^9, {
   3.8388018533289843`*^9, 3.838801962097694*^9}, {3.839573912838767*^9, 
   3.8395739201613083`*^9}},
 CellLabel->
  "In[436]:=",ExpressionUUID->"ab2ae627-5ff9-403d-a120-e4c629e0b531"],

Cell[TextData[{
 "It is then straightforward to calculate ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["I", "2", 
    RowBox[{"(", "0", ")"}]], TraditionalForm]],ExpressionUUID->
  "70bb88a1-45f0-4c37-afba-7a4b2f4b619e"],
 ", labelled here as Int20."
}], "Text",
 CellChangeTimes->{{3.8363816635194607`*^9, 3.836381687103815*^9}, {
  3.836382052279499*^9, 3.8363821194926167`*^9}, {3.83638490521347*^9, 
  3.836384917540909*^9}},ExpressionUUID->"bfde2e94-4ac0-475b-969f-\
490a5ec8dfcd"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Int20", " ", "=", " ", 
  RowBox[{"Integrate", "[", 
   RowBox[{"1", ",", 
    RowBox[{
     RowBox[{"Evaluate", "[", "z", "]"}], "\[Element]", "reg"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["C", "2"], " ", "=", " ", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Log", "[", "Int20", "]"}]}], " ", "-", " ", 
    RowBox[{"2", "*", 
     SubscriptBox["C", "1"]}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  SubscriptBox["C", "2"], "]"}]}], "Input",
 CellChangeTimes->{{3.8363816635194607`*^9, 3.836381687103815*^9}, {
   3.836382052279499*^9, 3.836382240892726*^9}, {3.8363824396877737`*^9, 
   3.836382468964446*^9}, {3.836383416511867*^9, 3.836383426729013*^9}, {
   3.836384752390709*^9, 3.83638475531861*^9}, {3.836384959287032*^9, 
   3.836384969177258*^9}, 3.8372497162660522`*^9, {3.83734855699507*^9, 
   3.837348559429515*^9}},
 CellLabel->
  "In[451]:=",ExpressionUUID->"92e5f1d9-d9df-4f8d-a4e4-2d24ab6be0a5"],

Cell[BoxData[
 FractionBox["467", "15"]], "Output",
 CellChangeTimes->{
  3.836382335245092*^9, {3.836382440786147*^9, 3.836382469455626*^9}, 
   3.836383593094657*^9, 3.836384913128813*^9, 3.836384975044682*^9, 
   3.836544742579422*^9, 3.836544837205659*^9, 3.836544916271599*^9, 
   3.8365458030577803`*^9, 3.836555856290721*^9, 3.8368297649373903`*^9, 
   3.8368298928451633`*^9, 3.836829923463212*^9, 3.836829966686349*^9, {
   3.836830188487864*^9, 3.836830208104376*^9}, 3.8368616607897243`*^9, 
   3.836861703153605*^9, 3.836861815286907*^9, 3.836942448036126*^9, 
   3.836942603195138*^9, 3.837000935277308*^9, 3.837157093474375*^9, 
   3.8371642668858223`*^9, 3.837249717417192*^9, 3.837348573192607*^9, 
   3.837350067935483*^9, 3.83735066937047*^9, 3.837355423164119*^9, 
   3.83735692304044*^9, 3.8375088487901983`*^9, 3.8375097472468348`*^9, 
   3.837509881903191*^9, 3.837867263236104*^9, 3.8381096912966003`*^9, 
   3.838109938545951*^9, 3.838112706755838*^9, 3.8381129425525103`*^9, 
   3.838113307161532*^9, 3.8382061166697598`*^9, 3.8382082148857527`*^9, 
   3.838239125512932*^9, 3.838239666423617*^9, 3.838239835536808*^9, 
   3.838280556159882*^9, 3.8382905328576813`*^9, 3.838292406101468*^9, 
   3.838398884710136*^9, 3.8387019652923594`*^9, 3.838701999948246*^9, 
   3.838801547210092*^9, 3.8388019703674927`*^9, 3.839573935501605*^9, 
   3.839573973973154*^9},
 CellLabel->
  "Out[451]=",ExpressionUUID->"b146b3be-3cef-47cc-b1c9-bda035f5fde5"],

Cell[BoxData[
 RowBox[{"Log", "[", 
  FractionBox["480", "467"], "]"}]], "Output",
 CellChangeTimes->{
  3.836382335245092*^9, {3.836382440786147*^9, 3.836382469455626*^9}, 
   3.836383593094657*^9, 3.836384913128813*^9, 3.836384975044682*^9, 
   3.836544742579422*^9, 3.836544837205659*^9, 3.836544916271599*^9, 
   3.8365458030577803`*^9, 3.836555856290721*^9, 3.8368297649373903`*^9, 
   3.8368298928451633`*^9, 3.836829923463212*^9, 3.836829966686349*^9, {
   3.836830188487864*^9, 3.836830208104376*^9}, 3.8368616607897243`*^9, 
   3.836861703153605*^9, 3.836861815286907*^9, 3.836942448036126*^9, 
   3.836942603195138*^9, 3.837000935277308*^9, 3.837157093474375*^9, 
   3.8371642668858223`*^9, 3.837249717417192*^9, 3.837348573192607*^9, 
   3.837350067935483*^9, 3.83735066937047*^9, 3.837355423164119*^9, 
   3.83735692304044*^9, 3.8375088487901983`*^9, 3.8375097472468348`*^9, 
   3.837509881903191*^9, 3.837867263236104*^9, 3.8381096912966003`*^9, 
   3.838109938545951*^9, 3.838112706755838*^9, 3.8381129425525103`*^9, 
   3.838113307161532*^9, 3.8382061166697598`*^9, 3.8382082148857527`*^9, 
   3.838239125512932*^9, 3.838239666423617*^9, 3.838239835536808*^9, 
   3.838280556159882*^9, 3.8382905328576813`*^9, 3.838292406101468*^9, 
   3.838398884710136*^9, 3.8387019652923594`*^9, 3.838701999948246*^9, 
   3.838801547210092*^9, 3.8388019703674927`*^9, 3.839573935501605*^9, 
   3.839573974050302*^9},
 CellLabel->
  "Out[452]=",ExpressionUUID->"9787f1bc-f27f-49ed-ae79-bcbc9016b7fc"],

Cell[BoxData["0.02745684623303929`"], "Output",
 CellChangeTimes->{
  3.836382335245092*^9, {3.836382440786147*^9, 3.836382469455626*^9}, 
   3.836383593094657*^9, 3.836384913128813*^9, 3.836384975044682*^9, 
   3.836544742579422*^9, 3.836544837205659*^9, 3.836544916271599*^9, 
   3.8365458030577803`*^9, 3.836555856290721*^9, 3.8368297649373903`*^9, 
   3.8368298928451633`*^9, 3.836829923463212*^9, 3.836829966686349*^9, {
   3.836830188487864*^9, 3.836830208104376*^9}, 3.8368616607897243`*^9, 
   3.836861703153605*^9, 3.836861815286907*^9, 3.836942448036126*^9, 
   3.836942603195138*^9, 3.837000935277308*^9, 3.837157093474375*^9, 
   3.8371642668858223`*^9, 3.837249717417192*^9, 3.837348573192607*^9, 
   3.837350067935483*^9, 3.83735066937047*^9, 3.837355423164119*^9, 
   3.83735692304044*^9, 3.8375088487901983`*^9, 3.8375097472468348`*^9, 
   3.837509881903191*^9, 3.837867263236104*^9, 3.8381096912966003`*^9, 
   3.838109938545951*^9, 3.838112706755838*^9, 3.8381129425525103`*^9, 
   3.838113307161532*^9, 3.8382061166697598`*^9, 3.8382082148857527`*^9, 
   3.838239125512932*^9, 3.838239666423617*^9, 3.838239835536808*^9, 
   3.838280556159882*^9, 3.8382905328576813`*^9, 3.838292406101468*^9, 
   3.838398884710136*^9, 3.8387019652923594`*^9, 3.838701999948246*^9, 
   3.838801547210092*^9, 3.8388019703674927`*^9, 3.839573935501605*^9, 
   3.839573974119995*^9},
 CellLabel->
  "Out[453]=",ExpressionUUID->"14d1473f-fd9c-4e82-90ac-329b8e18c732"]
}, Open  ]],

Cell[TextData[{
 "This is again exactly the value calculated in Rudd, et al., J. Chem. Phys. \
(1968). However, a straightforward application of  the above calculation for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "2"], TraditionalForm]],
  FormatType->TraditionalForm,ExpressionUUID->
  "05425f61-f7b8-4715-b446-ae8ea6ef27ea"],
 " does not yield the same. We follow the exact same procedure as in d=2 to \
find:"
}], "Text",
 CellChangeTimes->{{3.8363816635194607`*^9, 3.836381687103815*^9}, {
   3.836382052279499*^9, 3.8363821686658697`*^9}, {3.836382337919899*^9, 
   3.836382396171382*^9}, {3.836382484807663*^9, 3.8363825229703217`*^9}, 
   3.8363847941471987`*^9, {3.83638523686467*^9, 3.836385251514861*^9}, {
   3.837000794141871*^9, 
   3.837000811799059*^9}},ExpressionUUID->"3221b9fc-1c23-4ccb-bbee-\
e3e348cc46a4"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Int21", " ", "=", " ", 
  RowBox[{"Total", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"surfaceIntegral", "[", 
      RowBox[{"j", ",", 
       RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["D", "2"], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "Int21"}], "/", "Int20"}], " ", "-", " ", 
   RowBox[{"2", "*", 
    SubscriptBox["D", "1"]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "Int21"}], "/", "Int20"}], ",", "16"}], "]"}]}], "Input",
 CellChangeTimes->{{3.8363816635194607`*^9, 3.836381687103815*^9}, {
   3.836382052279499*^9, 3.8363821686658697`*^9}, {3.836382337919899*^9, 
   3.836382396171382*^9}, {3.836382484807663*^9, 3.836382710416163*^9}, {
   3.83638343319324*^9, 3.836383452418956*^9}, {3.836384764455323*^9, 
   3.836384776974996*^9}, {3.8365445336444607`*^9, 3.836544561781005*^9}, {
   3.8368302687467213`*^9, 3.836830297661169*^9}, {3.837000764698432*^9, 
   3.837000766668515*^9}, 3.837000815829789*^9, 3.837156980000206*^9, {
   3.837157013885169*^9, 3.83715702763338*^9}, {3.837349829062923*^9, 
   3.8373498406153917`*^9}, 3.837509780959354*^9, {3.837867298379574*^9, 
   3.837867330964432*^9}, {3.837867398616021*^9, 3.8378673990229073`*^9}, {
   3.8381093514500027`*^9, 3.8381093670832033`*^9}, 3.83810994873591*^9, 
   3.8381127252610903`*^9, {3.838113138976777*^9, 3.838113145475379*^9}, {
   3.838113252884259*^9, 3.838113257226509*^9}, {3.838801569109766*^9, 
   3.8388015883030043`*^9}, {3.847983459229205*^9, 
   3.847983459766004*^9}},ExpressionUUID->"c454802c-2125-455a-a8e6-\
1fa5d7f97804"],

Cell[BoxData[
 FractionBox["1318", "45"]], "Output",
 CellChangeTimes->{3.838113350195814*^9, 3.83820614563069*^9, 
  3.838208241313796*^9, 3.83823915334908*^9, 3.8382398638007708`*^9, 
  3.8382805854714727`*^9, 3.838290561746612*^9, 3.838292434156805*^9, 
  3.838398934584552*^9, 3.838701988392571*^9, 3.838702028621622*^9, 
  3.838702460983119*^9, 3.838801582223048*^9, 3.838801613949057*^9, 
  3.838802001796075*^9},
 CellLabel->
  "Out[298]=",ExpressionUUID->"23e80492-6498-4971-a2d2-f1d2c6ffc4c3"],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["1069", "5604"]}]], "Output",
 CellChangeTimes->{3.838113350195814*^9, 3.83820614563069*^9, 
  3.838208241313796*^9, 3.83823915334908*^9, 3.8382398638007708`*^9, 
  3.8382805854714727`*^9, 3.838290561746612*^9, 3.838292434156805*^9, 
  3.838398934584552*^9, 3.838701988392571*^9, 3.838702028621622*^9, 
  3.838702460983119*^9, 3.838801582223048*^9, 3.838801613949057*^9, 
  3.8388020018712473`*^9},
 CellLabel->
  "Out[299]=",ExpressionUUID->"e9d1719c-502f-4221-af4d-c8af04c58a8b"],

Cell[BoxData[
 RowBox[{"-", "0.94075660242683797287651677373304782298`16."}]], "Output",
 CellChangeTimes->{3.838113350195814*^9, 3.83820614563069*^9, 
  3.838208241313796*^9, 3.83823915334908*^9, 3.8382398638007708`*^9, 
  3.8382805854714727`*^9, 3.838290561746612*^9, 3.838292434156805*^9, 
  3.838398934584552*^9, 3.838701988392571*^9, 3.838702028621622*^9, 
  3.838702460983119*^9, 3.838801582223048*^9, 3.838801613949057*^9, 
  3.8388020019268427`*^9},
 CellLabel->
  "Out[300]=",ExpressionUUID->"bad878ed-7f94-47e9-a1ac-b39eb6619fc2"]
}, Open  ]],

Cell["\<\
Here, the value to compare with is -0.15850285 from Rudd, et al., J. Chem. \
Phys. (1968). \
\>", "Text",
 CellChangeTimes->{{3.8363816635194607`*^9, 3.836381687103815*^9}, {
   3.836382052279499*^9, 3.8363821686658697`*^9}, {3.836382337919899*^9, 
   3.836382396171382*^9}, {3.836382484807663*^9, 3.836382750836891*^9}, {
   3.836382810525259*^9, 3.836382826069696*^9}, {3.8363828880178957`*^9, 
   3.836383258458395*^9}, {3.836383461819162*^9, 3.836383484033667*^9}, 
   3.836385015426078*^9, {3.836385195910368*^9, 3.8363852281289177`*^9}, 
   3.8370008408941793`*^9},ExpressionUUID->"27e2231e-2fb2-4349-aca2-\
c81e7c2b6875"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"getLinearForm", "[", 
   RowBox[{
    RowBox[{"ineqs", ":", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"_LessEqual", "|", "_GreaterEqual"}], ")"}], ".."}], "}"}]}], 
    ",", "vars_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"mults", ",", "lpolys", ",", "rhs", ",", "lhs"}], "}"}], ",", 
    RowBox[{
     RowBox[{"mults", "=", 
      RowBox[{
       RowBox[{"Map", "[", 
        RowBox[{"Head", ",", "ineqs"}], "]"}], "/.", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"LessEqual", "->", "1"}], ",", 
         RowBox[{"GreaterEqual", "->", 
          RowBox[{"-", "1"}]}]}], "}"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"lpolys", "=", 
      RowBox[{"mults", "*", 
       RowBox[{"Apply", "[", 
        RowBox[{"Subtract", ",", "ineqs", ",", 
         RowBox[{"{", "1", "}"}]}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"rhs", ",", "lhs"}], "}"}], "=", 
      RowBox[{"Normal", "[", 
       RowBox[{"CoefficientArrays", "[", 
        RowBox[{"lpolys", ",", "vars"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"lhs", ",", 
       RowBox[{"-", "rhs"}]}], "}"}]}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SubInequality", "[", "j_", "]"}], ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "outList", "}"}], ",", 
    RowBox[{
     RowBox[{"outList", "=", 
      RowBox[{"Expand", "[", 
       RowBox[{
        RowBox[{"InequalityList", "[", 
         RowBox[{"[", 
          RowBox[{"Delete", "[", 
           RowBox[{
            RowBox[{"Range", "[", 
             RowBox[{"1", ",", "sites"}], "]"}], ",", "j"}], "]"}], "]"}], 
         "]"}], "/.", 
        RowBox[{
         RowBox[{"Solve", "[", 
          RowBox[{
           RowBox[{"EqualityList", "[", 
            RowBox[{"[", "j", "]"}], "]"}], ",", 
           RowBox[{"z", "[", 
            RowBox[{"[", 
             RowBox[{"deltaDir", "[", "j", "]"}], "]"}], "]"}], ",", 
           "Reals"}], "]"}], "[", 
         RowBox[{"[", 
          RowBox[{"1", ",", "1"}], "]"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"toDelete", " ", "=", " ", 
      RowBox[{"Position", "[", 
       RowBox[{"outList", ",", "True"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"outList", " ", "=", " ", 
      RowBox[{"Delete", "[", 
       RowBox[{"outList", ",", "toDelete"}], "]"}]}]}]}], 
   "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HRepFile", " ", "[", "j_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "H", "}"}], ",", 
    RowBox[{
     RowBox[{"coeffs", " ", "=", " ", 
      RowBox[{"Normal", "[", 
       RowBox[{"getLinearForm", "[", 
        RowBox[{
         RowBox[{"SubInequality", "[", "j", "]"}], ",", 
         RowBox[{"Evaluate", "[", 
          RowBox[{"Delete", "[", 
           RowBox[{"z", ",", 
            RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], "]"}]}], "]"}], 
       "]"}]}], ";", 
     RowBox[{"toDelete", " ", "=", " ", 
      RowBox[{"Position", "[", 
       RowBox[{
        RowBox[{"coeffs", "[", 
         RowBox[{"[", "2", "]"}], "]"}], ",", "False"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"coeffs", "[", 
       RowBox[{"[", "1", "]"}], "]"}], "=", 
      RowBox[{"Delete", "[", 
       RowBox[{
        RowBox[{"coeffs", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", "toDelete"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"coeffs", "[", 
       RowBox[{"[", "2", "]"}], "]"}], "=", 
      RowBox[{"Delete", "[", 
       RowBox[{
        RowBox[{"coeffs", "[", 
         RowBox[{"[", "2", "]"}], "]"}], ",", "toDelete"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"H", " ", "=", " ", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Transpose", "[", 
         RowBox[{"{", 
          RowBox[{"coeffs", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "}"}], "]"}], ",", 
        RowBox[{"-", 
         RowBox[{"coeffs", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ",", "2"}], "]"}]}]}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"polyExp", "[", "j_", "]"}], ":=", 
   RowBox[{"CoefficientRules", "[", 
    RowBox[{
     RowBox[{"ReducedPoly", "[", "j", "]"}], ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"Delete", "[", 
       RowBox[{"z", ",", 
        RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"evenPoly", "[", 
    RowBox[{"p_", ",", "var_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"p", "/.", 
        RowBox[{"var", "->", "x"}]}], ")"}], "-", 
      RowBox[{"(", 
       RowBox[{"p", "/.", 
        RowBox[{"var", "->", 
         RowBox[{"-", "x"}]}]}], ")"}]}], "]"}], "===", "0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"reflectionRows", "[", 
    RowBox[{"H_", ",", "p_", ",", "subspace_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"added", "=", 
       RowBox[{"{", "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"sortedHRep", " ", "=", " ", 
       RowBox[{"Sort", "[", "H", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"k", "=", "2"}], ",", 
        RowBox[{"k", "<=", 
         RowBox[{
          RowBox[{"Dimensions", "[", "H", "]"}], "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", 
        RowBox[{"k", "++"}], ",", 
        RowBox[{
         RowBox[{"test", "=", " ", "H"}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"test", "[", 
           RowBox[{"[", 
            RowBox[{";;", ",", "k"}], "]"}], "]"}], "=", 
          RowBox[{"-", 
           RowBox[{"test", "[", 
            RowBox[{"[", 
             RowBox[{";;", ",", "k"}], "]"}], "]"}]}]}], ";", 
         RowBox[{"vect", " ", "=", " ", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"0", ",", 
            RowBox[{
             RowBox[{"Dimensions", "[", "H", "]"}], "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"vect", "[", 
           RowBox[{"[", "k", "]"}], "]"}], " ", "=", " ", "1"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Sort", "[", "test", "]"}], "===", "sortedHRep"}], " ", "&&",
             " ", 
            RowBox[{"evenPoly", "[", 
             RowBox[{"p", ",", 
              RowBox[{"subspace", "[", 
               RowBox[{"[", 
                RowBox[{"k", "-", "1"}], "]"}], "]"}]}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"added", " ", "=", " ", 
            RowBox[{"Join", "[", 
             RowBox[{"added", ",", 
              RowBox[{"{", "vect", "}"}], ",", "1"}], "]"}]}]}], 
          "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "added", "]"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"JustReflections", "[", "j_", "]"}], ":=", 
   RowBox[{"reflectionRows", "[", 
    RowBox[{
     RowBox[{"HRepFile", "[", "j", "]"}], ",", 
     RowBox[{"ReducedPoly", "[", "j", "]"}], ",", 
     RowBox[{"Delete", "[", 
      RowBox[{"z", ",", 
       RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"HRepWithRef", "[", "j_", "]"}], ":=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"HRepFile", "[", "j", "]"}], ",", 
     RowBox[{"JustReflections", "[", "j", "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"j", "=", "1"}], ",", 
   RowBox[{"j", "<=", "sites"}], ",", 
   RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"H", " ", "=", " ", 
     RowBox[{"HRepWithRef", "[", "j", "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"overallCoeff", " ", "=", " ", 
     RowBox[{"2", "^", 
      RowBox[{
       RowBox[{"Dimensions", "[", 
        RowBox[{"JustReflections", "[", "j", "]"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"Export", "[", 
     RowBox[{
      RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/D2/\>\"", "<>", 
       RowBox[{"ToString", "[", "d", "]"}], "<>", "\"\</HRep-\>\"", "<>", 
       RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"Dimensions", "[", "H", "]"}], "}"}], ",", 
        RowBox[{"Round", "[", 
         RowBox[{
          SuperscriptBox["10", "14"], "*", "H"}], "]"}]}], "]"}], ",", 
      "\"\<Table\>\"", ",", 
      RowBox[{"\"\<FieldSeparators\>\"", "->", "\"\< \>\""}]}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"polySet", " ", "=", " ", 
     RowBox[{"polyExp", "[", "j", "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"For", "[", 
     RowBox[{
      RowBox[{"k", "=", "1"}], ",", 
      RowBox[{"k", "<=", 
       RowBox[{"Length", "[", "polySet", "]"}]}], ",", 
      RowBox[{"k", "++"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Export", "[", 
        RowBox[{
         RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/D2/\>\"", "<>", 
          RowBox[{"ToString", "[", "d", "]"}], "<>", "\"\</Polynomial-\>\"", "<>", 
          RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
          RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<.dat\>\""}], ",", 
         RowBox[{"\"\<[[1,[\>\"", "<>", 
          RowBox[{"StringRiffle", "[", 
           RowBox[{
            RowBox[{"polySet", "[", 
             RowBox[{"[", 
              RowBox[{"k", ",", "1"}], "]"}], "]"}], ",", "\"\<,\>\""}], 
           "]"}], "<>", "\"\<]]]\>\""}]}], "]"}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"Export", "[", 
        RowBox[{
         RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/D2/\>\"", "<>", 
          RowBox[{"ToString", "[", "d", "]"}], "<>", "\"\</Coefficient-\>\"", 
          "<>", 
          RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
          RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<.dat\>\""}], ",", 
         RowBox[{"N", "[", 
          RowBox[{
           RowBox[{"overallCoeff", "*", 
            RowBox[{"polySet", "[", 
             RowBox[{"[", 
              RowBox[{"k", ",", "2"}], "]"}], "]"}]}], ",", "16"}], "]"}]}], 
        "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}]}], 
  "\[IndentingNewLine]", "]"}]}], "Input",
 CellChangeTimes->{{3.8373500086037416`*^9, 3.837350036548127*^9}, {
   3.83735058290853*^9, 3.8373505834335403`*^9}, {3.837355022833006*^9, 
   3.8373550242050133`*^9}, 3.837355270435058*^9, 3.837355324892654*^9, {
   3.837355356082716*^9, 3.8373553961360064`*^9}, {3.8373568409874477`*^9, 
   3.8373568578533363`*^9}, {3.83750979105827*^9, 3.837509792187233*^9}, 
   3.8375098257316523`*^9, {3.8381131077814407`*^9, 3.838113108786839*^9}, 
   3.838113152551111*^9, 3.838113461014789*^9, {3.847983447459548*^9, 
   3.8479834481347647`*^9}, {3.8479840376349688`*^9, 
   3.847984037641726*^9}},ExpressionUUID->"c72d7132-ae94-49b9-8c00-\
2418e0b5459f"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"eachEInt", "[", 
     RowBox[{"j_", ",", "k_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"deltaCoeffIn", "[", "ineq_", "]"}], ":=", 
       RowBox[{"(", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"solution", " ", "=", " ", 
          RowBox[{"Solve", "[", 
           RowBox[{"ineq", ",", "Reals"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Length", "[", "solution", "]"}], "==", "0"}], ",", 
           RowBox[{"Return", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", "}"}], ",", 
              RowBox[{"{", "}"}], ",", "0"}], "}"}], "]"}]}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"subs", " ", "=", "  ", 
          RowBox[{
           RowBox[{"Solve", "[", 
            RowBox[{"ineq", ",", "Reals"}], "]"}], "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"intVar", " ", "=", " ", 
          RowBox[{"subs", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"coeffs", " ", "=", " ", 
          RowBox[{
           RowBox[{"Normal", "[", 
            RowBox[{"CoefficientArrays", "[", 
             RowBox[{"ineq", ",", "intVar"}], "]"}], "]"}], "[", 
           RowBox[{"[", 
            RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"Return", "[", 
          RowBox[{"{", 
           RowBox[{"subs", ",", "intVar", ",", "coeffs"}], "}"}], "]"}]}], 
        "\[IndentingNewLine]", ")"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"sub", ",", "inVar", ",", "coeff"}], "}"}], " ", "=", " ", 
       RowBox[{"deltaCoeffIn", "[", 
        RowBox[{"EqualityList", "[", 
         RowBox[{"[", "j", "]"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"EqualityList", "[", 
            RowBox[{"[", "k", "]"}], "]"}], "//.", "sub"}], ")"}], "===", 
         "False"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "0", "]"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"sub2", ",", "inVar2", ",", "coeff2"}], "}"}], " ", "=", " ", 
       
       RowBox[{"deltaCoeffIn", "[", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"EqualityList", "[", 
           RowBox[{"[", "k", "]"}], "]"}], "//.", "sub"}], ")"}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"coeff2", "==", "0"}], ",", 
        RowBox[{"Return", "[", "0", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"poly", " ", "=", " ", 
       RowBox[{
        FractionBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             FractionBox["1", "2"], "*", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Magz", "[", "j", "]"}], " ", "-", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"w", "[", 
                   RowBox[{"[", "j", "]"}], "]"}], ".", "z"}], ")"}], "2"]}], 
              ")"}]}], ")"}], "*", 
           RowBox[{"(", 
            RowBox[{
             FractionBox["1", "2"], "*", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Magz", "[", "k", "]"}], " ", "-", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"w", "[", 
                   RowBox[{"[", "k", "]"}], "]"}], ".", "z"}], ")"}], "2"]}], 
              ")"}]}], ")"}]}], ")"}], 
         RowBox[{
          RowBox[{"Abs", "[", "coeff", "]"}], "*", 
          RowBox[{"Abs", "[", "coeff2", "]"}]}]], "//.", 
        RowBox[{"{", 
         RowBox[{"sub", ",", "sub2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"vars", "=", 
       RowBox[{"Delete", "[", 
        RowBox[{"z", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"Position", "[", 
             RowBox[{"z", ",", "inVar"}], "]"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", 
           RowBox[{
            RowBox[{"Position", "[", 
             RowBox[{"z", ",", "inVar2"}], "]"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ineqList", " ", "=", " ", 
       RowBox[{
        RowBox[{"And", "@@", "InequalityList"}], "//.", 
        RowBox[{"{", 
         RowBox[{"sub", ",", "sub2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "vars", "]"}], ">", "0"}], " ", "&&", " ", 
         RowBox[{"!", 
          RowBox[{"ineqList", "===", "0"}]}], " ", "&&", " ", 
         RowBox[{
          RowBox[{"RegionDimension", "[", 
           RowBox[{"ImplicitRegion", "[", 
            RowBox[{"ineqList", ",", 
             RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}], "]"}], ">", 
          "0"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Return", "[", 
          RowBox[{"Integrate", "[", 
           RowBox[{
            RowBox[{"FullSimplify", "[", "poly", "]"}], ",", 
            RowBox[{"vars", "\[Element]", 
             RowBox[{"ImplicitRegion", "[", 
              RowBox[{
               RowBox[{"FullSimplify", "[", "ineqList", "]"}], ",", 
               RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}]}]}], "]"}], 
          "]"}], ";"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "0", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"eachEInt", "[", 
      RowBox[{"j", ",", "k"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", 
       RowBox[{"sites", "-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"j", "+", "1"}], ",", "sites"}], "}"}]}], "]"}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{"Int22", "=", 
  RowBox[{
   RowBox[{"Total", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Integrate", "[", 
       RowBox[{
        RowBox[{"FullSimplify", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{"-", 
               RowBox[{
                RowBox[{"w", "[", 
                 RowBox[{"[", "j", "]"}], "]"}], ".", "z"}]}], "*", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"Magz", "[", "j", "]"}], " ", "-", " ", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"w", "[", 
                    RowBox[{"[", "j", "]"}], "]"}], ".", "z"}], ")"}], "2"]}],
                ")"}]}], 
             RowBox[{"Abs", "[", 
              RowBox[{"w", "[", 
               RowBox[{"[", 
                RowBox[{"j", ",", 
                 RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], "]"}], "]"}]], 
            "-", 
            FractionBox[
             RowBox[{"D", "[", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FractionBox["1", "2"], "*", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"Magz", "[", "j", "]"}], " ", "-", " ", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"w", "[", 
                    RowBox[{"[", "j", "]"}], "]"}], ".", "z"}], ")"}], "2"]}],
                    ")"}]}], ")"}], "2"], ",", 
               RowBox[{"z", "[", 
                RowBox[{"[", 
                 RowBox[{"deltaDir", "[", "j", "]"}], "]"}], "]"}]}], "]"}], 
             RowBox[{"Abs", "[", 
              RowBox[{"w", "[", 
               RowBox[{"[", 
                RowBox[{"j", ",", 
                 RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], "]"}], 
              "]"}]]}], ")"}], "/.", 
          RowBox[{
           RowBox[{"Solve", "[", 
            RowBox[{
             RowBox[{"EqualityList", "[", 
              RowBox[{"[", "j", "]"}], "]"}], ",", 
             RowBox[{"z", "[", 
              RowBox[{"[", 
               RowBox[{"deltaDir", "[", "j", "]"}], "]"}], "]"}], ",", 
             "Reals"}], "]"}], "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "1"}], "]"}], "]"}]}], "]"}], ",", 
        RowBox[{
         RowBox[{"Evaluate", "[", 
          RowBox[{"Delete", "[", 
           RowBox[{"z", ",", 
            RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], "]"}], "\[Element]",
          " ", 
         RowBox[{"ImplicitRegion", "[", 
          RowBox[{
           RowBox[{"FullSimplify", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"And", "@@", 
               RowBox[{"InequalityList", "[", 
                RowBox[{"[", 
                 RowBox[{"Delete", "[", 
                  RowBox[{
                   RowBox[{"Range", "[", 
                    RowBox[{"1", ",", "sites"}], "]"}], ",", "j"}], "]"}], 
                 "]"}], "]"}]}], ")"}], "/.", 
             RowBox[{
              RowBox[{"Solve", "[", 
               RowBox[{
                RowBox[{"EqualityList", "[", 
                 RowBox[{"[", "j", "]"}], "]"}], ",", 
                RowBox[{"z", "[", 
                 RowBox[{"[", 
                  RowBox[{"deltaDir", "[", "j", "]"}], "]"}], "]"}], ",", 
                "Reals"}], "]"}], "[", 
              RowBox[{"[", 
               RowBox[{"1", ",", "1"}], "]"}], "]"}]}], "]"}], ",", 
           RowBox[{"Evaluate", "[", 
            RowBox[{"Delete", "[", 
             RowBox[{"z", ",", 
              RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], "]"}]}], 
          "]"}]}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], "]"}], "]"}], "+", 
   RowBox[{"Total", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"eachEInt", "[", 
        RowBox[{"j", ",", "k"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "1", ",", 
         RowBox[{"sites", "-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"j", "+", "1"}], ",", "sites"}], "}"}]}], "]"}], ",", "2"}], 
    "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["E", "2"], "=", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{"Int22", "/", "Int20"}], ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "2"], "*", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"Int21", "/", "Int20"}], ")"}], "2"]}], "  ", "-", " ", 
    RowBox[{"2", "*", 
     SubscriptBox["E", "1"]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  SubscriptBox["E", "2"], "]"}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["d", "2"], "-", "1"}], "12"], "-", 
   RowBox[{"d", "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         SubscriptBox["D", "2"], "d"], "+", 
        FractionBox[
         RowBox[{"(", 
          RowBox[{"d", "-", "1"}], ")"}], "2"]}], ")"}], "+", 
      RowBox[{"2", "*", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          SubscriptBox["E", "2"], 
          SuperscriptBox["d", "2"]], "-", 
         RowBox[{
          FractionBox[
           RowBox[{"d", "+", "1"}], 
           RowBox[{"2", "*", 
            SuperscriptBox["d", "2"]}]], "*", 
          SubscriptBox["D", "2"]}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"5", "*", "d"}], "-", "1"}], ")"}], "*", 
           RowBox[{"(", 
            RowBox[{"1", "-", "d"}], ")"}]}], 
          RowBox[{"24", "*", "d"}]]}], ")"}]}]}], ")"}]}]}], "]"}]}], "Input",\

 CellChangeTimes->{{3.838109188970739*^9, 3.838109325640052*^9}, {
   3.838109784744829*^9, 3.838109874267909*^9}, {3.838110148752434*^9, 
   3.838110176507471*^9}, 3.8381104086931477`*^9, {3.8381139457432632`*^9, 
   3.838113947586849*^9}, 3.83811438030157*^9, 3.838114586361844*^9, 
   3.838702452139256*^9, 
   3.838702491501173*^9},ExpressionUUID->"733031dd-97c3-455b-96c5-\
cdb1866ae13a"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"integrationPartsDoubleDelta", "[", 
   RowBox[{"j_", ",", "k_"}], "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"deltaCoeffIn", "[", "ineq_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"solution", " ", "=", " ", 
        RowBox[{"Solve", "[", 
         RowBox[{"ineq", ",", "Reals"}], "]"}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "solution", "]"}], "==", "0"}], ",", 
         RowBox[{"Return", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", "}"}], ",", 
            RowBox[{"{", "}"}], ",", "0"}], "}"}], "]"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"subs", " ", "=", "  ", 
        RowBox[{"solution", "[", 
         RowBox[{"[", 
          RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"intVar", " ", "=", " ", 
        RowBox[{"subs", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"coeffs", " ", "=", " ", 
        RowBox[{
         RowBox[{"Normal", "[", 
          RowBox[{"CoefficientArrays", "[", 
           RowBox[{"ineq", ",", "intVar"}], "]"}], "]"}], "[", 
         RowBox[{"[", 
          RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"subs", ",", "intVar", ",", "coeffs"}], "}"}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"sub", ",", "inVar", ",", "coeff"}], "}"}], " ", "=", " ", 
     RowBox[{"deltaCoeffIn", "[", 
      RowBox[{"EqualityList", "[", 
       RowBox[{"[", "j", "]"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"EqualityList", "[", 
          RowBox[{"[", "k", "]"}], "]"}], "//.", "sub"}], ")"}], "===", 
       "False"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"{", "}"}], ",", "InequalityList"}], "}"}], "]"}]}], 
     "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"sub2", ",", "inVar2", ",", "coeff2"}], "}"}], " ", "=", " ", 
     RowBox[{"deltaCoeffIn", "[", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"EqualityList", "[", 
         RowBox[{"[", "k", "]"}], "]"}], "//.", "sub"}], ")"}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"coeff2", "==", "0"}], ",", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"{", "}"}], ",", "InequalityList"}], "}"}], "]"}]}], "]"}], 
    ";", "\[IndentingNewLine]", 
    RowBox[{"poly", " ", "=", " ", 
     RowBox[{
      FractionBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           FractionBox["1", "2"], "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Magz", "[", "j", "]"}], " ", "-", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{"w", "[", 
                 RowBox[{"[", "j", "]"}], "]"}], ".", "z"}], ")"}], "2"]}], 
            ")"}]}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{
           FractionBox["1", "2"], "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Magz", "[", "j", "]"}], " ", "-", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{"w", "[", 
                 RowBox[{"[", "k", "]"}], "]"}], ".", "z"}], ")"}], "2"]}], 
            ")"}]}], ")"}]}], ")"}], 
       RowBox[{
        RowBox[{"Abs", "[", "coeff", "]"}], "*", 
        RowBox[{"Abs", "[", "coeff2", "]"}]}]], "//.", 
      RowBox[{"{", 
       RowBox[{"sub", ",", "sub2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"vars", "=", 
     RowBox[{"Delete", "[", 
      RowBox[{"z", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"Position", "[", 
           RowBox[{"z", ",", "inVar"}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Position", "[", 
           RowBox[{"z", ",", "inVar2"}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], "}"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"ineqList", " ", "=", " ", 
     RowBox[{"InequalityList", "//.", 
      RowBox[{"{", 
       RowBox[{"sub", ",", "sub2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vars", "]"}], ">", "0"}], " ", "&&", " ", 
       RowBox[{"!", 
        RowBox[{"ineqList", "===", "0"}]}], "  ", "&&", " ", 
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"FullSimplify", "[", 
           RowBox[{"And", "@@", "ineq"}], "]"}], "===", "False"}], ")"}]}]}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"poly", ",", "vars", ",", "ineqList"}], "}"}], "]"}], ";"}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"0", ",", "vars", ",", "False"}], "}"}], "]"}], ";"}]}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
   ")"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"eachEDoubleDeltaInt", "[", 
   RowBox[{"j_", ",", "k_"}], "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{"poly", ",", "vars", ",", "ineqList"}], "}"}], " ", "=", " ", 
     RowBox[{"integrationPartsDoubleDelta", "[", 
      RowBox[{"j", ",", "k"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", " ", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vars", "]"}], ">", "0"}], " ", "&&", " ", 
       RowBox[{
        RowBox[{"Length", "[", "ineqList", "]"}], ">", "0"}], " ", "&&", 
       RowBox[{
        RowBox[{"RegionDimension", "[", 
         RowBox[{"ImplicitRegion", "[", 
          RowBox[{"ineqList", ",", 
           RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}], "]"}], "==", 
        RowBox[{
         RowBox[{"2", "*", "d"}], "-", "2"}]}]}], ",", "\[IndentingNewLine]", 
      
      RowBox[{
       RowBox[{"Return", "[", 
        RowBox[{"Integrate", "[", 
         RowBox[{"poly", ",", 
          RowBox[{"vars", "\[Element]", 
           RowBox[{"ImplicitRegion", "[", 
            RowBox[{
             RowBox[{"And", "@@", "ineqList"}], ",", 
             RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}]}]}], "]"}], 
        "]"}], ";"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "0", "]"}]}], "]"}], ";"}], 
   "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"integrationPartsSingleDelta", "[", "j_", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"solution", " ", "=", " ", 
     RowBox[{
      RowBox[{"Solve", "[", 
       RowBox[{
        RowBox[{"EqualityList", "[", 
         RowBox[{"[", "j", "]"}], "]"}], ",", 
        RowBox[{"z", "[", 
         RowBox[{"[", 
          RowBox[{"deltaDir", "[", "j", "]"}], "]"}], "]"}], ",", "Reals"}], 
       "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"poly", " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"-", 
           RowBox[{
            RowBox[{"w", "[", 
             RowBox[{"[", "j", "]"}], "]"}], ".", "z"}]}], "*", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Magz", "[", "j", "]"}], " ", "-", " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{"w", "[", 
                RowBox[{"[", "j", "]"}], "]"}], ".", "z"}], ")"}], "2"]}], 
           ")"}]}], 
         RowBox[{"Abs", "[", 
          RowBox[{"w", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", 
             RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], "]"}], "]"}]], "-", 
        FractionBox[
         RowBox[{"D", "[", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FractionBox["1", "2"], "*", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"Magz", "[", "j", "]"}], " ", "-", " ", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"w", "[", 
                    RowBox[{"[", "j", "]"}], "]"}], ".", "z"}], ")"}], "2"]}],
                ")"}]}], ")"}], "2"], ",", 
           RowBox[{"z", "[", 
            RowBox[{"[", 
             RowBox[{"deltaDir", "[", "j", "]"}], "]"}], "]"}]}], "]"}], 
         RowBox[{"Abs", "[", 
          RowBox[{"w", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", 
             RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}], "]"}], "]"}]]}], 
       ")"}], "/.", "solution"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"vars", " ", "=", " ", 
     RowBox[{"Delete", "[", 
      RowBox[{"z", ",", 
       RowBox[{"deltaDir", "[", "j", "]"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"ineqList", " ", "=", " ", 
     RowBox[{
      RowBox[{"InequalityList", "[", 
       RowBox[{"[", 
        RowBox[{"Delete", "[", 
         RowBox[{
          RowBox[{"Range", "[", 
           RowBox[{"1", ",", "sites"}], "]"}], ",", "j"}], "]"}], "]"}], 
       "]"}], "/.", "solution"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"Return", "[", 
     RowBox[{"{", 
      RowBox[{"poly", ",", "vars", ",", "ineqList"}], "}"}], "]"}], ";"}], 
   "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"eachESingleDeltaInt", "[", "j_", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{"poly", ",", "vars", ",", "ineqList"}], "}"}], "=", 
     RowBox[{"integrationPartsSingleDelta", "[", "j", "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"0", "<", 
       RowBox[{"RegionDimension", "[", 
        RowBox[{"ImplicitRegion", "[", 
         RowBox[{"ineqList", ",", 
          RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}], "]"}], "<", 
       RowBox[{
        RowBox[{"2", "*", "d"}], "-", "1"}]}], ",", 
      RowBox[{
       RowBox[{"Print", "[", "j", "]"}], ";"}]}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"If", " ", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vars", "]"}], ">", "0"}], " ", "&&", " ", 
       RowBox[{
        RowBox[{"Length", "[", "ineqList", "]"}], ">", "0"}], " ", "&&", " ", 
       
       RowBox[{
        RowBox[{"RegionDimension", "[", 
         RowBox[{"ImplicitRegion", "[", 
          RowBox[{"ineqList", ",", 
           RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}], "]"}], "==", 
        RowBox[{
         RowBox[{"2", "*", "d"}], "-", "1"}]}]}], ",", "\[IndentingNewLine]", 
      
      RowBox[{
       RowBox[{"Return", "[", 
        RowBox[{"Integrate", "[", 
         RowBox[{"poly", ",", 
          RowBox[{"vars", "\[Element]", 
           RowBox[{"ImplicitRegion", "[", 
            RowBox[{
             RowBox[{"And", "@@", 
              RowBox[{"(", "ineqList", ")"}]}], ",", 
             RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}]}]}], "]"}], 
        "]"}], ";"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "0", "]"}]}], "]"}], ";"}], 
   "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"subInequality", "[", "ineq_", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"toDelete", " ", "=", " ", 
     RowBox[{"Position", "[", 
      RowBox[{"ineq", ",", "True"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"Return", "[", 
     RowBox[{"Delete", "[", 
      RowBox[{"ineq", ",", "toDelete"}], "]"}], "]"}], ";"}], 
   "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HRepFile", " ", "[", 
     RowBox[{"ineqList_", ",", "vars_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "vars", "]"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"coeffs", " ", "=", " ", 
          RowBox[{"Normal", "[", 
           RowBox[{"getLinearForm", "[", 
            RowBox[{
             RowBox[{"subInequality", "[", "ineqList", "]"}], ",", 
             RowBox[{"Evaluate", "[", "vars", "]"}]}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"MemberQ", "[", 
            RowBox[{"ineqList", ",", "False"}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Return", "[", 
             RowBox[{"{", "}"}], "]"}], ";"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Return", "[", 
             RowBox[{"Join", "[", 
              RowBox[{
               RowBox[{"Transpose", "[", 
                RowBox[{"{", 
                 RowBox[{"coeffs", "[", 
                  RowBox[{"[", "2", "]"}], "]"}], "}"}], "]"}], ",", 
               RowBox[{"-", 
                RowBox[{"coeffs", "[", 
                 RowBox[{"[", "1", "]"}], "]"}]}], ",", "2"}], "]"}], "]"}], 
            ";"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", 
         RowBox[{"{", "}"}], "]"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"For", "[", 
    RowBox[{
     RowBox[{"j", "=", "1"}], ",", 
     RowBox[{"j", "<=", "sites"}], ",", 
     RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{"poly", ",", "vars", ",", "ineq"}], "}"}], "=", 
       RowBox[{"integrationPartsSingleDelta", "[", "j", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ineq", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"oneSide", "/@", 
          RowBox[{"FullSimplify", "[", "ineq", "]"}]}], "//", 
         "DeleteDuplicates"}], "//", 
        RowBox[{"DeleteCases", "[", "True", "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"H", " ", "=", " ", 
       RowBox[{"HRepFile", "[", 
        RowBox[{"ineq", ",", "vars"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Export", "[", 
       RowBox[{
        RowBox[{
        "\"\<basedir/latteIntegrals/MathematicaFormat/E2/\>\"", "<>", 
         RowBox[{"ToString", "[", "d", "]"}], "<>", "\"\</HRepSingle-\>\"", "<>", 
         RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Dimensions", "[", "H", "]"}], "}"}], ",", 
          RowBox[{"Round", "[", 
           RowBox[{
            SuperscriptBox["10", "14"], "*", "H"}], "]"}]}], "]"}], ",", 
        "\"\<Table\>\"", ",", 
        RowBox[{"\"\<FieldSeparators\>\"", "->", "\"\< \>\""}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"polySet", " ", "=", " ", 
       RowBox[{"polyExp", "[", 
        RowBox[{"poly", ",", "vars"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"posString", ",", "negString"}], "}"}], "=", 
       RowBox[{"polyStringWrite", "[", 
        RowBox[{"poly", ",", "vars"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"StringLength", "[", "posString", "]"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Export", "[", 
          RowBox[{
           RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/E2/\>\"", "<>", 
            RowBox[{"ToString", "[", "d", "]"}], "<>", 
            "\"\</PolynomialSinglePos-\>\"", "<>", 
            RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
           "posString"}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"Export", "[", 
          RowBox[{
           RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/E2/\>\"", "<>", 
            RowBox[{"ToString", "[", "d", "]"}], "<>", 
            "\"\</CoefficientSinglePos-\>\"", "<>", 
            RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
           
           RowBox[{"ToString", "[", 
            RowBox[{
             RowBox[{"AccountingForm", "[", "1*^-10", "]"}], "//", "N"}], 
            "]"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"StringLength", "[", "negString", "]"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Export", "[", 
          RowBox[{
           RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/E2/\>\"", "<>", 
            RowBox[{"ToString", "[", "d", "]"}], "<>", 
            "\"\</PolynomialSingleNeg-\>\"", "<>", 
            RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
           "negString"}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"Export", "[", 
          RowBox[{
           RowBox[{"\"\<basedir/latteIntegrals/MathematicaFormat/E2/\>\"", "<>", 
            RowBox[{"ToString", "[", "d", "]"}], "<>", 
            "\"\</CoefficientSingleNeg-\>\"", "<>", 
            RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<.dat\>\""}], ",", 
           
           RowBox[{"\"\<-\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{
              RowBox[{"AccountingForm", "[", "1*^-10", "]"}], "//", "N"}], 
             "]"}]}]}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}],
     "\[IndentingNewLine]", "]"}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"j", "=", "1"}], ",", 
   RowBox[{"j", "<=", "sites"}], ",", 
   RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
   RowBox[{"For", "[", 
    RowBox[{
     RowBox[{"k", "=", 
      RowBox[{"j", "+", "1"}]}], ",", 
     RowBox[{"k", "<=", "sites"}], ",", 
     RowBox[{"k", "++"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{"poly", ",", "vars", ",", "ineq"}], "}"}], "=", 
       RowBox[{"integrationPartsDoubleDelta", "[", 
        RowBox[{"j", ",", "k"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", 
         RowBox[{"(", 
          RowBox[{"poly", " ", "===", "0"}], ")"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"ineq", "=", 
          RowBox[{
           RowBox[{"oneSide", "/@", "ineq"}], "//", 
           RowBox[{"DeleteCases", "[", "True", "]"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"H", " ", "=", " ", 
          RowBox[{"HRepFile", "[", 
           RowBox[{"ineq", ",", "vars"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Length", "[", "H", "]"}], ">", "0"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"overallCoeff", " ", "=", "1"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"Export", "[", 
             RowBox[{
              RowBox[{
              "\"\<basedir/latteIntegrals/MathematicaFormat/E2/\>\"", "<>", 
               RowBox[{"ToString", "[", "d", "]"}], "<>", 
               "\"\</HRepDouble-\>\"", "<>", 
               RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
               RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<.dat\>\""}], 
              ",", 
              RowBox[{"Join", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"Dimensions", "[", "H", "]"}], "}"}], ",", 
                RowBox[{"Round", "[", 
                 RowBox[{
                  SuperscriptBox["10", "14"], "*", "H"}], "]"}]}], "]"}], 
              ",", "\"\<Table\>\"", ",", 
              RowBox[{"\"\<FieldSeparators\>\"", "->", "\"\< \>\""}]}], "]"}],
             ";", "\[IndentingNewLine]", 
            RowBox[{"polySet", " ", "=", " ", 
             RowBox[{"polyExp", "[", 
              RowBox[{"poly", ",", "vars"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"posString", ",", "negString"}], "}"}], "=", 
             RowBox[{"polyStringWrite", "[", 
              RowBox[{"poly", ",", "vars"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"StringLength", "[", "posString", "]"}], ">", "0"}], 
              ",", 
              RowBox[{
               RowBox[{
                RowBox[{"Export", "[", 
                 RowBox[{
                  RowBox[{
                  "\"\<basedir/latteIntegrals/MathematicaFormat/E2/\>\"", "<>", 
                   RowBox[{"ToString", "[", "d", "]"}], "<>", 
                   "\"\</PolynomialDoublePos-\>\"", "<>", 
                   RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
                   RowBox[{"ToString", "[", "k", "]"}], "<>", 
                   "\"\<.dat\>\""}], ",", "posString"}], "]"}], 
                "\[IndentingNewLine]", 
                RowBox[{"Export", "[", 
                 RowBox[{
                  RowBox[{
                  "\"\<basedir/latteIntegrals/MathematicaFormat/E2/\>\"", "<>", 
                   RowBox[{"ToString", "[", "d", "]"}], "<>", 
                   "\"\</CoefficientDoublePos-\>\"", "<>", 
                   RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
                   RowBox[{"ToString", "[", "k", "]"}], "<>", 
                   "\"\<.dat\>\""}], ",", 
                  RowBox[{"ToString", "[", 
                   RowBox[{
                    RowBox[{"AccountingForm", "[", "1*^-10", "]"}], "//", 
                    "N"}], "]"}]}], "]"}]}], ";"}]}], "\[IndentingNewLine]", 
             "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"StringLength", "[", "negString", "]"}], ">", "0"}], 
              ",", 
              RowBox[{
               RowBox[{"Export", "[", 
                RowBox[{
                 RowBox[{
                 "\"\<basedir/latteIntegrals/MathematicaFormat/E2/\>\"", "<>", 
                  RowBox[{"ToString", "[", "d", "]"}], "<>", 
                  "\"\</PolynomialDoubleNeg-\>\"", "<>", 
                  RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
                  RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<.dat\>\""}],
                  ",", "negString"}], "]"}], ";", "\[IndentingNewLine]", 
               RowBox[{"Export", "[", 
                RowBox[{
                 RowBox[{
                 "\"\<basedir/latteIntegrals/MathematicaFormat/E2/\>\"", "<>", 
                  RowBox[{"ToString", "[", "d", "]"}], "<>", 
                  "\"\</CoefficientDoubleNeg-\>\"", "<>", 
                  RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<-\>\"", "<>", 
                  RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<.dat\>\""}],
                  ",", 
                 RowBox[{"\"\<-\>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{
                    RowBox[{"AccountingForm", "[", "1*^-10", "]"}], "//", 
                    "N"}], "]"}]}]}], "]"}], ";"}]}], "\[IndentingNewLine]", 
             "]"}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
       "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  "\[IndentingNewLine]", "]"}]}], "Input",
 CellChangeTimes->{{3.838205793500214*^9, 3.8382058963061743`*^9}, {
   3.838205937626645*^9, 3.838206069482603*^9}, 3.838206245229589*^9, 
   3.8382083356377068`*^9, {3.838210629912551*^9, 3.838210648476387*^9}, {
   3.8382107126354103`*^9, 3.838210720497122*^9}, {3.838212439938354*^9, 
   3.8382124483999453`*^9}, {3.83821249067132*^9, 3.83821249406826*^9}, {
   3.838213153095707*^9, 3.8382132475237637`*^9}, {3.838213469225194*^9, 
   3.838213510455449*^9}, {3.8382220334812517`*^9, 3.838222109458503*^9}, {
   3.838222188041273*^9, 3.8382222892067842`*^9}, {3.8382397186269493`*^9, 
   3.8382397192569942`*^9}, {3.838280412389112*^9, 3.8382804872519693`*^9}, 
   3.838290392254016*^9, {3.8382904254658403`*^9, 3.838290451878167*^9}, {
   3.8382932079414597`*^9, 3.838293214733433*^9}, {3.838293524943301*^9, 
   3.838293535269477*^9}, {3.8382935913521643`*^9, 3.838293641491974*^9}, {
   3.8382944632746973`*^9, 3.838294503551024*^9}, {3.8382979363060417`*^9, 
   3.8382979599727716`*^9}, {3.838298347656946*^9, 3.838298353616577*^9}, {
   3.838298431701371*^9, 3.838298436871513*^9}, {3.838298501813876*^9, 
   3.8382985392636957`*^9}, {3.838298583472804*^9, 3.838298625320547*^9}, {
   3.838298763384967*^9, 3.838298783996244*^9}, {3.8382995251127853`*^9, 
   3.83829958431798*^9}, {3.838299996336713*^9, 3.8383000102107887`*^9}, {
   3.83830006924303*^9, 3.838300074409604*^9}, {3.838300123659738*^9, 
   3.8383001283369923`*^9}, {3.838300167946225*^9, 3.838300170919997*^9}, {
   3.838300223112505*^9, 3.838300251582357*^9}, 3.838303234386999*^9, {
   3.8383041393261957`*^9, 3.83830414150773*^9}, {3.838399813641817*^9, 
   3.838399845906262*^9}, 3.838702520721587*^9, 3.8387090409929028`*^9, {
   3.847983343859692*^9, 3.8479834265004587`*^9}, {3.847984037647903*^9, 
   3.847984037693619*^9}},ExpressionUUID->"f715c12c-f408-4aec-afba-\
fb64eb8ff085"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Int22a", " ", "=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"eachESingleDeltaInt", "[", "j", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "1", ",", "sites"}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Int22b", " ", "=", " ", 
  RowBox[{"Total", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"eachEDoubleDeltaInt", "[", 
       RowBox[{"j", ",", "k"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "1", ",", "sites"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", 
        RowBox[{"j", "+", "1"}], ",", "sites"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", "2", "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Int22", " ", "=", " ", 
  RowBox[{"Int22a", " ", "+", " ", "Int22b"}]}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", "Int22", "]"}], "\[IndentingNewLine]", 
 RowBox[{"Int22", " ", "=", " ", 
  RowBox[{"Total", "[", "Int22", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  RowBox[{"Int22", ",", "16"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["E", "2"], "=", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{"Int22", "/", "Int20"}], ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "2"], "*", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"Int21", "/", "Int20"}], ")"}], "2"]}], "  ", "-", " ", 
    RowBox[{"2", "*", 
     SubscriptBox["E", "1"]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  SubscriptBox["E", "2"], "]"}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["d", "2"], "-", "1"}], "12"], "-", 
   RowBox[{"d", "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         SubscriptBox["D", "2"], "d"], "+", 
        FractionBox[
         RowBox[{"(", 
          RowBox[{"d", "-", "1"}], ")"}], "2"]}], ")"}], "+", 
      RowBox[{"2", "*", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          SubscriptBox["E", "2"], 
          SuperscriptBox["d", "2"]], "-", 
         RowBox[{
          FractionBox[
           RowBox[{"d", "+", "1"}], 
           RowBox[{"2", "*", 
            SuperscriptBox["d", "2"]}]], "*", 
          SubscriptBox["D", "2"]}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"5", "*", "d"}], "-", "1"}], ")"}], "*", 
           RowBox[{"(", 
            RowBox[{"1", "-", "d"}], ")"}]}], 
          RowBox[{"24", "*", "d"}]]}], ")"}]}]}], ")"}]}]}], "]"}]}], "Input",\

 CellChangeTimes->{{3.838205793500214*^9, 3.8382058963061743`*^9}, {
   3.838205937626645*^9, 3.838206069482603*^9}, 3.838206245229589*^9, {
   3.8382083356377068`*^9, 3.838208337473199*^9}, {3.838208378087431*^9, 
   3.838208410471918*^9}, {3.8382089406345453`*^9, 3.838208964570418*^9}, {
   3.8382402946754293`*^9, 3.8382403030125217`*^9}, {3.838293679651389*^9, 
   3.8382936829440117`*^9}, {3.838297553596387*^9, 3.838297557680696*^9}, {
   3.838297738961132*^9, 3.838297778302532*^9}, {3.83829788083321*^9, 
   3.838297900752264*^9}, {3.838298657466765*^9, 3.83829866086414*^9}, {
   3.83829901519026*^9, 3.8382990494405727`*^9}, {3.838398682576109*^9, 
   3.838398691321274*^9}},
 CellLabel->
  "In[201]:=",ExpressionUUID->"dd01affd-8df1-425a-b5d5-cf195dcc272f"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["418", "45"]}], ",", 
   RowBox[{"-", 
    FractionBox["13", "6"]}], ",", 
   RowBox[{"-", 
    FractionBox["337", "180"]}], ",", 
   RowBox[{"-", 
    FractionBox["337", "180"]}], ",", 
   RowBox[{"-", 
    FractionBox["1367", "420"]}], ",", 
   RowBox[{"-", 
    FractionBox["1367", "420"]}], ",", 
   RowBox[{"-", 
    FractionBox["1367", "420"]}], ",", 
   RowBox[{"-", 
    FractionBox["1367", "420"]}], ",", 
   RowBox[{"-", 
    FractionBox["1763", "840"]}], ",", 
   RowBox[{"-", 
    FractionBox["1763", "840"]}], ",", 
   RowBox[{"-", 
    FractionBox["1763", "840"]}], ",", 
   RowBox[{"-", 
    FractionBox["1763", "840"]}], ",", 
   RowBox[{"-", 
    FractionBox["13", "6"]}], ",", 
   RowBox[{"-", 
    FractionBox["337", "180"]}], ",", 
   RowBox[{"-", 
    FractionBox["337", "180"]}], ",", 
   RowBox[{"-", 
    FractionBox["1793", "840"]}], ",", 
   RowBox[{"-", 
    FractionBox["1793", "840"]}], ",", 
   RowBox[{"-", 
    FractionBox["1793", "840"]}], ",", 
   RowBox[{"-", 
    FractionBox["1793", "840"]}], ",", 
   RowBox[{"-", 
    FractionBox["263", "140"]}], ",", 
   RowBox[{"-", 
    FractionBox["263", "140"]}], ",", 
   RowBox[{"-", 
    FractionBox["263", "140"]}], ",", 
   RowBox[{"-", 
    FractionBox["263", "140"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.838299083887054*^9, 3.83839909474685*^9, 
  3.838399910530839*^9, 3.838702703828643*^9},
 CellLabel->
  "Out[201]=",ExpressionUUID->"c993743d-3041-4090-8d90-7970b2b79265"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["60959", "1680"], ",", 
   FractionBox["221", "72"], ",", 
   FractionBox["5581", "2240"], ",", 
   FractionBox["5581", "2240"], ",", 
   FractionBox["77359", "20160"], ",", 
   FractionBox["56863", "20160"], ",", 
   FractionBox["62527", "20160"], ",", 
   FractionBox["42031", "20160"], ",", 
   FractionBox["3367", "2880"], ",", 
   FractionBox["2909", "5040"], ",", 
   FractionBox["3367", "2880"], ",", 
   FractionBox["2909", "5040"], ",", 
   FractionBox["221", "90"], ",", 
   FractionBox["20171", "10080"], ",", 
   FractionBox["20171", "10080"], ",", 
   FractionBox["23357", "20160"], ",", 
   FractionBox["17", "30"], ",", 
   FractionBox["23357", "20160"], ",", 
   FractionBox["17", "30"], ",", 
   FractionBox["184", "105"], ",", 
   FractionBox["103", "140"], ",", 
   FractionBox["61", "60"], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.838299083887054*^9, 3.83839909474685*^9, 
  3.838399910530839*^9, 3.8387027680033703`*^9},
 CellLabel->
  "Out[202]=",ExpressionUUID->"5093cfd7-0102-4405-91dc-ef4e44db7b25"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["136061", "5040"], ",", 
   FractionBox["65", "72"], ",", 
   FractionBox["2497", "4032"], ",", 
   FractionBox["2497", "4032"], ",", 
   FractionBox["11743", "20160"], ",", 
   RowBox[{"-", 
    FractionBox["8753", "20160"]}], ",", 
   RowBox[{"-", 
    FractionBox["3089", "20160"]}], ",", 
   RowBox[{"-", 
    FractionBox["4717", "4032"]}], ",", 
   RowBox[{"-", 
    FractionBox["18743", "20160"]}], ",", 
   RowBox[{"-", 
    FractionBox["7669", "5040"]}], ",", 
   RowBox[{"-", 
    FractionBox["18743", "20160"]}], ",", 
   RowBox[{"-", 
    FractionBox["7669", "5040"]}], ",", 
   FractionBox["13", "45"], ",", 
   FractionBox["433", "3360"], ",", 
   FractionBox["433", "3360"], ",", 
   RowBox[{"-", 
    FractionBox["3935", "4032"]}], ",", 
   RowBox[{"-", 
    FractionBox["439", "280"]}], ",", 
   RowBox[{"-", 
    FractionBox["3935", "4032"]}], ",", 
   RowBox[{"-", 
    FractionBox["439", "280"]}], ",", 
   RowBox[{"-", 
    FractionBox["53", "420"]}], ",", 
   RowBox[{"-", 
    FractionBox["8", "7"]}], ",", 
   RowBox[{"-", 
    FractionBox["181", "210"]}], ",", 
   RowBox[{"-", 
    FractionBox["263", "140"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.838299083887054*^9, 3.83839909474685*^9, 
  3.838399910530839*^9, 3.838702768082041*^9},
 CellLabel->
  "Out[203]=",ExpressionUUID->"d34792a8-53e9-473c-b768-25c50da32cd1"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.99623015873016`", ",", "0.9027777777777778`", ",", 
   "0.6192956349206349`", ",", "0.6192956349206349`", ",", 
   "0.5824900793650793`", ",", 
   RowBox[{"-", "0.4341765873015873`"}], ",", 
   RowBox[{"-", "0.15322420634920636`"}], ",", 
   RowBox[{"-", "1.169890873015873`"}], ",", 
   RowBox[{"-", "0.9297123015873016`"}], ",", 
   RowBox[{"-", "1.521626984126984`"}], ",", 
   RowBox[{"-", "0.9297123015873016`"}], ",", 
   RowBox[{"-", "1.521626984126984`"}], ",", "0.28888888888888886`", ",", 
   "0.1288690476190476`", ",", "0.1288690476190476`", ",", 
   RowBox[{"-", "0.9759424603174603`"}], ",", 
   RowBox[{"-", "1.5678571428571428`"}], ",", 
   RowBox[{"-", "0.9759424603174603`"}], ",", 
   RowBox[{"-", "1.5678571428571428`"}], ",", 
   RowBox[{"-", "0.1261904761904762`"}], ",", 
   RowBox[{"-", "1.1428571428571428`"}], ",", 
   RowBox[{"-", "0.861904761904762`"}], ",", 
   RowBox[{"-", "1.8785714285714286`"}]}], "}"}]], "Output",
 CellChangeTimes->{3.838299083887054*^9, 3.83839909474685*^9, 
  3.838399910530839*^9, 3.838702768142889*^9},
 CellLabel->
  "Out[204]=",ExpressionUUID->"f826fb3b-2e6a-4c90-b4c0-2d6cfedaeb21"],

Cell[BoxData[
 FractionBox["146257", "10080"]], "Output",
 CellChangeTimes->{3.838299083887054*^9, 3.83839909474685*^9, 
  3.838399910530839*^9, 3.8387027682205667`*^9},
 CellLabel->
  "Out[205]=",ExpressionUUID->"7c20fee8-5f7a-4cf9-94ba-0b99070eecb4"],

Cell[BoxData["14.50962301587301587301587301587301587301`16."], "Output",
 CellChangeTimes->{3.838299083887054*^9, 3.83839909474685*^9, 
  3.838399910530839*^9, 3.838702768267523*^9},
 CellLabel->
  "Out[206]=",ExpressionUUID->"4a2ad2e4-c4a8-4023-8ecc-24d1d5ac779c"],

Cell[BoxData[
 RowBox[{"-", "0.8141613559707348`"}]], "Output",
 CellChangeTimes->{3.838299083887054*^9, 3.83839909474685*^9, 
  3.838399910530839*^9, 3.838702768327446*^9},
 CellLabel->
  "Out[208]=",ExpressionUUID->"8b795e7e-d7ed-4770-9a94-6b33a9450d58"],

Cell[BoxData["0.4791887031715439`"], "Output",
 CellChangeTimes->{3.838299083887054*^9, 3.83839909474685*^9, 
  3.838399910530839*^9, 3.838702768384625*^9},
 CellLabel->
  "Out[209]=",ExpressionUUID->"8ceaec49-0ab3-4127-902a-ad14f00237e3"]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"polyStringWrite", "[", 
     RowBox[{"poly_", ",", "vars_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"polySet", " ", "=", " ", 
       RowBox[{"polyExp", "[", 
        RowBox[{"poly", ",", "vars"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"negSet", " ", "=", " ", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"polySet", "[", 
             RowBox[{"[", 
              RowBox[{";;", ",", "2"}], "]"}], "]"}], "//", "N"}], ")"}], ",", 
          RowBox[{"_", "?", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"#", "<", "0"}], "&"}], ")"}]}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"posSet", " ", "=", " ", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"polySet", "[", 
             RowBox[{"[", 
              RowBox[{";;", ",", "2"}], "]"}], "]"}], "//", "N"}], ")"}], ",", 
          RowBox[{"_", "?", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"#", ">=", "0"}], "&"}], ")"}]}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "posSet", "]"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"posSet", " ", "=", " ", 
          RowBox[{"polySet", "[", 
           RowBox[{"[", "posSet", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         
         RowBox[{"posString", " ", "=", " ", "\"\<[\>\""}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"m", "=", "1"}], ",", 
           RowBox[{"m", "<=", 
            RowBox[{"Length", "[", "posSet", "]"}]}], ",", 
           RowBox[{"m", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"posString", " ", "=", " ", 
             RowBox[{"posString", "<>", "\"\<[\>\"", "<>", 
              RowBox[{"ToString", "[", 
               RowBox[{"Round", "[", 
                RowBox[{
                 SuperscriptBox["10", "10"], "*", 
                 RowBox[{"posSet", "[", 
                  RowBox[{"[", 
                   RowBox[{"m", ",", "2"}], "]"}], "]"}]}], "]"}], "]"}], 
              "<>", "\"\<,[\>\"", "<>", 
              RowBox[{"StringRiffle", "[", 
               RowBox[{
                RowBox[{"posSet", "[", 
                 RowBox[{"[", 
                  RowBox[{"m", ",", "1"}], "]"}], "]"}], ",", "\"\<,\>\""}], 
               "]"}], "<>", "\"\<]],\>\""}]}], ";"}]}], "\[IndentingNewLine]",
           "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"posString", " ", "=", " ", 
          RowBox[{
           RowBox[{"StringDrop", "[", 
            RowBox[{"posString", ",", 
             RowBox[{"-", "1"}]}], "]"}], "<>", "\"\<]\>\""}]}], ";"}], 
        "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"posString", "=", "\"\<\>\""}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "negSet", "]"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"negSet", " ", "=", " ", 
          RowBox[{"polySet", "[", 
           RowBox[{"[", "negSet", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         
         RowBox[{"negString", " ", "=", " ", "\"\<[\>\""}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"m", "=", "1"}], ",", 
           RowBox[{"m", "<=", 
            RowBox[{"Length", "[", "negSet", "]"}]}], ",", 
           RowBox[{"m", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"negString", " ", "=", " ", 
             RowBox[{"negString", "<>", "\"\<[\>\"", "<>", 
              RowBox[{"ToString", "[", 
               RowBox[{"Round", "[", 
                RowBox[{
                 RowBox[{"-", 
                  SuperscriptBox["10", "10"]}], "*", 
                 RowBox[{"negSet", "[", 
                  RowBox[{"[", 
                   RowBox[{"m", ",", "2"}], "]"}], "]"}]}], "]"}], "]"}], 
              "<>", "\"\<,[\>\"", "<>", 
              RowBox[{"StringRiffle", "[", 
               RowBox[{
                RowBox[{"negSet", "[", 
                 RowBox[{"[", 
                  RowBox[{"m", ",", "1"}], "]"}], "]"}], ",", "\"\<,\>\""}], 
               "]"}], "<>", "\"\<]],\>\""}]}], ";"}]}], "\[IndentingNewLine]",
           "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"negString", " ", "=", " ", 
          RowBox[{
           RowBox[{"StringDrop", "[", 
            RowBox[{"negString", ",", 
             RowBox[{"-", "1"}]}], "]"}], "<>", "\"\<]\>\""}]}], ";"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"negString", "=", "\"\<\>\""}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{"posString", ",", "negString"}], "}"}], "]"}], ";"}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"$Assumptions", " ", "=", " ", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x1", ",", "x2", ",", "x3", ",", "x4", ",", "x5", ",", "x6"}], 
     "}"}], " ", "\[Element]", " ", "Reals"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"N", "[", 
   RowBox[{
    RowBox[{"Total", "[", "Int22", "]"}], ",", "16"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"poly", ",", "vars", ",", "ineq"}], "}"}], "=", 
   RowBox[{"integrationPartsDoubleDelta", "[", 
    RowBox[{"1", ",", "5"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"poly", ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"poly", " ", "=", " ", 
   RowBox[{"-", 
    SuperscriptBox["x2", "2"]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"posString", ",", "negString"}], "}"}], "=", 
  RowBox[{"polyStringWrite", "[", 
   RowBox[{"poly", ",", "vars"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"StringLength", "[", "negString", "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"StringLength", "[", "posString", "]"}], 
  "]"}], "\[IndentingNewLine]", "negString", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"polySet", " ", "=", " ", 
   RowBox[{"polyExp", "[", 
    RowBox[{"poly", ",", "vars"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"polySet", "[", 
   RowBox[{"[", 
    RowBox[{";;", ",", "2"}], "]"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"negSet", " ", "=", " ", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Position", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"polySet", "[", 
         RowBox[{"[", 
          RowBox[{";;", ",", "2"}], "]"}], "]"}], "//", "N"}], ")"}], ",", 
      RowBox[{"_", "?", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"#", "<", "0"}], "&"}], ")"}]}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"posSet", " ", "=", " ", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Position", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"polySet", "[", 
         RowBox[{"[", 
          RowBox[{";;", ",", "2"}], "]"}], "]"}], "//", "N"}], ")"}], ",", 
      RowBox[{"_", "?", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"#", ">=", "0"}], "&"}], ")"}]}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"polySet", "=", 
   RowBox[{"polySet", "[", 
    RowBox[{"[", "negSet", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"polyString", " ", "=", " ", "\"\<[\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"m", "=", "1"}], ",", 
    RowBox[{"m", "<=", 
     RowBox[{"Length", "[", "polySet", "]"}]}], ",", 
    RowBox[{"m", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"polyString", " ", "=", " ", 
      RowBox[{"polyString", "<>", "\"\<[\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"Round", "[", 
         RowBox[{
          SuperscriptBox["10", "10"], "*", 
          RowBox[{"polySet", "[", 
           RowBox[{"[", 
            RowBox[{"m", ",", "2"}], "]"}], "]"}]}], "]"}], "]"}], "<>", 
       "\"\<,[\>\"", "<>", 
       RowBox[{"StringRiffle", "[", 
        RowBox[{
         RowBox[{"polySet", "[", 
          RowBox[{"[", 
           RowBox[{"m", ",", "1"}], "]"}], "]"}], ",", "\"\<,\>\""}], "]"}], 
       "<>", "\"\<]],\>\""}]}], ";"}]}], "\[IndentingNewLine]", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"polyString", " ", "=", " ", 
   RowBox[{
    RowBox[{"StringDrop", "[", 
     RowBox[{"polyString", ",", 
      RowBox[{"-", "1"}]}], "]"}], "<>", "\"\<]\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Length", "[", "negString", "]"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.838208707142795*^9, 3.8382087241178293`*^9}, {
   3.838212960934873*^9, 3.83821305705116*^9}, {3.838213089453896*^9, 
   3.838213098163622*^9}, {3.838215551092388*^9, 3.838215592659944*^9}, {
   3.838215814838696*^9, 3.83821581590874*^9}, {3.838215942127603*^9, 
   3.838216022039906*^9}, {3.838216092118404*^9, 3.838216128854268*^9}, {
   3.838216198086979*^9, 3.8382163417991123`*^9}, {3.838216440667445*^9, 
   3.8382164637974863`*^9}, {3.838216658267974*^9, 3.83821665932545*^9}, {
   3.838216712002369*^9, 3.83821677181185*^9}, {3.8382168305559683`*^9, 
   3.83821686827852*^9}, {3.838216962952116*^9, 3.838217030128339*^9}, {
   3.8382170802364693`*^9, 3.838217157283669*^9}, {3.8382172508353243`*^9, 
   3.8382173010738907`*^9}, {3.838217345726655*^9, 3.8382174131209183`*^9}, {
   3.8382174651497726`*^9, 3.838217495670088*^9}, {3.838217658890051*^9, 
   3.8382178628440647`*^9}, {3.8382179903053617`*^9, 3.838218137896433*^9}, {
   3.838218171531969*^9, 3.838218292257675*^9}, {3.838218368369285*^9, 
   3.838218428627151*^9}, {3.838218484356442*^9, 3.8382185816290283`*^9}, {
   3.8382187699034348`*^9, 3.838218949038636*^9}, {3.838218996760756*^9, 
   3.838219131467238*^9}, {3.838219191783599*^9, 3.838219333672863*^9}, {
   3.8382193691809587`*^9, 3.838219424812756*^9}, {3.8382194579073563`*^9, 
   3.838219507913528*^9}, {3.838219671662875*^9, 3.838219757066593*^9}, {
   3.838219952342496*^9, 3.8382199979736*^9}, {3.838220047225974*^9, 
   3.838220080919405*^9}, {3.838220124856185*^9, 3.838220129252441*^9}, {
   3.8382201910159883`*^9, 3.838220233909957*^9}, {3.8382202643880177`*^9, 
   3.8382203640548153`*^9}, 3.838220439123061*^9, {3.838220502101635*^9, 
   3.83822051031255*^9}, 3.838220570329043*^9, {3.838220634904224*^9, 
   3.8382206637423887`*^9}, {3.838220733270993*^9, 3.838220744751326*^9}, {
   3.838220811848957*^9, 3.838220826370261*^9}, {3.838220973902396*^9, 
   3.838221092904491*^9}, {3.8382214229977283`*^9, 3.838221487565174*^9}, {
   3.838221602843081*^9, 3.838221616532599*^9}, {3.838221922677857*^9, 
   3.838221930485497*^9}, {3.838241718545351*^9, 3.838241759677459*^9}, {
   3.838241902838962*^9, 3.8382419091085663`*^9}, {3.838241959398758*^9, 
   3.838241976425828*^9}, {3.838242010145236*^9, 3.83824201724527*^9}, {
   3.838242450003749*^9, 3.838242454986771*^9}, {3.838242509874359*^9, 
   3.8382425980009336`*^9}, {3.838242654972405*^9, 3.838242695222275*^9}, {
   3.838242745420355*^9, 3.838242829172772*^9}, {3.838242861256675*^9, 
   3.838243107135817*^9}, {3.838243184546541*^9, 3.838243194739934*^9}, {
   3.838243343801154*^9, 3.838243345963229*^9}, {3.838247494744164*^9, 
   3.8382476138408403`*^9}, {3.838247701729439*^9, 3.838247811025239*^9}, {
   3.838247849213593*^9, 3.838247864487464*^9}, {3.838247940390503*^9, 
   3.8382479581417847`*^9}, {3.838248005801303*^9, 3.838248017970314*^9}, {
   3.838248797934575*^9, 3.8382488524892893`*^9}, {3.8382488966092463`*^9, 
   3.838248966395138*^9}, {3.838249082730853*^9, 3.838249176967228*^9}, {
   3.838249211023978*^9, 3.838249277415105*^9}, {3.838249328948522*^9, 
   3.838249338642352*^9}, {3.838249388210874*^9, 3.838249409700634*^9}, {
   3.838249485385714*^9, 3.838249558697872*^9}, {3.838249656857737*^9, 
   3.8382496887571163`*^9}, {3.8382497687017193`*^9, 3.838249786613268*^9}, {
   3.838249820475559*^9, 3.83825002064536*^9}, {3.8382501054827213`*^9, 
   3.838250137568719*^9}, {3.838250286549464*^9, 3.8382503114841127`*^9}, 
   3.847983600420515*^9},ExpressionUUID->"c131fc4e-ec6a-4e67-8bd0-\
a04098e56723"]
},
WindowSize->{808, 599},
WindowMargins->{{97, Automatic}, {Automatic, 0}},
FrontEndVersion->"12.3 for Mac OS X x86 (64-bit) (June 19, 2021)",
StyleDefinitions->"Default.nb",
ExpressionUUID->"3d6f9874-8906-4c7a-bff1-70358b54ac77"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[558, 20, 2096, 43, 288, "Text",ExpressionUUID->"98bf8393-6c63-45f0-93be-f29ffd94c896"],
Cell[2657, 65, 8082, 191, 381, "Input",ExpressionUUID->"ed4d4abb-87c1-4f3f-9d2d-574b4feef35f"],
Cell[10742, 258, 364, 8, 58, "Text",ExpressionUUID->"fd2a4a22-5b5d-4914-b9c0-d983c0f041ef"],
Cell[11109, 268, 4911, 95, 115, "Input",ExpressionUUID->"afcdac5a-7e95-421c-9071-880540ea7e2e"],
Cell[16023, 365, 3484, 54, 36, "Text",ExpressionUUID->"2f59cdff-a3bc-40ac-a5d6-992e9ad57b9f"],
Cell[CellGroupData[{
Cell[19532, 423, 3503, 52, 30, "Input",ExpressionUUID->"c757d0e1-012f-413b-951c-083767d8eef5"],
Cell[23038, 477, 942, 16, 35, "Output",ExpressionUUID->"9c2e26af-78dd-4c81-b610-dc67e2824a78"]
}, Open  ]],
Cell[23995, 496, 3546, 53, 35, "Text",ExpressionUUID->"75b648bd-ac89-4adb-bd1e-75d1d957badd"],
Cell[CellGroupData[{
Cell[27566, 553, 846, 18, 52, "Input",ExpressionUUID->"70bb00f4-1d36-4691-a0de-c96cb8cc4bc6"],
Cell[28415, 573, 1653, 24, 34, "Output",ExpressionUUID->"e1073f11-9568-4e1e-9ee0-0eb67703a781"]
}, Open  ]],
Cell[30083, 600, 1433, 26, 151, "Text",ExpressionUUID->"523d62d6-2e5a-4278-a2d4-75e1524d1756"],
Cell[CellGroupData[{
Cell[31541, 630, 3586, 93, 402, "Input",ExpressionUUID->"21f53e28-e625-4ab3-8fd2-a2f763bfef1a"],
Cell[35130, 725, 1190, 19, 54, "Output",ExpressionUUID->"2318c518-f08c-4865-b74b-af090999c8ae"]
}, Open  ]],
Cell[CellGroupData[{
Cell[36357, 749, 682, 17, 70, "Input",ExpressionUUID->"9434cd58-094d-43be-993d-b036c028967d"],
Cell[37042, 768, 1524, 24, 51, "Output",ExpressionUUID->"1907f97b-cd48-4684-9e5e-e9d70a5f49d3"],
Cell[38569, 794, 1491, 22, 34, "Output",ExpressionUUID->"85d4e7aa-bb6b-4778-8a38-b50f822b5d41"]
}, Open  ]],
Cell[40075, 819, 442, 9, 81, "Text",ExpressionUUID->"9d4b628d-8664-447f-8950-39051f6e1924"],
Cell[40520, 830, 1756, 45, 146, "Input",ExpressionUUID->"a51883d4-daf5-47a4-8572-0c332014f1d4"],
Cell[CellGroupData[{
Cell[42301, 879, 49366, 1109, 3920, "Input",ExpressionUUID->"ee0adbbb-781a-4989-b46a-86f9b1aad08b"],
Cell[91670, 1990, 3744, 55, 54, "Output",ExpressionUUID->"e20888d3-8c5d-4904-9b16-275938c65530"],
Cell[95417, 2047, 3961, 59, 56, "Output",ExpressionUUID->"a526958d-68fc-4f5f-ba86-9fad417243f7"],
Cell[99381, 2108, 3744, 55, 54, "Output",ExpressionUUID->"896b6f37-6ca5-43af-9674-c9d6e1747b8c"],
Cell[103128, 2165, 3701, 53, 34, "Output",ExpressionUUID->"e3cab884-99c2-4d1c-ab30-1a1205eefe82"],
Cell[106832, 2220, 3702, 53, 34, "Output",ExpressionUUID->"2ccfce55-5efe-48e9-a653-0ab8545055de"],
Cell[110537, 2275, 4096, 69, 38, "Output",ExpressionUUID->"06c0abba-de8b-4361-b980-ff0223dc329a"],
Cell[114636, 2346, 6418, 160, 142, "Output",ExpressionUUID->"8c63861f-1c05-4a86-9c12-1410ca1ce0e4"],
Cell[121057, 2508, 3987, 66, 38, "Output",ExpressionUUID->"968ef69c-c648-4961-af70-68b2e38c8227"],
Cell[125047, 2576, 4406, 82, 62, "Output",ExpressionUUID->"324c58b5-587e-44e0-a2e3-482f53b3a63a"]
}, Open  ]],
Cell[CellGroupData[{
Cell[129490, 2663, 12287, 230, 322, "Input",ExpressionUUID->"6c19dc58-4b71-4df9-837b-97f66171c8c6"],
Cell[141780, 2895, 2041, 34, 77, "Output",ExpressionUUID->"9c670320-d0c1-4415-8cdb-3e3b8ffd1965"],
Cell[143824, 2931, 4166, 117, 274, "Output",ExpressionUUID->"39085576-dca5-485f-93bf-34442b9f2c26"],
Cell[147993, 3050, 2222, 58, 54, "Output",ExpressionUUID->"67924d66-79b7-43ad-9484-fb357b9fd95f"],
Cell[150218, 3110, 1392, 23, 54, "Output",ExpressionUUID->"f282da51-0bba-4034-97ba-d593a7bb67f6"],
Cell[151613, 3135, 1376, 20, 34, "Output",ExpressionUUID->"2f0c85c1-e835-4587-b464-f03eabcc6c6e"],
Cell[152992, 3157, 1359, 21, 34, "Output",ExpressionUUID->"43f44c3a-1fb8-44a3-9e85-284de1277718"],
Cell[154354, 3180, 1353, 20, 34, "Output",ExpressionUUID->"bee2b919-af19-4b21-80a8-84dbaeca1179"]
}, Open  ]],
Cell[155722, 3203, 129, 3, 30, "Input",ExpressionUUID->"f650c2d8-af76-412b-99b5-fa46e1a33a34"],
Cell[155854, 3208, 7133, 97, 30, "Input",ExpressionUUID->"19b95e8d-8177-45fc-8e99-2d06d6b7b9f6"],
Cell[162990, 3307, 1600, 30, 151, "Text",ExpressionUUID->"1ab9e84a-9032-4551-9d67-f55dbd635a4d"],
Cell[164593, 3339, 10338, 295, 661, "Input",ExpressionUUID->"ab2ae627-5ff9-403d-a120-e4c629e0b531"],
Cell[174934, 3636, 491, 12, 36, "Text",ExpressionUUID->"bfde2e94-4ac0-475b-969f-490a5ec8dfcd"],
Cell[CellGroupData[{
Cell[175450, 3652, 1009, 24, 73, "Input",ExpressionUUID->"92e5f1d9-d9df-4f8d-a4e4-2d24ab6be0a5"],
Cell[176462, 3678, 1469, 23, 51, "Output",ExpressionUUID->"b146b3be-3cef-47cc-b1c9-bda035f5fde5"],
Cell[177934, 3703, 1500, 24, 51, "Output",ExpressionUUID->"9787f1bc-f27f-49ed-ae79-bcbc9016b7fc"],
Cell[179437, 3729, 1465, 22, 34, "Output",ExpressionUUID->"14d1473f-fd9c-4e82-90ac-329b8e18c732"]
}, Open  ]],
Cell[180917, 3754, 835, 17, 81, "Text",ExpressionUUID->"3221b9fc-1c23-4ccb-bbee-e3e348cc46a4"],
Cell[CellGroupData[{
Cell[181777, 3775, 1772, 37, 73, "Input",ExpressionUUID->"c454802c-2125-455a-a8e6-1fa5d7f97804"],
Cell[183552, 3814, 502, 9, 51, "Output",ExpressionUUID->"23e80492-6498-4971-a2d2-f1d2c6ffc4c3"],
Cell[184057, 3825, 524, 10, 51, "Output",ExpressionUUID->"e9d1719c-502f-4221-af4d-c8af04c58a8b"],
Cell[184584, 3837, 540, 9, 34, "Output",ExpressionUUID->"bad878ed-7f94-47e9-a1ac-b39eb6619fc2"]
}, Open  ]],
Cell[185139, 3849, 638, 11, 35, "Text",ExpressionUUID->"27e2231e-2fb2-4349-aca2-c81e7c2b6875"],
Cell[185780, 3862, 11588, 306, 1133, "Input",ExpressionUUID->"c72d7132-ae94-49b9-8c00-2418e0b5459f"],
Cell[197371, 4170, 12735, 349, 1093, "Input",ExpressionUUID->"733031dd-97c3-455b-96c5-cdb1866ae13a"],
Cell[210109, 4521, 26055, 635, 2700, "Input",ExpressionUUID->"f715c12c-f408-4aec-afba-fb64eb8ff085"],
Cell[CellGroupData[{
Cell[236189, 5160, 3357, 91, 238, "Input",ExpressionUUID->"dd01affd-8df1-425a-b5d5-cf195dcc272f"],
Cell[239549, 5253, 1539, 52, 90, "Output",ExpressionUUID->"c993743d-3041-4090-8d90-7970b2b79265"],
Cell[241091, 5307, 1089, 28, 90, "Output",ExpressionUUID->"5093cfd7-0102-4405-91dc-ef4e44db7b25"],
Cell[242183, 5337, 1403, 44, 90, "Output",ExpressionUUID->"d34792a8-53e9-473c-b768-25c50da32cd1"],
Cell[243589, 5383, 1187, 25, 77, "Output",ExpressionUUID->"f826fb3b-2e6a-4c90-b4c0-2d6cfedaeb21"],
Cell[244779, 5410, 252, 5, 51, "Output",ExpressionUUID->"7c20fee8-5f7a-4cf9-94ba-0b99070eecb4"],
Cell[245034, 5417, 265, 4, 34, "Output",ExpressionUUID->"4a2ad2e4-c4a8-4023-8ecc-24d1d5ac779c"],
Cell[245302, 5423, 256, 5, 34, "Output",ExpressionUUID->"8b795e7e-d7ed-4770-9a94-6b33a9450d58"],
Cell[245561, 5430, 239, 4, 34, "Output",ExpressionUUID->"8ceaec49-0ab3-4127-902a-ad14f00237e3"]
}, Open  ]],
Cell[245815, 5437, 12779, 294, 1090, "Input",ExpressionUUID->"c131fc4e-ec6a-4e67-8bd0-a04098e56723"]
}
]
*)

